/*H+ Slider.h
*
*    SCCS ID: @(#)Slider.h 1.1 91/08/02 SMI 
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Defines the Slider class.
*
*    AUTHOR: Joe and Warzecha and Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                   
*    --------- ---------- -----------------------
*    JDZ       05/10/91   Preparation for code review
*
*H-*/

/*C+ Slider
*
*    DESCRIPTION:
*
*    A Slider object displays a value in graphical form and allows
*    a user to change a value by moving the slider bar to the value 
*    required.  A function can be written that will be triggered any 
*    time the user inputs a new value.  
*
*    INHERITED CLASSES:
*
*    Gauge - Provides basic capabilities to display a value in a 
*    graphical form, but does not allow the user to input values.
*
*    ORDINARY DATA MEMBERS:
*
*    Boolean	 displayValue - Specifies whether or not the slider value
*    should be displayed.
*
*    Boolean	 displayEndBoxes - Specifies whether or not end boxes should
*    be displayed.
*
*    PUBLIC MEMBER FUNCTIONS:
*
*            int      getValue (int &) - Returns the value of the Slider.
*
*            void     setSliderWidth (int) - Sets the width of the Slider.
*            Default: XView Default.
*
*            void     setDisplayValue (Boolean) - Sets whether or not the
*            value of the Slider should be displayed numerically.  
*            Default: Value displays (in characters).
*
*            void     setDisplayEndBoxes (Boolean) - Sets whether or not 
*            the end boxes should be displayed.  Default: Not displayed.
*
*    PROTECTED MEMBER FUNCTIONS:
*
*    virtual void     createObject (UIObject *) - Create XView object.
*
*    static  void     notifyProc (Panel_item, int, Event *) - Internal 
*            wrapper around the XView event proc. for displaying objects.
*
*    PRIVATE MEMBER FUNCTIONS:
*
*            void     setSliderDefaults () - Initializes the data members.
*
*C-*/
#ifndef _SLIDER_H
#define _SLIDER_H

// Includes
#include "Gauge.h"

// Class Definition
class Slider : public Gauge {
public:
    		 	 Slider ();
    		 	 Slider (char *);
		 	 Slider (Image &);
			~Slider ();

    int	 	 	 getValue (int &);

    void	         setSliderWidth (int newWidth)
                         { sliderWidth = newWidth;
			   if (owObject)
			     xv_set (owObject,
				     PANEL_SLIDER_WIDTH, sliderWidth,
				     NULL); }

    void	 	 setDisplayValue (Boolean display)
			 { displayValue = display;
			   if (owObject) xv_set (owObject,
			                         PANEL_SHOW_VALUE,
						 displayValue,
						 NULL); }

    void	 	 setDisplayEndBoxes (Boolean display)
			 { displayEndBoxes = display;
			   if (owObject) xv_set (owObject,
			                         PANEL_SLIDER_END_BOXES,
						 displayEndBoxes,
						 NULL); }

protected:
    Boolean	 	 displayValue;
    Boolean	 	 displayEndBoxes;
    
    virtual void 	 createObject (UIObject *);
    static  void	 notifyProc (Panel_item, int, Event *);

private:
    void		 setSliderDefaults ();
};

#endif /* _SLIDER_H */
