/*H+ UIDisplay.h
*
*    SCCS ID: @(#)UIDisplay.h 1.1 91/08/02 SMI 
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Defines the UIDisplay class.
*
*    AUTHOR: Joe Warzecha and Mark soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                   
*    --------- ---------- -----------------------
*    JDZ       05/01/91   Preparation for code review
*
*H-*/

/*C+ UIDisplay
*
*    DESCRIPTION:
*
*    The UIDisplay class is the base class for all of the display region
*    classes.  Classes derived off of UIDisplay are used for specifying	
*    a region to display text or grpahics on a window.
*
*    INHERITED CLASSES:
*
*    UIObject - The base class of all UI classes in the UIC.
*
*    ORDINARY DATA MEMBERS:
*
*    Boolean	 extendWidth  - TRUE if width should be extended.
*
*    Boolean	 extendHeight - TRUE if height should be extended.
*
*    Boolean	 border       - TRUE if borders should be displayed.
*
*    PUBLIC MEMBER FUNCTIONS:
*
*    virtual  char   *getDragFileName () - Returns the name of the file 
*             that was just dragged into this TextDisplay object.
*
*             void    setDisplayBorders (Boolean) - Sets whether
*             or not the borders should be displayed around  this
*             display.  Default: Borders not displayed.
*
*             void    setExtendToEdge (Dimension) - Depending on 
*             the value of the Dimension argument (HEIGHT or WIDTH), 
*             determines if that Dimension should be extended to 
*             the border of the window in which it has been placed. 
*             Default: Does NOT extend its height or width to the 
*             window in which it is placed.  Default: Don't extend to edge.
*
*    PROTECTED MEMBER FUNCTIONS:
*
*    static Notify_value eventProc (Xv_window,
*                                   Event *,
*                                   Notify_arg,
*                                   Notify_event_type) - Internal wrapper
*           around the XView event proc for objects of classes derived
*           from UIDisplay.
*
*C-*/
#ifndef _UIDISPLAY_H
#define _UIDISPLAY_H

// Includes
#include "UIObject.h"

#include <sys/param.h>
#include <xview/rect.h>
#include <xview/win_input.h>
#include <xview/window.h>

// Class Definition
class UIDisplay : public UIObject {
public:
    	  	  UIDisplay ();
		  ~UIDisplay ();

    virtual char *getDragFileName ();

    void	  setDisplayBorders (Boolean flag)
		  { border = flag;
		    if (owObject) xv_set (owObject,
					  OPENWIN_SHOW_BORDERS, flag,
		    			  NULL); }

    void	  setExtendToEdge (Dimension);

protected:
    Boolean	 extendWidth;
    Boolean	 extendHeight;
    Boolean	 border;

    static Notify_value eventProc (Xv_window,
				   Event *,
				   Notify_arg,
				   Notify_event_type);
};

#endif /* _UIDISPLAY_H */
