/*H+ panel_list_structs.h
*
*    SCCS ID: @(#)panel_list_structs.h 1.1 91/08/02 SMI 
*
*    (c) Copyright 1989 Sun Microsystems, Inc. Sun design patents
*    pending in the U.S. and foreign countries.
*
*    NOTICE TO USER: The source code, including the glyphs or icons 
*    forming a par of the OPEN LOOK TM Graphic User Interface, on this 
*    tape and in these files is copyrighted under U.S. and international
*    laws. Sun Microsystems, Inc. of Mountain View, California owns
*    the copyright and has design patents pending on many of the icons. 
*    AT&T is the owner of the OPEN LOOK trademark associated with the
*    materials on this tape. Users and possessors of this source code 
*    are hereby granted a nonexclusive, royalty-free copyright and 
*    design patent license to use this code in individual and 
*    commercial software. A royalty-free, nonexclusive trademark
*    license to refer to the code and output as "OPEN LOOK" compatible 
*    is available from AT&T if, and only if, the appearance of the 
*    icons or glyphs is not changed in any manner except as absolutely
*    necessary to accommodate the standard resolution of the screen or
*    other output device, the code and output is not changed except as 
*    authorized herein, and the code and output is validated by AT&T. 
*    Bigelow & Holmes is the owner of the Lucida (R) trademark for the
*    fonts and bit-mapped images associated with the materials on this 
*    tape. Users are granted a royalty-free, nonexclusive license to use
*    the trademark only to identify the fonts and bit-mapped images if, 
*    and only if, the fonts and bit-mapped images are not modified in any
*    way by the user. 
*
*    Any use of this source code must include, in the user documentation 
*    and internal comments to the code, notices to the end user as  
*    follows:
*
*    (c) Copyright 1989 Sun Microsystems, Inc. Sun design patents
*    pending in the U.S. and foreign countries. OPEN LOOK is a 
*    trademark of AT&T. Used by written permission of the owners.
*
*    (c) Copyright Bigelow & Holmes 1986, 1985. Lucida is a registered 
*    trademark of Bigelow & Holmes. Permission to use the Lucida 
*    trademark is hereby granted only in association with the images 
*    and fonts described in this file.
*
*    SUN MICROSYSTEMS, INC., AT&T, AND BIGELOW & HOLMES 
*    MAKE NO REPRESENTATIONS ABOUT THE SUITABILITY OF
*    THIS SOURCE CODE FOR ANY PURPOSE. IT IS PROVIDED "AS IS" 
*    WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND. 
*    SUN  MICROSYSTEMS, INC., AT&T AND BIGELOW  & HOLMES, 
*    SEVERALLY AND INDIVIDUALLY, DISCLAIM ALL WARRANTIES 
*    WITH REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED
*    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
*    PARTICULAR PURPOSE. IN NO EVENT SHALL SUN MICROSYSTEMS,
*    INC., AT&T OR BIGELOW & HOLMES BE LIABLE FOR ANY
*    SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
*    OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA 	
*    OR PROFITS, WHETHER IN AN ACTION OF  CONTRACT, NEGLIGENCE
*    OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
*    WITH THE USE OR PERFORMANCE OF THIS SOURCE CODE.
*
*    PURPOSE:
*
*    Provides access to PANEL_LIST internal structure information for
*    the implementation of the ScrollChoice class.  This is a copy of
*    the p_lst_impl.h header file in the private includes part of the
*    XView toolkit source.
*
*    AUTHOR: XView Engineering
*
*    REVISION HISTORY:
*
*    Who       When       Why                   
*    --------- ---------- -----------------------
*    MGS       05/10/91   Preparation for code review.
*
*H-*/
#ifndef _PANEL_LIST_STRUCTS_H 
#define _PANEL_LIST_STRUCTS_H

// Includes
#include <xview/panel.h>
#include <xview/font.h>
#include <xview/openmenu.h>
#include <xview/scrollbar.h>

// Macro Definitions
#define PRIMARY_CHOICE	0
#define SHELF_CHOICE	1
#define PANEL_LIST_DEFAULT_ROW	5

// Type Definitions
typedef struct panel_list_info_struct		panel_list_info_t;
typedef struct row_info_struct			row_info_t;

typedef enum {
    OP_NONE,
    OP_CHANGE,
    OP_INSERT,
    OP_DELETE
} Edit_operation;

// Structure Definitions
struct row_info_struct {
	Xv_opaque	client_data;	/* Client data with each row */
	int		display_str_len; /* length of displayed string */
	Xv_Font		font;		/* NULL => use WIN_FONT */
	Pixrect		*glyph;
	int		row;            /* Row number */
	u_char		*string;
	int		string_y;

	unsigned edit_selected : 1;	/* selected in edit mode */
	unsigned free_string : 1;	/* free malloc'ed string */
	unsigned selected : 1;          /* selected in read mode (= current) */
	unsigned show : 1;		/* row is to be painted */

	/* Chaining */
	struct row_info_struct *next;
	struct row_info_struct *prev;
};
 
struct panel_list_info_struct {
	Panel_item	public_self;
        Panel		parent_panel;   /* Panel we're in */
	Rect		list_box;	/* Box enclosing list of rows */
	Scrollbar	list_sb;	/* Scrollbar for list_box */
	Menu		edit_menu;	/* Row panel edit mode menu */
	Edit_operation	edit_op;	/* current edit operation, if any */
	Pixfont		*font;		/* font of parent panel */
	XFontStruct     *font_struct;	/* font structure of parent panel */
	Menu		read_menu;	/* Row panel read mode menu */
	Panel_item	text_item;	/* Text item used during editing */
	row_info_t	*text_item_row;	/* Text item is editing this row */
	int		text_item_view_start; /* first row in view when text
					       * item was made visible. */
	void		(*event_proc)();
					/* Client panel event proc for list */

	/* control */
	unsigned choose_none: 1;	/* no current row is okay */
	unsigned choose_one : 1;    /* TRUE: exclusive, FALSE: non-exclusive */
	unsigned edit_mode : 1;		/* TRUE: read-write, FALSE: read */
	unsigned has_kbd_focus : 1;	/* panel list has keyboard focus */
	unsigned initialized : 1;	/* set of XV_END_CREATE has occurred */
	unsigned left_hand_sb: 1; /* list_box Scrollbar is on left hand side */
	unsigned read_only : 1;		/* TRUE: read, FALSE: read/write */
	unsigned setting_current_row:1;

	/* sizes and positions */
	unsigned short	nlevels;	/* Number of levels */
	unsigned short	height;		/* Height of the scroll list */
	unsigned short	width;		/* Width of the scroll list */
	unsigned short	nrows;		/* Number of rows */
	unsigned short	rows_displayed;	/* Number of rows displayed */
	unsigned short	row_height;  /* Height of each row. 0 => font height */
	unsigned short  string_x;	/* left margin of each row's string */

	/* Current data */
	row_info_t	*rows;
        char	 	*selection[2];  /* Seln string cache */
        int	 	have_seln[2];   /* Seln ownership info cache */
	char		*seln_client;   /* Selection client */
	row_info_t	*current_row;	/* last row selected */
	row_info_t	*last_edit_row; /* last row selected for editing */
	row_info_t	*char_match_row; /* last row character-matched row */
};

#endif _PANEL_LIST_STRUCTS_H /* _PANEL_LIST_STRUCTS_H */
