/*H+ guic.h
*
*    Copyright (c) 1990,1991 by Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file terms of the license.
*
*    SCCS ID: @(#)guic.h 1.1 91/08/02 SMI 
*
*    PURPOSE:
*
*    Declares external functions for guic.c.
*
*    REVISION HISTORY:
*
*    Who       When       Why                   
*    --------- ---------- -----------------------
*    MGS       05/10/91   Preparation for code review.
*
*H-*/

#ifndef _GUIC_H
#define _GUIC_H

/* Includes */

/* This is included to obtain the XV_VERSION_NUMBER macro if this is
   OpenWindows V3 */
#include <xview/xview.h>

/* Macro Definitions */

/* If XV_VERSION_NUMBER is defined, then this is OpenWindows V3 */
#ifdef XV_VERSION_NUMBER
#ifndef OWNV3
#define OWNV3
#endif
#endif

/* Type Definitions */

/*
 * Structure containing GIL object information.
 */
typedef struct GUIC_OBJ {
	struct GUIC_OBJ *next;
	char	       *background_color;
	G_BUTTON_TYPES  button_type;
	char          **choices;
	G_LABEL_TYPES  *choice_label_type;
	char          **choice_color;
	int             columns;
	int		color_needed;
	int		constant_width;
	char	       *done_handler;
	G_DRAWING_MODELS drawing_model;
	G_EVENT_TYPES  *events;
	char           *event_handler;
	char	       *foreground_color;
	int             height;
	char	       *help;
	int             hscrollbar;
	char           *icon;
	char           *icon_mask;
	char           *label;
	int		label_bold;
	G_LABEL_TYPES   label_type;
	G_LAYOUT_TYPES  layout_type;
	int		mapped;
	int             max_value;
	char	       *menu_handler;
	char          **menu_item_color;
	int	       *menu_item_default;
	char	      **menu_item_handler;
	char	      **menu_item_label;
	G_LABEL_TYPES  *menu_item_label_type;
	char	      **menu_item_menu;
	char	       *menu_name;
	char	       *menu_title;
	G_MENU_TYPES	menu_type;
	int             min_value;
	int		multiple_selections;
	char           *name;
	char           *notify_handler;
	G_LAYOUT_TYPES  orientation;
	char           *original_name;
	struct GUIC_OBJ *owner;
	int		pinnable;
	int		pinned;
	int		read_only;
	char	       *repaint_proc;
	int		resizable;
	int             rows;
	int             scrollable_height;
	int             scrollable_width;
	G_SETTING_TYPES	setting_type;
	int		selection_required;
	int		show_border;
	int		show_endboxes;
	int		show_footer;
	int             show_range;
	int             show_value;
	int		slider_width;
	int		stored_length;
	G_TEXT_TYPES	text_type;
	int             ticks;
	G_TYPES         type;
	int		value_length;
	int             value_x;
	int             value_y;
	int             vscrollbar;
	int             width;
	int             x;
	int             y;
}               GUIC_OBJ;

/*D+ char *guic_application ()
*
* Purpose: Returns the name of the application.
*
* char *name (R) The name of the application.
*D-*/
extern char *guic_application();

/*D+ char *guic_application_file ()
*
* Purpose: Returns the name of the application file.
*
* char *name (R) The name of the application file.
*D-*/
extern char *guic_application_file();

/*D+ char *guic_boolean_string (int i)
*
* Purpose: Returns the a string containing a boolean value "TRUE" or "FALSE".
*
* int   i    (I) Input value that determines the out string.
* char *name (R) The boolean value.
*D-*/
extern char *guic_boolean_string();

/*D+ char *guic_calloc (int n, int s)
*
* Purpose: Calloc with error checking.
*
* int   n (I) The number of memory chunks to allocate.
* int   s (I) The size of the memory chunks.
* char *m (R) Pointer to allocated memory.
*D-*/
extern char *guic_calloc();

/*D+ void guic_error (char *s)
*
* Purpose: Outputs an error message.
*
* char *s (I) The message to output.
*D-*/
extern void guic_error();

/*D+ char *guic_gil_file ()
*
* Purpose: Returns the name of the application GIL file.
*
* char *name (R) The name of the application GIL file.
*D-*/
extern char *guic_gil_file();

/*D+ int guic_identifier_ok (char *name)
*
* Purpose: Determines if the specified name is a legal C++ identifier.
*
* char *name   (I) The name of the identifier.
* int   result (R) TRUE if legal, FALSE if illegal.
*D-*/
extern int guic_identifier_ok();

/*D+ int guic_iscontrol (GUIC_OBJ *object)
*
* Purpose: Determines if the specified object is a control object.
*
* GUIC_OBJ *object (I) The object to check.
* int       result (R) TRUE if a control object, FALSE if not.
*D-*/
extern int guic_iscontrol();

/*D+ int guic_ispane (GUIC_OBJ *object)
*
* Purpose: Determines if the specified object is a pane object.
*
* GUIC_OBJ *object (I) The object to check.
* int       result (R) TRUE if a pane object, FALSE if not.
*D-*/
extern int guic_ispane();

/*D+ int guic_iswindow (GUIC_OBJ *object)
*
* Purpose: Determines if the specified object is a window object.
*
* GUIC_OBJ *object (I) The object to check.
* int       result (R) TRUE if a window object, FALSE if not.
*D-*/
extern int guic_iswindow();

/*D+ char *guic_malloc (int s)
*
* Purpose: Malloc with error checking.
*
* int   s (I) The size of the memory to allocate.
* char *m (R) Pointer to allocated memory.
*D-*/
extern char *guic_malloc();

/*D+ char *guic_program ()
*
* Purpose: Returns the name of the program being executed (usually "guic").
*
* char *name (R) The name of the program.
*D-*/
extern char *guic_program();

/*D+ char *guic_realloc (char *b, int s)
*
* Purpose: Realloc with error checking.
*
* char *b (I) Pointer to the memory to reallocate.
* int   s (I) The size of the memory to allocate.
* char *m (R) Pointer to allocated memory.
*D-*/
extern char *guic_realloc();

/*D+ int guic_stub_written (char *name)
*
* Purpose: Determines if the specified stub has been written.
*
* char *name   (I) The name of the stub.
* int   result (R) TRUE if a the stub was written, FALSE if not.
*D-*/
extern int guic_stub_written();

/*D+ int guic_silent ()
*
* Purpose: Determines if guic is in the silent mode.
*
* int result (R) TRUE if in the silent mode, FALSE if not.
*D-*/
extern int guic_silent();

/*D+ int guic_verbose ()
*
* Purpose: Determines if guic is in the verbose mode.
*
* int result (R) TRUE if in the verbose mode, FALSE if not.
*D-*/
extern int guic_verbose();

/*D+ int guic_medium_names ()
*
* Purpose: Determines if medium length names are to be generated.
*
* int result (R) TRUE if yes, FALSE if no.
*D-*/
extern int guic_medium_names ();

/*D+ int guic_long_names ()
*
* Purpose: Determines if long names are to be generated.
*
* int result (R) TRUE if yes, FALSE if no.
*D-*/
extern int guic_long_names ();

#endif /* _GUIC_H */
