/*H+ HotRegion.cc
*
*    SCCS ID: @(#)HotRegion.cc 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Defines the member function of the HotRegion class.
*
*    AUTHOR: Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review. 
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)HotRegion.cc 1.1 91/08/02 SMI"; 
#endif

// Includes
#include "HotRegion.h"

/*F+ void HotRegion::setDetectionFunction (HotRegionDetectionFunction det_func)
*
*    PURPOSE:
*
*    Allows the user to specify a detection function for detecting whether
*    or not an event is in this hot region.
*
*    PARAMETERS:
*
*    det_func - The user defined detection function.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void HotRegion::setDetectionFunction (HotRegionDetectionFunction det_func)
{
  detectionFunction = det_func;
}

/*F+ void HotRegion::setRegionHandler (UICHandlerFunction region_handler)
*
*    PURPOSE:
*
*    Allows the user to specify a function to handle an event that
*    occurs inside of this hot region.
*
*    PARAMETERS:
*
*    region_handler - The user defined region handler.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void HotRegion::setRegionHandler (UICHandlerFunction region_handler)
{
  regionHandler = region_handler;
}

/*F+ Boolean HotRegion::isHotRegion (int x, int y, UIObject *event_object)
*
*    PURPOSE:
*
*    Detects if an event occurring at the specified x-y location
*    is in this hot region.  If it is, the user supplied region
*    handler is called and a TRUE result is returned.  Normally this
*    is only used internally, but it can be used externally.
*
*    PARAMETERS:
*
*    x            - The x coordinate of an event.
*    y            - The y coordinate of an event.
*    event_object - The object the event occurred on.
*
*    RETURN VALUES:
*
*    If the specified x-y location is in the hot region, the return
*    value is TRUE.  Otherwise, it is FALSE.
*
*    SIDE EFFECTS:
*
*    The pointer to the object the event occurred on is saved in
*    a private Hot Region data member, hotRegionObject.  The pointer
*    is returned by the getHotRegionObject member function.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Boolean HotRegion::isHotRegion (int x, int y, UIObject *event_object)
{
  Boolean result;
  
  // If the user supplied a detection function...
  if (detectionFunction)
    // Call the user defined detection function
    result = (*detectionFunction) (x, y);
  // Else...
  else
    // Call the default detection function (rectangular hot region)
    result = defaultDetectionFunction (x, y);
  
  // If the x-y location is in the hot region...
  if (result) {
    // Save a pointer to the hot region object
    hotRegionObject = event_object;

    // Call the user defined region handler
    if (regionHandler)
      (*regionHandler) (this); 
  }
  
  return result;
}

#ifdef TEST
#endif /* TEST */
