/*H+ ScrollChoice.cc
*
*    SCCS ID: @(#)ScrollChoice.cc 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Defines the member functions of the ScrollChoice class.
*
*    AUTHOR: Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review. 
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)ScrollChoice.cc 1.1 91/08/02 SMI"; 
#endif

// Includes
#include <memory.h>

#include "CharacterFont.h"
#include "ScrollChoice.h"

#include "panel_list_structs.h"

/*F+ ScrollChoice::ScrollChoice ()
*
*    PURPOSE:
*
*    Constructor of the ScrollChoice class.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
ScrollChoice::ScrollChoice () : ()
{
  setScrollChoiceDefaults ();
}

/*F+ ScrollChoice::ScrollChoice (char *string_label)
*
*    PURPOSE:
*
*    Constructor of the ScrollChoice class.
*
*    PARAMETERS:
*
*    string_label - Specifies a string for the ScrollChoice label.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
ScrollChoice::ScrollChoice (char *string_label) : (string_label)
{
  setScrollChoiceDefaults ();
}

/*F+ ScrollChoice::ScrollChoice (Image &image_label)
*
*    PURPOSE:
*
*    Constructor of the ScrollChoice class.
*
*    PARAMETERS:
*
*    image_label - Specifies an image for the ScrollChoice label.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
ScrollChoice::ScrollChoice (Image &image_label) : (image_label)
{
  setScrollChoiceDefaults ();
}

/*F+ ScrollChoice::setScrollChoiceDefaults ()
*
*    PURPOSE:
*
*    Initializes the data members of the ScrollChoice class.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::setScrollChoiceDefaults ()
{
  rowHeight = UIC_NOT_SET;
  displayedRows = 5;
  internalSelection = UIC_NOT_SET;
  readOnly = FALSE;
  selectionRequired = FALSE;
  lastSelected = UIC_NOT_SET;
  listMenu = (UIMenu *) NULL;
  rowID = 0;
  labelFont = (CharacterFont *) NULL;
  listFont = (CharacterFont *) NULL;
}

/*F+ ScrollChoice::~ScrollChoice ()
*
*    PURPOSE:
*
*    Destructor of the ScrollChoice class.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
ScrollChoice::~ScrollChoice ()
{
}

/*F+ void ScrollChoice::deleteAllChoices ()
*
*    PURPOSE:
*
*    Deletes all of the rows in the list.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::deleteAllChoices ()
{
  int num_choices = getNumberOfChoices ();
  
  // Get rid of all of the selections
  selections.dropItems ();

  // If the XView object exists...
  if (owObject) {
    for (int i=num_choices-1; i > -1; i--)
      xv_set (owObject, PANEL_LIST_DELETE, i, NULL);
  }
  // Else, this is pre-XView list manipulation...
  else {
    // Delete all of the strings
    for (int i=num_choices-1; i > -1; i--) {
      char *string_entry = (char *) choices.removeItem (i);
      if (string_entry) delete (string_entry);
    }

    // Get rid of all the images
    images.dropItems ();
  }

  lastSelected = UIC_NOT_SET;
}

/*F+ void ScrollChoice::deleteChoice (int row_number)
*
*    PURPOSE:
*
*    Deletes the specified row from the list.
*
*    PARAMETERS:
*
*    row_number - The number of the row to delete.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::deleteChoice (int row_number)
{
  int num_choices = getNumberOfChoices ();

  if ((row_number <= num_choices) && (row_number > 0)) {
    // Make sure the row to be deleted is not selected
    setSelectedChoice (row_number, FALSE);

    // Decrease the row numbers of the selections higher then the specified row
    updateSelections (row_number, -1);

    row_number--;

    // If the XView object already exists...
    if (owObject) {
      xv_set (owObject, PANEL_LIST_DELETE, row_number, NULL);
      
      if (lastSelected == row_number)
	internalSelection = lastSelected = UIC_NOT_SET;
      else if (lastSelected > row_number)
	internalSelection = --lastSelected;
    }
    // Else, this is a pre-XView list...
    else {
      char *string_entry = (char *) choices.removeItem (row_number);
      Image *image_entry = (Image *) images.removeItem (row_number);
      
      if (string_entry) delete (string_entry);
    }
  }
}

/*F+ void ScrollChoice::deleteChoice (char *entry)
*
*    PURPOSE:
*
*    Deletes the first row with the specified string value.
*
*    PARAMETERS:
*
*    entry - The string value of the row to delete.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::deleteChoice (char *entry)
{
  int num_choices = getNumberOfChoices ();
  int row_number;

  // If the XView object already exists...
  if (owObject) {
    // Loop through all of the choices...
    for (row_number = num_choices-1; row_number > -1; row_number--) {
      char *temp_string = (char *) xv_get (owObject,
					   PANEL_LIST_STRING,
					   row_number);
      
      // If this is the specified item...
      if (temp_string && !strcmp (entry, temp_string)) {
	// Make sure it is no longer selected
	setSelectedChoice (row_number+1, FALSE);

	// Decrease the row numbers of the selections greater then this row
	updateSelections (row_number+1, -1);
	
	xv_set (owObject, PANEL_LIST_DELETE, row_number, NULL);
	
	if (lastSelected == row_number)
	  internalSelection = lastSelected = UIC_NOT_SET;
	else if (lastSelected > row_number)
	  internalSelection = --lastSelected;
	break;
      }
    }
  } 
  // Else, this is a pre-XView list...
  else {
    // Loop through all of the choices...
    for (row_number = num_choices-1; row_number > -1; row_number--) {
      char *temp_string = (char *) choices [row_number];
      
      // If this is the specified item...
      if (!strcmp (entry, temp_string)) {
	// Make sure it is no longer selected
	setSelectedChoice (row_number+1, FALSE);

	// Decrease the row numbers of the selections greater then this row
	updateSelections (row_number+1, -1);
	
	// Remove the item from the internal lists
	char *string_entry = (char *) choices.removeItem (row_number);
	Image *image_entry = (Image *) images.removeItem (row_number);
	
	if (string_entry) delete (string_entry);
	break;
      }
    }
  }
}

/*F+ void ScrollChoice::deleteChoice (Image &entry)
*
*    PURPOSE:
*
*    Deletes the first row with the specified image value.
*
*    PARAMETERS:
*
*    entry - The image value of the row to delete.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::deleteChoice (Image &entry)
{
  int num_choices = getNumberOfChoices ();
  int row_number;

  // If the XView object already exists...
  if (owObject) {
    // Loop through all of the choices...
    for (row_number = num_choices-1; row_number > -1; row_number--) {
      ListClientData *data = (ListClientData *)
	                     xv_get (owObject,
				     PANEL_LIST_CLIENT_DATA,
				     row_number);
      
      // If this is the specified item...
      if (data->image == &entry) {
	// Make sure it is no longer selected
	setSelectedChoice (row_number+1, FALSE);

	// Decrease the row numbers of the selections greater then this row
	updateSelections (row_number+1, -1);
	
	xv_set (owObject, PANEL_LIST_DELETE, row_number, NULL);
	
	if (lastSelected == row_number)
	  internalSelection = lastSelected = UIC_NOT_SET;
	else if (lastSelected > row_number)
	  internalSelection = --lastSelected;
	break;
      }
    }
  } 
  // Else, this is a pre-XView list...
  else {
    // Loop through all of the choices...
    for (row_number = num_choices-1; row_number > -1; row_number--) {
      Image *temp_image = (Image *) choices [row_number];
      
      // If this is the specified item...
      if (temp_image == &entry) {
	// Make sure it is no longer selected
	setSelectedChoice (row_number+1, FALSE);

	// Decrease the row numbers of the selections greater then this row
	updateSelections (row_number+1, -1);
	
	char *string_entry = (char *) choices.removeItem (row_number);
	Image *image_entry = (Image *) images.removeItem (row_number);
	
	if (string_entry) delete (string_entry);
	break;
      }
    }
  }
}

/*F+ void ScrollChoice::addChoice (char *entry)
*
*    PURPOSE:
*
*    Adds a row with the specified value to the end of the list.
*
*    PARAMETERS:
*
*    entry - The string value of the row to add.
*
*    RETURN VALUES:
*
*    Returns the position of the entry in the list.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int ScrollChoice::addChoice (char *entry)
{
  int row_number = getNumberOfChoices ();	

  // If the XView object already exists...
  if (owObject) {
    ListClientData *client_data = new ListClientData (++rowID, (Image *) NULL);

    // If a font has been specified...
    if (listFont)
      xv_set (owObject, PANEL_LIST_INSERT, row_number, 
	                PANEL_LIST_FONT,   row_number,
	                                   listFont->getXViewObject (),
	                PANEL_LIST_STRING, row_number, entry,
	                PANEL_LIST_CLIENT_DATA, row_number, client_data,
	      NULL);
    else
      xv_set (owObject, PANEL_LIST_INSERT, row_number, 
	                PANEL_LIST_STRING, row_number, entry,
	                PANEL_LIST_CLIENT_DATA, row_number, client_data,
	      NULL);
    
    // If a selection is required and there is one choice in the list...
    if (selectionRequired && (getNumberOfChoices () == 1))
      /* Set the first choice in the list as selected.  This ensures
	 that an item is always selected in a "selection required"
	 list */
      setSelectedChoice (1, TRUE);
  }
  // Else, this is pre-XView list construction...
  else {
    char *temp_string = strcpy (new char [strlen (entry) + 1], entry);
    
    // Add the entry to the internal lists
    choices.addItem ((Generic *) temp_string);
    images.addItem ((Generic *) NULL);
  }
  
  // Return the row number of the new item
  return (row_number + 1);
}

/*F+ void ScrollChoice::addChoice (Image &entry)
*
*    PURPOSE:
*
*    Adds a row with the specified value to the end of the list.
*
*    PARAMETERS:
*
*    entry - The image value of the row to add.
*
*    RETURN VALUES:
*
*    Returns the position of the entry in the list.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int ScrollChoice::addChoice (Image &entry)
{
  int row_number = getNumberOfChoices ();	

  // If the XView object already exists...
  if (owObject) {
    // Create a client data structure to be stored with the new row
    ListClientData *client_data = new ListClientData (++rowID, &entry);

    ((UIObject *) &entry)->createObject (this);
    xv_set (owObject, PANEL_LIST_INSERT, row_number, 
		      PANEL_LIST_STRING, row_number, "  ",
		      PANEL_LIST_GLYPH,  row_number, entry.getXViewObject (), 
		      PANEL_LIST_CLIENT_DATA, row_number, client_data,
	    NULL);

    // If a selection is required and there is one choice in the list...
    if (selectionRequired && (getNumberOfChoices () == 1))
      /* Set the first choice in the list as selected.  This ensures
	 that an item is always selected in a "selection required"
	 list */
      setSelectedChoice (1, TRUE);
  }
  // Else, this is pre-XView list construction...
  else {
    choices.addItem ((Generic *) NULL);
    images.addItem ((Generic *) &entry);
  }
  
  return (row_number + 1);
}

/*F+ void ScrollChoice::addChoice (char *string, Image &image)
*
*    PURPOSE:
*
*    Adds a row with the specified values to the end of the list.
*
*    PARAMETERS:
*
*    string - The string value of the row to add.
*    image  - The image value of the row to add.
*
*    RETURN VALUES:
*
*    Returns the position of the entry in the list.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int ScrollChoice::addChoice (char *string, Image &image)
{
  int row_number = getNumberOfChoices ();

  // If the XView object already exists...
  if (owObject) {
    // Create a client data structure to be stored with the new row
    ListClientData *client_data = new ListClientData (++rowID, &image);

    // Create XView image
    ((UIObject *) &image)->createObject (this);
    
    // If a font is specified...
    if (listFont) {
      xv_set (owObject, PANEL_LIST_INSERT, row_number, 
	                PANEL_LIST_FONT,   row_number,
	                                   listFont->getXViewObject (),
	                PANEL_LIST_STRING, row_number, string,
	                PANEL_LIST_GLYPH,  row_number, image.getXViewObject (),
			PANEL_LIST_CLIENT_DATA, row_number, client_data,
	      NULL);
    }
    // Else, no font is specified...
    else
      xv_set (owObject, PANEL_LIST_INSERT, row_number, 
	                PANEL_LIST_STRING, row_number, string,
	                PANEL_LIST_GLYPH, row_number, image.getXViewObject (),
	                PANEL_LIST_CLIENT_DATA, row_number, client_data,
	      NULL);
    
    // If a selection is required and there is one choice in the list...
    if (selectionRequired && (getNumberOfChoices () == 1))
      /* Set the first choice in the list as selected.  This ensures
	 that an item is always selected in a "selection required"
	 list */
      setSelectedChoice (1, TRUE);
  }
  // Else, this is pre-XView list construction...
  else {
    char *temp_string = strcpy (new char [strlen (string) + 1], string);
    
    choices.addItem ((Generic *) temp_string);
    images.addItem ((Generic *) &image);
  }
  
  return (row_number + 1);
}

/*F+ void ScrollChoice::addChoice (int row_number, char *entry)
*
*    PURPOSE:
*
*    Inserts a row with the specified value at the specified position
*    in the list.
*
*    PARAMETERS:
*
*    row_number - The position of the new row.
*    entry      - The string value of the row to add.
*
*    RETURN VALUES:
*
*    Returns the position of the entry in the list.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int ScrollChoice::addChoice (int row_number, char *entry)
{
  int num_items = getNumberOfChoices ();

  row_number--;

  // If the row number is greater then the number of items in the list...
  if (row_number > num_items-1)
    // Add the item to the end of the list
    row_number = addChoice (entry) - 1;
  else {
    // If the XView object already exists...
    if (owObject) {
      ListClientData *client_data =
		      new ListClientData (++rowID, (Image *) NULL);
    
      if (listFont)
	xv_set (owObject, PANEL_LIST_INSERT, row_number, 
		          PANEL_LIST_FONT,   row_number,
					     listFont->getXViewObject (),
			  PANEL_LIST_STRING, row_number, entry,
			  PANEL_LIST_CLIENT_DATA, row_number, client_data,
		NULL);
      else
	xv_set (owObject, PANEL_LIST_INSERT, row_number, 
			  PANEL_LIST_STRING, row_number, entry,
			  PANEL_LIST_CLIENT_DATA, row_number, client_data,
		NULL);

      // If a selection is required and there is one choice in the list...
      if (selectionRequired && (getNumberOfChoices () == 1))
	/* Set the first choice in the list as selected.  This ensures
	   that an item is always selected in a "selection required"
	   list */
	setSelectedChoice (1, TRUE);
    }
    // Else, this is a pre-XView list...
    else {
      char  *temp_string = strcpy (new char [strlen (entry) + 1], entry);

      // Add the current last item to the end of the list
      choices.addItem (choices [num_items-1]);
      images.addItem (images [num_items-1]);

      // Shift all of the items after the new row
      for (int i=num_items-2; i >= row_number; i--) {
	choices [i+1] = choices [i];
	images [i+1] = images [i];
      }

      // Put the new row into the list
      choices [row_number] = (Generic *) temp_string;
      images [row_number] = (Generic *) NULL;

      // Update the the selected row numbers that are greater then the new one
      updateSelections (row_number+1, 1);
    }
  }
  
  return (row_number + 1);
}

/*F+ void ScrollChoice::addChoice (int row_number, Image &entry)
*
*    PURPOSE:
*
*    Inserts a row with the specified value at the specified position
*    in the list.
*
*    PARAMETERS:
*
*    row_number - The position of the new row.
*    entry      - The image value of the row to add.
*
*    RETURN VALUES:
*
*    Returns the position of the entry in the list.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int ScrollChoice::addChoice (int row_number, Image &entry)
{
  int num_items = getNumberOfChoices ();

  row_number--;

  // If the row number is greater then the number of items in the list...
  if (row_number > num_items-1)
    // Add the item to the end of the list
    row_number = addChoice (entry) - 1;
  else {
    // If the XView object already exists...
    if (owObject) {
      ListClientData *client_data = new ListClientData (++rowID, &entry);

      ((UIObject *) &entry)->createObject (this);
      xv_set (owObject, PANEL_LIST_INSERT, row_number,
		        PANEL_LIST_STRING, row_number, "  ",
		        PANEL_LIST_GLYPH,  row_number, entry.getXViewObject (),
		        PANEL_LIST_CLIENT_DATA, row_number, client_data,
	      NULL);

      // If a selection is required and there is one choice in the list...
      if (selectionRequired && (getNumberOfChoices () == 1))
	/* Set the first choice in the list as selected.  This ensures
	   that an item is always selected in a "selection required"
	   list */
	setSelectedChoice (1, TRUE);
    }
    // Else, this is a pre-XView list...
    else {
      // Add the current last item to the end of the list
      choices.addItem (choices [num_items-1]);
      images.addItem (images [num_items-1]);

      // Shift all of the items after the new row
      for (int i=num_items-2; i >= row_number; i--) {
	choices [i+1] = choices [i];
	images [i+1] = images [i];
      }

      // Put the new row into the list
      choices [row_number] = (Generic *) NULL;
      images [row_number] = (Generic *) &entry;

      // Update the the selected row numbers that are greater then the new one
      updateSelections (row_number+1, 1);
    }
  }

  return (row_number + 1);
}

/*F+ void ScrollChoice::addChoice (int row_number, char *string, Image &image)
*
*    PURPOSE:
*
*    Inserts a row with the specified value at the specified position
*    in the list.
*
*    PARAMETERS:
*
*    row_number - The position of the new row.
*    string     - The string value of the row to add.
*    image      - The image value of the row to add.
*
*    RETURN VALUES:
*
*    Returns the position of the entry in the list.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int ScrollChoice::addChoice (int row_number, char *string, Image &image)
{
  int num_items = getNumberOfChoices ();

  row_number--;

  // If the row number is greater then the number of items in the list...
  if (row_number > num_items-1)
    // Add the item to the end of the list
    row_number = addChoice (string, image) - 1;
  else {
    // If the XView object already exists...
    if (owObject) {
      ListClientData *client_data = new ListClientData (++rowID, &image);

      // Create XView image
      ((UIObject *) &image)->createObject (this);

      // If a font is specified...
      if (listFont) {
	xv_set (owObject, PANEL_LIST_INSERT, row_number, 
	                  PANEL_LIST_FONT,   row_number,
					     listFont->getXViewObject (),
			  PANEL_LIST_STRING, row_number, string,
			  PANEL_LIST_GLYPH,  row_number,
		                             image.getXViewObject (),
			  PANEL_LIST_CLIENT_DATA, row_number, client_data,
		NULL);
      }
      // Else , no font specified...
      else
	xv_set (owObject, PANEL_LIST_INSERT, row_number, 
			  PANEL_LIST_STRING, row_number, string,
			  PANEL_LIST_GLYPH,  row_number,
		                             image.getXViewObject (),
		          PANEL_LIST_CLIENT_DATA, row_number, client_data,
		NULL);

      // If a selection is required and there is one choice in the list...
      if (selectionRequired && (getNumberOfChoices () == 1))
	/* Set the first choice in the list as selected.  This ensures
	   that an item is always selected in a "selection required"
	   list */
	setSelectedChoice (1, TRUE);
    }
    // Else, this is a pre-XView list...
    else {
      char  *temp_string = strcpy (new char [strlen (string) + 1], string);
    
      // Add the current last item to the end of the list
      choices.addItem (choices [num_items-1]);
      images.addItem (images [num_items-1]);

      // Shift all of the items after the new row
      for (int i=num_items-2; i >= row_number; i--) {
	choices [i+1] = choices [i];
	images [i+1] = images [i];
      }

      // Put the new row into the list
      choices [row_number] = (Generic *) temp_string;
      images [row_number] = (Generic *) &image;

      // Update the the selected row numbers that are greater then the new one
      updateSelections (row_number+1, 1);
    }
  }
  
  return (row_number + 1);
}

/*F+ void ScrollChoice::setMenu (UIMenu &menu)
*
*    PURPOSE:
*
*    Specifies the menu to be shown if the user presses the menu
*    mouse button while on top of the ScrollChoice object.
*
*    PARAMETERS:
*
*    menu - The menu to show.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::setMenu (UIMenu &menu)
{
  listMenu = &menu;

  // If the XView object already exists...
  if (owObject) {
    // If an old menu exists, delete it
    Menu old_menu = (Menu) xv_get (owObject, PANEL_ITEM_MENU);
    if (old_menu) xv_destroy_safe (old_menu);

    // Create the new XView menu object and set the menu
    ((UIObject *) listMenu)->createObject (this);
    xv_set (owObject, PANEL_ITEM_MENU, listMenu->getXViewObject (), NULL);
  }	
}

/*F+ char *ScrollChoice::getChoice (char* &return_value)
*
*    PURPOSE:
*
*    Returns the string value associated with the last list item
*    chosen.
*
*    PARAMETERS:
*
*    return_value - The string value of the last selected item.
*
*    RETURN VALUES:
*
*    The string value of the last selected item is returned.  Memory
*    to store the string is allocated on the heap and needs to be
*    deleted by the caller.  If there was no last selected item,
*    NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
char *ScrollChoice::getChoice (char* &return_value)
{
  return_value = (char *) NULL;

  if (lastSelected != UIC_NOT_SET)
    (void) getValue (lastSelected+1, return_value);

  return return_value;
}

/*F+ Image *ScrollChoice::getChoice (Image* &return_value)
*
*    PURPOSE:
*
*    Returns a pointer to the image value associated with the last list item
*    chosen.
*
*    PARAMETERS:
*
*    return_value - A pointer to the image value of the last selected item.
*
*    RETURN VALUES:
*
*    A pointer to the image value of the last selected item is returned.
*    If there was no last selected item, NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Image *ScrollChoice::getChoice (Image* &return_value)
{
  return_value = (Image *) NULL;

  if (lastSelected != UIC_NOT_SET)
    (void) getValue (lastSelected+1, return_value);

  return return_value;
}

/*F+ int ScrollChoice::getChoice (int &return_value)
*
*    PURPOSE:
*
*    Returns the row number associated with the last list item chosen.
*
*    PARAMETERS:
*
*    return_value - The row number of the last selected item.
*
*    RETURN VALUES:
*
*    The row number of the last selected item is returned.
*    If there was no last selected item, UIC_NOT_SET is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int ScrollChoice::getChoice (int &return_value)
{
  return (return_value = (lastSelected != UIC_NOT_SET) ?
	                 lastSelected + 1 : UIC_NOT_SET);
}

/*F+ char *ScrollChoice::getChoice (int selection, char* &return_value)
*
*    PURPOSE:
*
*    This version of getChoice returns the string value associated
*    with the selection specified out the list of current selections.
*
*    PARAMETERS:
*
*    return_value - The string value of the specified selection out of
*                   the list of selections.
*
*    RETURN VALUES:
*
*    The string value of the selection is returned.  The memory containing
*    the string is allocated off of the heap.  The caller must delete
*    memory.  If the selection specified does not exist, NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
char *ScrollChoice::getChoice (int selection, char *&return_value)
{
  return_value = (char *) NULL;

  if ((selection > 0) && (selection <= selections.getSize ())) {
    int row_number = (int) selections [selection-1];
    (void) getValue (row_number+1, return_value);
  }

  return (return_value);
}     

/*F+ Image *ScrollChoice::getChoice (int selection, Image* &return_value)
*
*    PURPOSE:
*
*    This version of getChoice returns a pointer to the image value associated
*    with the selection specified out the list of current selections.
*
*    PARAMETERS:
*
*    return_value - A pointer to the image value of the specified selection
*                   out of the list of selections.
*
*    RETURN VALUES:
*
*    A pointer to the image value of the selection is returned.  If the
*    selection specified does not exist, NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Image *ScrollChoice::getChoice (int selection, Image *&return_value)
{
  return_value = (Image *) NULL;

  if ((selection > 0) && (selection <= selections.getSize ())) {
    int row_number = (int) selections [selection-1];
    (void) getValue (row_number+1, return_value);
  }

  return (return_value);
}     

/*F+ int ScrollChoice::getChoice (int selection, int &return_value)
*
*    PURPOSE:
*
*    This version of getChoice returns the row number associated
*    with the selection specified out the list of current selections.
*
*    PARAMETERS:
*
*    return_value - The row number of the specified selection out of
*                   the list of selections.
*
*    RETURN VALUES:
*
*    The row number of the selection is returned.  If the selection
*    specified does not exist, UIC_NOT_SET is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int ScrollChoice::getChoice (int selection, int &return_value)
{
  return_value = UIC_NOT_SET;

  if ((selection > 0) && (selection <= selections.getSize ()))
    return_value = ((int) selections [selection-1]) + 1;

  return (return_value);
}     

/*F+ int ScrollChoice::getNumberSelected ()
*
*    PURPOSE:
*
*    Returns the number of currently selected choices.
*
*    RETURN VALUES:
*
*    Returns the number of currently selected choices.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int ScrollChoice::getNumberSelected ()
{
  return (selections.getSize ());
}
 
/*F+ char *ScrollChoice::getValue (char* &return_value)
*
*    PURPOSE:
*
*    Returns the string value associated with the last list item
*    chosen.
*
*    PARAMETERS:
*
*    return_value - The string value of the last selected item.
*
*    RETURN VALUES:
*
*    The string value of the last selected item is returned.  Memory
*    to store the string is allocated on the heap and needs to be
*    deleted by the caller.  If there was no last selected item,
*    NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
char *ScrollChoice::getValue (char* &return_value)
{ 
  return (getChoice (return_value));
}

/*F+ Image *ScrollChoice::getValue (Image* &return_value)
*
*    PURPOSE:
*
*    Returns a pointer to the image value associated with the last list item
*    chosen.
*
*    PARAMETERS:
*
*    return_value - A pointer to the image value of the last selected item.
*
*    RETURN VALUES:
*
*    A pointer to the image value of the last selected item is returned.
*    If there was no last selected item, NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Image *ScrollChoice::getValue (Image* &return_value)
{
  return (getChoice (return_value));
}

/*F+ char *ScrollChoice::getValue (int row_number, char* &return_value)
*
*    PURPOSE:
*
*    Returns the string value associated with the specified row.
*
*    PARAMETERS:
*
*    return_value - The string value of the specified row.
*
*    RETURN VALUES:
*
*    The string value of the specified row is returned.  Memory
*    to store the string is allocated on the heap and needs to be
*    deleted by the caller.  If the specified row does not exist,
*    NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
char *ScrollChoice::getValue (int row_number, char* &value)
{
  value = (char *) NULL;

  // If the row exists...
  if ((row_number > 0) && (row_number <= getNumberOfChoices ())) {
    char *temp_string;

    row_number--;
    
    // If the XView object already exists...
    if (owObject) {
      temp_string = (char *) xv_get (owObject,
				     PANEL_LIST_STRING, row_number,
				     NULL);
      
      if (temp_string)
	// If the string is a blank or a selection indication arrow...
	if (!strcmp (temp_string, "  ") || !strcmp (temp_string, "<-"))
	  temp_string = (char *) NULL;
    }
    // Else, this is a pre-XView list...
    else
      temp_string = (char *) choices [row_number];

    if (temp_string)
      value = strcpy (new char [strlen (temp_string)+1], temp_string);
  }

  return (value);
}

/*F+ Image *ScrollChoice::getValue (int row_number, Image* &return_value)
*
*    PURPOSE:
*
*    Returns a pointer to the image value associated with the specified row.
*
*    PARAMETERS:
*
*    return_value - A pointer to the image value of the specified row.
*
*    RETURN VALUES:
*
*    A pointer to the image value of the specified row is returned.
*    If the specified row does not exist, NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Image *ScrollChoice::getValue (int row_number, Image* &value)
{
  value = (Image *) NULL;

  // If the row exists...
  if ((row_number > 0) && (row_number <= getNumberOfChoices ())) {
    row_number--;

    // If the XView object already exists...
    if (owObject) {
      // Get the client data for the row
      ListClientData *client_data = (ListClientData *)
	                            xv_get (owObject,
					    PANEL_LIST_CLIENT_DATA, row_number,
					    NULL);

      // Get the image value contained in the client data
      value = ((client_data) ? client_data->image : (Image *) NULL);
    }
    // Else, this is a pre-XView list...
    else
      value = (Image *) images [row_number];
  }

  return (value);
}

/*F+ void ScrollChoice::replaceChoice (int row_number, char *entry)
*
*    PURPOSE:
*
*    Replaces the string value of the specified row.  If the row number
*    is higher then the existing number of rows, a new item is placed
*    at the end of the list.
*
*    PARAMETERS:
*
*    entry - The new string value.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::replaceChoice (int row_number, char *entry)
{
  // If the row exists...
  if ((row_number > 0) && (row_number <= getNumberOfChoices ())) {
    row_number--;
    
    // If the XView object already exists...
    if (owObject) {
      // If a font is specified...
      if (listFont)
	xv_set (owObject, PANEL_LIST_STRING, row_number, entry,
		          PANEL_LIST_FONT,   row_number,
		                             listFont->getXViewObject (),
		NULL);
      // Else, no font specified
      else
	xv_set (owObject, PANEL_LIST_STRING, row_number, entry, NULL);
    }
    // Else, pre-XView list...
    else {
      char *temp_string = strcpy (new char [strlen (entry)+1], entry);

      delete ((char *) choices [row_number]);
      choices [row_number] = (Generic *) temp_string;
    }
  }
  // Else, add a new row...
  else
    (void) addChoice (entry);
}

/*F+ void ScrollChoice::replaceChoice (int row_number, Image &entry)
*
*    PURPOSE:
*
*    Replaces the image value of the specified row.  If the row number
*    is higher then the existing number of rows, a new item is placed
*    at the end of the list.
*
*    PARAMETERS:
*
*    entry - The new image value.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::replaceChoice (int row_number, Image &entry)
{
  // If the row exists...
  if ((row_number > 0) && (row_number <= getNumberOfChoices ())) {
    row_number--;
    
    // If the XView object already exists...
    if (owObject) {
      // Get the client data for the row
      ListClientData *client_data = (ListClientData *)
	                            xv_get (owObject,
					    PANEL_LIST_CLIENT_DATA, row_number,
					    NULL);

      // Set the client data image value
      client_data->image = &entry;

      // Set the row XView image
      ((UIObject *) &entry)->createObject (this);
      xv_set (owObject,
	      PANEL_LIST_GLYPH, row_number, entry.getXViewObject (),
	      NULL);
    }
    // Else, pre-XView list...
    else
      images [row_number] = (Generic *) &entry;
  }
  // Else, add a new row...
  else
    (void) addChoice (entry);
}

/*F+ void ScrollChoice::replaceChoice (int    row_number,
*                                      char  *string,
*                                      Image &image)
*
*    PURPOSE:
*
*    Replaces the string and image values of the specified row.  If the row
*    number is higher then the existing number of rows, a new item is placed
*    at the end of the list.
*
*    PARAMETERS:
*
*    entry - The new image value.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::replaceChoice (int row_number, char *string, Image &image)
{
  // If the row exists...
  if ((row_number > 0) && (row_number <= getNumberOfChoices ())) {
    row_number--;
    
    // If the XView object already exists...
    if (owObject) {
      // Get the client data for the row
      ListClientData *client_data = (ListClientData *)
	                            xv_get (owObject,
					    PANEL_LIST_CLIENT_DATA, row_number,
					    NULL);

      // Set the client data UIC image value
      client_data->image = &image;

      // Create XView object
      ((UIObject *) &image)->createObject (this);

      // If a font is specified...
      if (listFont)
	xv_set (owObject, PANEL_LIST_STRING, row_number, string,
	                  PANEL_LIST_GLYPH,  row_number,
                                             image.getXViewObject (),
		          PANEL_LIST_FONT,   row_number,
		                             listFont->getXViewObject (),
		NULL);
      // Else, no font specified
      else
	xv_set (owObject, PANEL_LIST_STRING, row_number, string,
	                  PANEL_LIST_GLYPH,  row_number,
                                             image.getXViewObject (),
                NULL);
    }
    // Else, pre-XView list...
    else {
      char *temp_string = strcpy (new char [strlen (string)+1], string);

      delete ((char *) choices [row_number]);

      choices [row_number] = (Generic *) temp_string;
      images [row_number] = (Generic *) &image;
    }
  }
  // Else, add a new row...
  else
    (void) addChoice (string, image);
}

/*F+ void ScrollChoice::setSelectedChoice (int choice_index, Boolean flag)
*
*    PURPOSE:
*
*    Sets the selection status of the specified row.
*
*    PARAMETERS:
*
*    choice_index - The number of the row to set the selection status of.
*    flag         - TRUE if the row is to set as selected, FALSE if not
*                   selected.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::setSelectedChoice (int choice_index, Boolean flag)
{
  int num_choices = selections.getSize ();
    
  if (flag) {
    int temp_int = choice_index - 1;
    
    // If a single choice item...
    if (!multipleChoice && selections.getSize ()) {
      // Turn off the old choice
      int old_item = (int) selections.removeItem (0);
      if (owObject) showFeedback (old_item+1, FALSE);
    }
    else {
      /* If the selected item is already in the list, remove it
         and place it at the end of the list */

      // Loop through the selections list
      for (int i=0; i < num_choices; i ++) {
	// If this is the item of interest...
        if (((int) selections [i]) == choice_index-1) {
	  // Remove it from the list
	  (void) selections.removeItem (i);
	  break;
        }
      }
    }

    selections.addItem ((Generic *) temp_int);
    internalSelection = lastSelected = temp_int;
  }
  else {
    // Loop through the selections list
    for (int i=0; i < num_choices; i ++) {
      // If this is the item of interest...
      if (((int) selections [i]) == choice_index-1) {
	// Remove it from the list
	(void) selections.removeItem (i);
	break;
      }
    }
  }

  if (owObject)
    showFeedback (choice_index, flag);
}

/*F+ void ScrollChoice::setSelectedChoice (char *choice_value, Boolean flag)
*
*    PURPOSE:
*
*    Sets the selection status of the specified row.
*
*    PARAMETERS:
*
*    choice_value - The string value of the row to set the selection status of.
*    flag         - TRUE if the row is to set as selected, FALSE if not
*                   selected.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::setSelectedChoice (char *choice_value, Boolean flag)
{
  int   num_choices = choices.getSize ();
  char *temp_string;

  for (int i = 0; i < num_choices ; i++) {
    temp_string = (char *) choices [i];
      
    if (!strcmp (temp_string, choice_value)) {
      setSelectedChoice (i+1, flag);
      break;
    }
  }
}

/*F+ void ScrollChoice::setSelectedChoice (Image &choice_value, Boolean flag)
*
*    PURPOSE:
*
*    Sets the selection status of the specified row.
*
*    PARAMETERS:
*
*    choice_value - The image value of the row to set the selection status of.
*    flag         - TRUE if the row is to set as selected, FALSE if not
*                   selected.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::setSelectedChoice (Image &choice_value, Boolean flag)
{
  int row_index = images.findItem ((Generic *) &choice_value);
  setSelectedChoice (row_index+1, flag);
}

/*F+ Boolean ScrollChoice::isSelectedChoice (int row_number)
*
*    PURPOSE:
*
*    Determines whether or not the specified row has been selected.
*
*    RETURN VALUES:
*
*    Returns TRUE if the specified row has been selected.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Boolean ScrollChoice::isSelectedChoice (int row_number)
{
  int choice = row_number - 1;
  int num_selections = selections.getSize ();

  // Loop throught the selections...
  for (int i=0; i < num_selections; i++)
    // If this one matches...
    if (choice == (int) selections [i])
      return TRUE;

  return FALSE;
}

/*F+ Boolean ScrollChoice::scrollChoiceEventProc (Panel_item  item,
*                                                 Event      *xview_event)
*
*    PURPOSE:
*
*    Internal event proc wrapper for the ScrollChoice class.  This is
*    called by XView whenever an event occurs on the ScrollChoice
*    object.  This proc then handles scrolling, showing feedback,
*    and detecting and handling notification operations for the
*    panel list.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::scrollChoiceEventProc (Panel_item item, Event *xview_event)
{
  ScrollChoice *object = (ScrollChoice *) xv_get (item,
						  XV_KEY_DATA,
						  UIC_OBJECT_KEY);
  
  int         choice;
  InputEvent *event;
  int old_choice = object->internalSelection; // Save the previous selection
  
  object->selectStatus = UNKNOWN;

  // Set the internal UIC event information
  object->setEventInformation (xview_event);

  // If the user specified an event handler...
  if (object->eventHandler) {
    (*object->eventHandler) (object);

    // The user needs to provide his own feedback mechanisms...
    panel_default_handle_event (item, xview_event);
    return;
  }

  // Get the UIC event information
  event = object->getEvent ();

  int num_choices = object->getNumberOfChoices ();

  // If the event was a press of the mouse menu button...
  if ((event->getAction () == ACTION_MENU) && (event->isButtonDown ())) {
    // Get the list menu, if it exists...
    Menu menu = (Menu) xv_get (item, PANEL_ITEM_MENU);

    if (object->listMenu && menu) {
      // Show the list menu
      Frame frame = (Frame) xv_get (menu, XV_KEY_DATA, UIC_FRAME_KEY);
      menu_show (menu, frame, xview_event);
    }

    // Erase the internal UIC event information
    object->resetEventInformation ();

    // All done with this event...
    panel_default_handle_event (item, xview_event);
    return;
  }

  // If there is nothing in this list...
  if (num_choices < 1) {
    // Erase the internal UIC event information
    object->resetEventInformation ();

    // All done with this event...
    panel_default_handle_event (item, xview_event);
    return;
  }
  
  // If this event is requesting scrollbar access...
  if (event->getID () == SCROLLBAR_REQUEST) {
    /* Add a choice to the end of the list and delete it:

       This will force XView to scroll the list to the correct location
       (It's a terrible hack, but it works...) */
    object->addChoice (num_choices+1, " ");
    object->deleteChoice (num_choices+1);
  }
  
  // If the event is the select mouse button being pressed...
  if ((event->getAction () == ACTION_SELECT) && (event->isButtonDown ())) {
    // Get the X-Y location of the event
    int ex = event->getX ();
    int ey = event->getY ();

    /* Obtain a pointer to the panel list private data section and
       get the current scrollbar position */
    panel_list_info_t *p_list = (panel_list_info_t *)
                                ((Xv_panel_list *) item)->private_data;
    int sb_pos = (int) xv_get (p_list->list_sb, SCROLLBAR_VIEW_START);

    // Get the pixel height of a panel list row
    int height = (int) xv_get (item, PANEL_LIST_ROW_HEIGHT);

    // If the event X coordinate is less then the actual list box X coord...
    if (ex < p_list->list_box.r_left)
      // Invalidate the event by setting "choice" to an illegal value
      choice = -1;
    // Else, the event is within the X bounds...
    else
      /* Calculate the choice by getting the Y offset into the list,
	 dividing the offset by the row height, and adding the current
	 scrollbar position.  The "10" in the equation is due to a
	 gap between the top of the actual list box (r_top) and the first
	 list entry. */
      choice = (ey - p_list->list_box.r_top + 10) / height + sb_pos - 1;

    // If the calculated choice is not legal...
    if ((choice < 0) || (choice > num_choices-1)) {
      // All done with this event...
      panel_default_handle_event (item, xview_event);
      return;
    }

    /* If this is a multiple choice list, then don't worry about the
       value of the previous choice.  Pretend this choice was the
       previous choice so we can use it detect whether or not to
       deselect the current multiple choice selection. */
    if (object->multipleChoice) old_choice = choice;

    int num_selected = object->selections.getSize ();

    /* If the old choice is set (this is always true for multiple choice)... */
    if (old_choice != UIC_NOT_SET) {
      /* 1) If the current choice is not equal to the previous choice
            (never true for multiple choice) or

	 2) If the list is not multiple choice and the previous choice and
	    the current choice are the same and no selection is required or

	 3) If the list is not multiple choice and the previous choice and
	    the current choice are the same and the list has more then
	    one entry in a selected state or

	 4) If the list is mulitple choice, the current choice is already
	    in a selected state and no selection is required or

	 5) If the list is mulitple choice, the current choice is already
	    in a selected state and the list has more then one entry
	    in a selected state...

	 Then... */
      if ((old_choice != choice) ||
	  (((!object->selectionRequired) || (num_selected > 1)) &&
	   (!object->multipleChoice)) ||
	  (object->multipleChoice && object->isSelectedChoice (choice+1) &&
	   ((!object->selectionRequired) || (num_selected > 1)))) {
	// Set the status to DESELECT
        object->selectStatus = DESELECT;

	/* Deselect the previous choice (which is the current choice in the
	   case of a multiple choice list). */
	object->setSelectedChoice (old_choice+1, FALSE);

	/* Set the last selected value to the deselected item number
	   so the user can obtain it via a getChoice call */
	object->internalSelection = object->lastSelected = choice;

	// Call the notify proc
        object->notifyProc ();
      }
    }

    /* If the previous choice and the current choice are the same and
       the the choice was deselected... */
    if ((old_choice == choice) && (object->selectStatus == DESELECT)) {
      // There was no choice selected this time...
      object->internalSelection = UIC_NOT_SET;

      // ...and we are done with this event...
      panel_default_handle_event (item, xview_event);
      return;
    }

    /* The current choice is a new selection or must be kept on because
       selection is required... */
    object->selectStatus = SELECT;
    object->setSelectedChoice (choice+1, TRUE);

    // Call the notify proc
    object->notifyProc ();
  }

  // Erase the internal UIC event information
  object->resetEventInformation ();

  panel_default_handle_event (item, xview_event);
}

/*F+ void ScrollChoice::notifyProc ()
*
*    PURPOSE:
*
*    Internal notify proc wrapper for the ScrollChoice class.  This is
*    called by the scrollChoiceEventProc when a notify occurs.
*    This proc then calls the user specified UIC target and notify handlers.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::notifyProc ()
{
  // Notify the associated targets
  notifyNotifyTargets ();

  // Call the user specified notify handler
  if (notifyHandler)
    (*notifyHandler) (this);
}

/*F+ void ScrollChoice::showFeedback (int row_number, Boolean flag)
*
*    PURPOSE:
*
*    Shows selection or deselection visual feedback for the specified row.
*
*    PARAMETERS:
*
*    row_number - The row to show the visual feedback for.
*    flag       - If true, turn the row on.  If FALSE, turn the row off.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::showFeedback (int row_number, Boolean flag)
{
  char    *family = listFont->getFamily ();
  Xv_font  font;
  int      size = listFont->getSize ();

  row_number--;

  // Use the frame for an XView font parent
  Frame frame = (Frame) xv_get (owObject, XV_KEY_DATA, UIC_FRAME_KEY);

  // If the selection is on...
  if (flag) {
    char *temp_string = (char *) xv_get (owObject,
					 PANEL_LIST_STRING,
					 row_number);

    // If the string is blank, replace it with a selection arrow (<-)
    if (!strcmp (temp_string, "  "))
      xv_set (owObject, PANEL_LIST_STRING, row_number, "<-", NULL);

    // Create a bold font using the current list font family and size
    font = (Xv_font) xv_create (frame, FONT,
				FONT_FAMILY, family,
				FONT_STYLE,  FONT_STYLE_BOLD,
				FONT_SIZE,   size,
				NULL);
  }
  // Else, the selection is off...
  else {
    char *temp_string = (char *) xv_get (owObject,
					 PANEL_LIST_STRING,
					 row_number);

    // If the string is a selection arrow (<-), replace it with a blank string
    if (!strcmp (temp_string, "<-"))
      xv_set (owObject, PANEL_LIST_STRING, row_number, "  ", NULL);

    // Create a normal font using the current list font family and size
    font = (Xv_font) xv_create (frame, FONT,
				FONT_FAMILY, family,
				FONT_STYLE,  FONT_STYLE_NORMAL,
				FONT_SIZE,   size,
				NULL);
  }

  // Set the font of the row to the newly created font
  xv_set (owObject, PANEL_LIST_FONT, row_number, font, NULL);

  delete family;
}

/*F+ void ScrollChoice::createObject (UIObject *parent)
*
*    PURPOSE:
*
*    Creates the XView object of the ScrollChoice object.
*
*    PARAMETERS:
*
*    parent - The UIC parent object (a ComponentDisplay).
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void ScrollChoice::createObject (UIObject *parent)
{
  /* NOTE: row height, label fonts, and read only status must be specified
           at the time of the xv_create call... */

  // If row height was specified...
  if (rowHeight != UIC_NOT_SET) {
    // If a lbel font was specified...
    if (labelFont) {
      // Create the XView label font object...
      ((UIObject *) labelFont)->createObject (parent);
      
      // If this list is read only...
      if (readOnly)
	owObject = xv_create (parent->getXViewObject (), PANEL_LIST,
			      PANEL_LABEL_FONT, labelFont->getXViewObject (),
			      PANEL_NOTIFY_PROC,         notifyProc,
			      PANEL_READ_ONLY,	         TRUE,
			      PANEL_LIST_DISPLAY_ROWS,   displayedRows,
			      PANEL_LIST_ROW_HEIGHT,	 rowHeight,
			      NULL);
      // Else, not read only...
      else
	owObject = xv_create (parent->getXViewObject (), PANEL_LIST,
			      PANEL_LABEL_FONT, labelFont->getXViewObject (),
			      PANEL_NOTIFY_PROC,         notifyProc,
			      PANEL_LIST_DISPLAY_ROWS,   displayedRows,
			      PANEL_LIST_ROW_HEIGHT,	 rowHeight,
			      NULL);
    }
    // Else, no label font specified...
    else {
      // If this list is read only...
      if (readOnly)
	owObject = xv_create (parent->getXViewObject (), PANEL_LIST,
			      PANEL_NOTIFY_PROC,         notifyProc,
			      PANEL_READ_ONLY,	         TRUE,
			      PANEL_LIST_DISPLAY_ROWS,   displayedRows,
			      PANEL_LIST_ROW_HEIGHT,	 rowHeight,
			      NULL);
      // Else, not read only...
      else
	owObject = xv_create (parent->getXViewObject (), PANEL_LIST,
			      PANEL_NOTIFY_PROC,         notifyProc,
			      PANEL_LIST_DISPLAY_ROWS,   displayedRows,
			      PANEL_LIST_ROW_HEIGHT,	 rowHeight,
			      NULL);
    }
  }
  // Else, row height was not specified...
  else {
    // If a label font was specified...
    if (labelFont) {
      // Create the XView label font object...
      ((UIObject *) labelFont)->createObject (parent);
      
      // If read only was specified...
      if (readOnly)
	owObject = xv_create (parent->getXViewObject (), PANEL_LIST,
			      PANEL_LABEL_FONT, labelFont->getXViewObject (),
			      PANEL_NOTIFY_PROC,         notifyProc,
			      PANEL_READ_ONLY,	         TRUE,
			      PANEL_LIST_DISPLAY_ROWS,   displayedRows,
			      NULL);
      // Else, read only was not specified...
      else
	owObject = xv_create (parent->getXViewObject (), PANEL_LIST,
			      PANEL_LABEL_FONT, labelFont->getXViewObject (),
			      PANEL_NOTIFY_PROC,         notifyProc,
			      PANEL_LIST_DISPLAY_ROWS,   displayedRows,
			      NULL);
    }
    // Else, no label font specified
    else {
      // If read only was specified...
      if (readOnly)
	owObject = xv_create (parent->getXViewObject (), PANEL_LIST,
			      PANEL_NOTIFY_PROC,         notifyProc,
			      PANEL_READ_ONLY,	         TRUE,
			      PANEL_LIST_DISPLAY_ROWS,   displayedRows,
			      NULL);
      // Else, read only was not specified...
      else
	owObject = xv_create (parent->getXViewObject (), PANEL_LIST,
			      PANEL_NOTIFY_PROC,         notifyProc,
			      PANEL_LIST_DISPLAY_ROWS,   displayedRows,
			      NULL);
    }
  }
  
  // Save the "this" pointer as key data
  xv_set (owObject, XV_KEY_DATA, UIC_OBJECT_KEY, this, NULL);
  
  // Set the XView help data
  setHelpData ();
  
  xv_set (owObject, PANEL_CHOOSE_NONE, TRUE, NULL);

  if (xLoc != UIC_NOT_SET)
    xv_set (owObject, XV_X, xLoc, NULL);
 
  if (yLoc != UIC_NOT_SET)
    xv_set (owObject, XV_Y, yLoc, NULL);
  
  if (width != UIC_NOT_SET)
    xv_set (owObject, PANEL_LIST_WIDTH, width, NULL);
  
  if (height != UIC_NOT_SET)
    xv_set (owObject, XV_HEIGHT, height, NULL);
  
  if (valueX != UIC_NOT_SET)
    xv_set (owObject, PANEL_VALUE_X, valueX, NULL);
  
  if (valueY != UIC_NOT_SET)
    xv_set (owObject, PANEL_VALUE_Y, valueY, NULL);
  
  if (foregroundColor != UIC_NOT_SET)
    xv_set (owObject, PANEL_ITEM_COLOR, foregroundColor, NULL);
  
  if (label != (char *) NULL)
    xv_set (owObject, PANEL_LABEL_STRING, label, NULL);
  else if (imageLabel) {
    ((UIObject *) imageLabel)->createObject (parent);
    xv_set (owObject,
	    PANEL_LABEL_IMAGE, imageLabel->getXViewObject (),
	    NULL);
  }

  if (labelPosition == ABOVE)
    xv_set (owObject, PANEL_LAYOUT, PANEL_VERTICAL, NULL);
  
  if (listMenu) {
    ((UIObject *) listMenu)->createObject (parent);
    xv_set (owObject, PANEL_ITEM_MENU, listMenu->getXViewObject (), NULL);
  }
  
  // If a font is specified...
  if (listFont) {
    // Create a temporary XView font object
    ((UIObject *) listFont)->createObject (parent);
    
    // Get the font family and size
    char *family = listFont->getFamily ();
    int   size = listFont->getSize ();

    // Delete the temporary XView font
    xv_destroy_safe (listFont->getXViewObject ());
    
    /* Recreate the UIC font with the style hard-coded as "normal"
       
       This is done because the ScrollChoice uses bold fonts to
       to show selection feedback.  Therefore, the user is not allowed
       use bold fonts for list strings. */
    listFont = new CharacterFont (family, FONT_STYLE_NORMAL, size);

    delete family;
  }
  // Else, no font specified, create a default UIC font...
  else
    listFont = new CharacterFont (FONT_FAMILY_DEFAULT,
				  FONT_STYLE_NORMAL,
				  FONT_SIZE_DEFAULT);
  
  // The XV_SHOW setting should be done before items are placed in the list
  if (!showFlag) xv_set (owObject, XV_SHOW, FALSE, NULL);
  
  // Create the XView font object
  ((UIObject *) listFont)->createObject (parent);
  
  ListClientData *data;

  // Loop through the internal choices list...
  int num_choices = choices.getSize ();
  for (int i = 0; i < num_choices; i++) {
    // Insert an entry into the panel list
    xv_set (owObject, PANEL_LIST_INSERT, i,
	              PANEL_LIST_FONT,   i, listFont->getXViewObject (),
	    NULL);
    
    // If a string is specified for this entry...
    if ((char *) choices [i]) {
      xv_set (owObject, PANEL_LIST_STRING, i, (char *) choices [i], NULL);
      
      // After the string is copied into the list, it is not needed internally
      delete ((char *) choices [i]);
      choices [i] = (Generic *) NULL;
    }
    // No string specified...
    else
      // Insert a buffer string for showing feedback in the future ("<-")
      xv_set (owObject, PANEL_LIST_STRING, i, "  ", NULL);
    
    // If an image is specified for this entry...
    if ((Image *) images [i]) {
      ((UIObject *) images [i])->createObject (parent);
      xv_set (owObject,
	      PANEL_LIST_GLYPH, i, ((Image *) images [i])->getXViewObject (),
	      NULL);
    }

    /* All entries have client data that is used to provide a unique
       ID for the row and to store the UIC image associated with the row */
    data = new ListClientData (++rowID, (Image *) images [i]); 
    xv_set (owObject, PANEL_LIST_CLIENT_DATA, i, data, NULL);
  }
  
  /* This sets the event proc which handles all of the user interaction
     with the ScrollChoice object */
  xv_set (owObject, PANEL_EVENT_PROC, scrollChoiceEventProc, NULL);
  
  // Specifies the events to consume
  int num_events = events.getSize ();
  for (i = 0; i < num_events; i++)
    xv_set (owObject, WIN_CONSUME_EVENT, (int) events [i], NULL);
  
  /* If there are no user specified pre-selected item and a selection is
     required, set the first item to be selected */
  int num_selected = selections.getSize ();
  if ((num_selected == 0) &&
      selectionRequired &&
      (num_choices > 0))
    setSelectedChoice (1, TRUE);
  // Else, set the pre-selections...
  else
    for (i=0; i < num_selected; i++)
      setSelectedChoice (((int) selections [i]) + 1, TRUE);
  
  /* Clear the internal lists that are no longer used.

     NOTE: The ScrollChoice still uses the "selections" list. */
  choiceColors.dropItems ();
  choices.dropItems ();
  images.dropItems ();
}
