/*H+ Scroller.cc
*
*    SCCS ID: @(#)Scroller.cc 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Defines the member functions of the Scroller class.
*
*    AUTHOR: Joe Warzecha and Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review. 
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)Scroller.cc 1.1 91/08/02 SMI"; 
#endif

// Includes
#include "Scroller.h"
#include "ColorMap.h"

/*F+ Scroller::Scroller ()
*
*    PURPOSE:
*
*    Scroller constructor.  Default direction is VERTICAL.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Scroller::Scroller () : ()
{
  orientation = VERTICAL;
  setScrollerDefaults ();
}

/*F+ Scroller::Scroller (Orientation direction)
*
*    PURPOSE:
*
*    Scroller constructor.
*
*    direction - Specifies the orientationof the Scroller, VERTICAL or
*                HORIZONTAL.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Scroller::Scroller (Orientation direction) : ()
{
  orientation = direction;
  setScrollerDefaults ();
}

/*F+ void Scroller::setScrollerDefaults ()
*
*    PURPOSE:
*
*    Scroller constructor.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void Scroller::setScrollerDefaults ()
{
  scrollAmount = UIC_NOT_SET;
  viewLength = UIC_NOT_SET;
  pageLength = UIC_NOT_SET;
  objectLength = UIC_NOT_SET;
  currentLocation = UIC_NOT_SET;
  splittable = FALSE;
}

/*F+ Scroller::~Scroller ()
*
*    PURPOSE:
*
*    Scroller destructor.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Scroller::~Scroller ()
{
}

/*F+ Boolean Scroller::isScroller ()
*
*    PURPOSE:
*
*    Always returns TRUE (because this is a Scroller).
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Boolean Scroller::isScroller ()
{
  return TRUE;
}

/*F+ void Scroller::createObject (UIObject *parent)
*
*    PURPOSE:
*
*    Creates the XView object of the Scroller.
*
*    PARAMETERS:
*
*    parent - The parent of the Scroller or who the Scroller is attached to.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void Scroller::createObject (UIObject *parent)
{
  if (orientation == VERTICAL)
    owObject = xv_create (parent->getXViewObject (), SCROLLBAR,
			  SCROLLBAR_DIRECTION, SCROLLBAR_VERTICAL,
			  NULL);
  else
    owObject = xv_create (parent->getXViewObject (), SCROLLBAR,
			  SCROLLBAR_DIRECTION, SCROLLBAR_HORIZONTAL,
			  NULL);
  
  // Set the XView help data
  setHelpData ();
  
  if (foregroundColor != UIC_NOT_SET)
    xv_set (owObject,
	    WIN_CMS,              colorMap->getXViewObject (),
	    WIN_FOREGROUND_COLOR, foregroundColor,
	    NULL);
  
  if (backgroundColor != UIC_NOT_SET)
    xv_set (owObject,
	    WIN_CMS,              colorMap->getXViewObject (),
	    WIN_BACKGROUND_COLOR, backgroundColor,
	    NULL);
    
  if (scrollAmount != UIC_NOT_SET)
    xv_set (owObject, SCROLLBAR_PIXELS_PER_UNIT, scrollAmount, NULL);
  
  if (viewLength != UIC_NOT_SET)
    xv_set (owObject, SCROLLBAR_VIEW_LENGTH, viewLength, NULL);
  
  if (pageLength != UIC_NOT_SET)
    xv_set (owObject, SCROLLBAR_PAGE_LENGTH, pageLength, NULL);
  
  if (objectLength != UIC_NOT_SET)
    xv_set (owObject, SCROLLBAR_OBJECT_LENGTH, objectLength, NULL);
  
  if (splittable)
    xv_set (owObject, SCROLLBAR_SPLITTABLE, TRUE, NULL);

  xv_set (owObject, SCROLLBAR_INACTIVE, (active ? FALSE : TRUE), NULL);
}

#ifdef TEST
#endif /* TEST */
