/*H+ TextDisplay.cc
*
*    SCCS ID: @(#)TextDisplay.cc 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Defines the member functions of the TextDisplay class.
*
*    AUTHOR: Joe Warzecha and Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review. 
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)TextDisplay.cc 1.1 91/08/02 SMI"; 
#endif

// Includes
#include "CharacterFont.h"
#include "TextDisplay.h"

/*F+ TextDisplay::TextDisplay ()
*
*    PURPOSE:
*
*    TextDisplay constructor.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
TextDisplay::TextDisplay () : ()
{
  file = (char *) NULL;
  readOnly = TRUE;
  modified = FALSE;
}

/*F+ TextDisplay::~TextDisplay ()
*
*    PURPOSE:
*
*    TextDisplay destructor.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
TextDisplay::~TextDisplay ()
{
  if (file)
    delete file;
}

/*F+ TextDisplayStatus TextDisplay::loadFile (char *file_name)
*
*    PURPOSE:
*
*    Loads the specified file into the TextDisplay.
*
*    RETURN VALUES:
*
*    Returns the status of the TextDisplay after the attempted file load.
*    (See TEXTSW_STATUS in the XView manual).
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
TextDisplayStatus TextDisplay::loadFile (char *file_name)
{
  loadStatus = TEXTSW_STATUS_OKAY;
    
  // Make a copy of the file name
  if (file) delete file;
  file = strcpy (new char [strlen (file_name) + 1], file_name);

  // If the XView object exists...
  if (owObject)
    xv_set (owObject,
	    TEXTSW_STATUS, &loadStatus,
	    TEXTSW_FILE,   file,
	    TEXTSW_FIRST,  0,
	    NULL);

  return (loadStatus);
}

/*F+ char *TextDisplay::getDragFileName ()
*
*    PURPOSE:
*
*    Gets the name of the file that was drag and dropped into the
*    the TextDisplay.
*
*    RETURN VALUES:
*
*    Returns the name of the file.  The memory is allocated on the heap,
*    so the caller must delete it.  If no file was drag and dropped into
*    the TextDisplay, NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
char *TextDisplay::getDragFileName ()
{
  DragMessage   *msg = (DragMessage *)
                       xv_get (textsw_first (owObject), WIN_MESSAGE_DATA);
  Display       *display = (Display *)
                           XV_DISPLAY_FROM_WINDOW (textsw_first (owObject));
  long           long_offset = 0L;
  long           long_length = (long) (MAXPATHLEN / 4);
  int            delete_flag = TRUE;
  Atom           actual_type;
  int            actual_format;
  u_long         nitems;
  u_long         bytes_after;
  unsigned int  *value;
  char	        *return_value;
  
  if (msg == NULL || msg->property == None)
    return (char *) NULL;

  if (XGetWindowProperty (display, msg->window, msg->property,
			  long_offset, long_length, delete_flag,
			  AnyPropertyType, &actual_type, &actual_format,
			  &nitems, &bytes_after, &value) != Success)
    return (char *) NULL;
  
  return_value = strcpy (new char [strlen ((char *) value) + 1],
			 (char *) value);
  XFree ((char *) value);
  return (return_value);
}

/*F+ void TextDisplay::createObject (UIObject *parent)
*
*    PURPOSE:
*
*    Creates the TextDisplay XView object.
*
*    PARAMETER:
*
*    parent - The parent of the TextDisplay.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void TextDisplay::createObject (UIObject *parent)
{
  // If a font is specified...
  if (font) {
    // Create the XView font object
    ((UIObject *) font)->createObject (parent);

    if (border == TRUE)
      owObject = xv_create (parent->getXViewObject (), TEXTSW, 
			    OPENWIN_SHOW_BORDERS,      TRUE,
			    XV_FONT, font->getXViewObject (),
			    NULL);
    else
      owObject = xv_create (parent->getXViewObject (), TEXTSW,
			    XV_FONT, font->getXViewObject (),
			    NULL);
  }
  // Else, no font specified...
  else {
    if (border == TRUE)
      owObject = xv_create (parent->getXViewObject (), TEXTSW, 
			    OPENWIN_SHOW_BORDERS,      TRUE,
			    NULL);
    else
      owObject = xv_create (parent->getXViewObject (), TEXTSW, NULL);
  }
  
  /* Save the "this" pointer as key data on both the TermSW object and
     its first view (textsw_first) */
  xv_set (owObject, XV_KEY_DATA, UIC_OBJECT_KEY, this, NULL);
  xv_set (textsw_first (owObject), XV_KEY_DATA, UIC_OBJECT_KEY, this, NULL);

  // Save the XView frame
  xv_set (owObject,
	  XV_KEY_DATA, UIC_FRAME_KEY, parent->getXViewObject (),
	  NULL);
  
  // Set the XView help data
  setHelpData ();
  
  if (foregroundColor != UIC_NOT_SET)
    xv_set (owObject,
	    WIN_FOREGROUND_COLOR, foregroundColor,
	    NULL);
  
  if (backgroundColor != UIC_NOT_SET)
    xv_set (owObject,
	    WIN_BACKGROUND_COLOR, backgroundColor,
	    NULL);

  if (xLoc != UIC_NOT_SET)
    xv_set (owObject, XV_X, xLoc, NULL);

  if (yLoc != UIC_NOT_SET)
    xv_set (owObject, XV_Y, yLoc, NULL);
  
  if (extendWidth)
    xv_set (owObject, XV_WIDTH, WIN_EXTEND_TO_EDGE, NULL);
  else if (width != UIC_NOT_SET)
    xv_set (owObject, XV_WIDTH, width, NULL);
  
  if (extendHeight)
    xv_set (owObject, XV_HEIGHT, WIN_EXTEND_TO_EDGE, NULL);
  else if (height != UIC_NOT_SET)
    xv_set (owObject, XV_HEIGHT, height, NULL);
  
  if (readOnly == TRUE)
    xv_set (owObject, TEXTSW_BROWSING, TRUE, NULL);
  
  if (file != (char *) NULL)
    xv_set (owObject,
	    TEXTSW_STATUS, &loadStatus,
	    TEXTSW_FILE,   file,
	    TEXTSW_FIRST,  1,
	    NULL);
  
  // If the user specified an event handler...
  if (eventHandler || eventTargets.getSize ()) {
    int noEvents = events.getSize (); 
    for (int i = 0; i < noEvents; i++) 
      xv_set (textsw_first (owObject),
	      WIN_CONSUME_EVENT, (int) events [i],
	      NULL);
  
    notify_interpose_event_func (textsw_first (owObject),
				 (Notify_func) eventProc, 
				 NOTIFY_SAFE);
  }
  
  xv_set (owObject, XV_SHOW, showFlag, NULL);
}
       
#ifdef TEST
#endif /* TEST */
