/*H+ UIComponent.cc
*
*    SCCS ID: @(#)UIComponent.cc 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Defines the member functions of the UIComponent class.
*
*    AUTHOR: Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review. 
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)UIComponent.cc 1.1 91/08/02 SMI"; 
#endif

// Includes
#include "CharacterFont.h"
#include "ColorMap.h"
#include "UIComponent.h"

/*F+ UIComponent::UIComponent ()
*
*    PURPOSE:
*
*    Constructor of the UIComponent class.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
UIComponent::UIComponent () : ()
{
  setUIComponentDefaults ();
}

/*F+ UIComponent::UIComponent (char *string_label)
*
*    PURPOSE:
*
*    Constructor of the UIComponent class.
*
*    PARAMETERS:
*
*    string_label - Specifies a string for the UIComponent label.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
UIComponent::UIComponent (char *string_label) : (string_label)
{
  setUIComponentDefaults ();
}

/*F+ UIComponent::UIComponent (Image &image_label)
*
*    PURPOSE:
*
*    Constructor of the UIComponent class.
*
*    PARAMETERS:
*
*    image_label - Specifies an image for the UIComponent label.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
UIComponent::UIComponent (Image &image_label) : ()
{
  setUIComponentDefaults ();
  imageLabel = &image_label;
}

/*F+ UIComponent::setUIComponentDefaults ()
*
*    PURPOSE:
*
*    Initializes the data members of the UIComponent class.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::setUIComponentDefaults ()
{
  valueX = UIC_NOT_SET;
  valueY = UIC_NOT_SET;
  active = TRUE;
  labelPosition = LEFT;
  notifyHandler = UIC_NO_FUNCTION;
  notifyResult = UIC_NOT_SET;
  imageLabel = (Image *) NULL;
  labelFont = (CharacterFont *) NULL;
  valueFont = (CharacterFont *) NULL;
}

/*F+ UIComponent::~UIComponent ()
*
*    PURPOSE:
*
*    Destructor of the UIComponent class.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
UIComponent::~UIComponent ()
{
}

/*F+ void UIComponent::setLabel (char *string_label)
*
*    PURPOSE:
*
*    Specifies the string to use for the object's label.
*
*    PARAMETERS
*
*    string_label - The string to use.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::setLabel (char *string_label)
{
  if (label != (char *) NULL)
    delete label;

  label = strcpy (new char [strlen (string_label) + 1], string_label);
  
  if (owObject) xv_set (owObject, PANEL_LABEL_STRING, label, NULL);
}

/*F+ void UIComponent::setLabel (Image &_label)
*
*    PURPOSE:
*
*    Specifies the image to use for the object's label.
*
*    PARAMETERS
*
*    image_label - The image to use.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::setLabel (Image &image_label)
{
  imageLabel = &image_label;
  if (owObject) {
    ((UIObject *) imageLabel)->createObject (this);
    xv_set (owObject,
	    PANEL_LABEL_IMAGE, imageLabel->getXViewObject (),
	    NULL);
  }
}

/*F+ void UIComponent::setLabelPosition (Position position)
*
*    PURPOSE:
*
*    Specifies the position of the label relative to the object.
*
*    PARAMETERS
*
*    position - The position of the label (ABOVE or LEFT).
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::setLabelPosition (Position position)
{
  labelPosition = position;

  if (owObject) {
    if (labelPosition == ABOVE)
      xv_set (owObject, PANEL_LAYOUT, PANEL_VERTICAL, NULL);
    else
      xv_set (owObject, PANEL_LAYOUT, PANEL_HORIZONTAL, NULL);
  }
}

/*F+ void UIComponent::setForegroundColor (Color color)
*
*    PURPOSE:
*
*    Sets the foreground color of the object.
*
*    PARAMETERS:
*
*    color - A UIC color structure containing the RGB components of the color.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::setForegroundColor (Color color)
{
  if (!colorMap) {
    useDefaultColorMap ();
    
    if (owObject) {
      ((UIObject *) colorMap)->createObject (this);
      xv_set (owObject, WIN_CMS, colorMap->getXViewObject(), NULL);
    }
  }
    
  foregroundColor = colorMap->findColor (color);
    
  if (owObject)
    xv_set (owObject, PANEL_ITEM_COLOR, foregroundColor, NULL);
}

/*F+ void UIComponent::setForegroundColor (char *color)
*
*    PURPOSE:
*
*    Sets the foreground color of the object.
*
*    PARAMETERS:
*
*    color - The name of the color.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::setForegroundColor (char *color)
{
  if (!colorMap) {
    useDefaultColorMap ();

    if (owObject) {
      ((UIObject *) colorMap)->createObject (this);
      xv_set (owObject, WIN_CMS, colorMap->getXViewObject(), NULL);
    }
  }
    
  foregroundColor = colorMap->findColor (color);
    
  if (owObject)
    xv_set (owObject, PANEL_ITEM_COLOR, foregroundColor, NULL);
}

/*F+ int UIComponent::addNotifyTarget (UIObject          &object,
*                                      UICTargetFunction  func)
*
*    PURPOSE:
*
*    Adds a target to the list of notify targets for this object.
*
*    PARAMETERS:
*
*    object - The target object.
*    func   - The target function.
*
*    RETURN VALUES:
*
*    Returns the position in the target list for this object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int UIComponent::addNotifyTarget (UIObject &object, UICTargetFunction func)
{
  UITarget *target = new UITarget;

  target->targetObject = &object;
  target->targetFunction = func;

  notifyTargets.addItem (target);
  return notifyTargets.getSize ();
}

/*F+ void UIComponent::notifyNotifyTargets ()
*
*    PURPOSE:
*
*    Internal function that loops through all of the notify targets
*    associated with this component and calls the corresponding
*    target functions.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::notifyNotifyTargets ()
{
  for (int i = notifyTargets.getSize () - 1; i > -1; i--) {
    UITarget *target = (UITarget *) notifyTargets [i];

    if (target->targetFunction)
      (*target->targetFunction) (this, target->targetObject);
  }
}

/*F+ void UIComponent::setNotifyHandler ()
*
*    PURPOSE:
*
*    Specifies the notify handler.
*
*    PARAMETERS:
*
*    handler - The user defined notify handler.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::setNotifyHandler (UICHandlerFunction handler)
{
  notifyHandler = handler;
}

/*F+ void UIComponent::eventProc (Panel_item item, Event *event)
*
*    PURPOSE:
*
*    Internal XView event proc wrapper.  This is called whenever an
*    event occurs on a component.
*
*    PARAMETERS:
*
*    item  - The item the event occurred on.
*    event - The XView event information structure.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::eventProc (Panel_item item, Event *event)
{
  UIComponent *object = (UIComponent *) xv_get (item,
					        XV_KEY_DATA,
						UIC_OBJECT_KEY);

#ifdef OWNV3
  /* In XView V3, some of the components are compund items and the event
     may occur on the child item instead of the parent item.  If this
     happens, the parent (XV_OWNER) must be obtained in order to
     obtain a "this" pointer */
  if (!object) {
    Panel_item compound_item = xv_get (item, PANEL_ITEM_OWNER);
    object = (UIComponent *)
             xv_get (compound_item, XV_KEY_DATA, UIC_OBJECT_KEY);
  }
#endif

  object->setEventInformation (event);
  
  // Call the user specified event handler
  if (object->eventHandler)
    (*object->eventHandler) (object);
  
  object->resetEventInformation ();
  panel_default_handle_event (item, event);
}

/*F+ void UIComponent::createWithFonts (UIObject *parent,
*                                       Xv_pkg   *package,
*                                       void     *terminator)
*
*    PURPOSE:
*
*    Internal function for creating an XView object that sets the values
*    of fonts in the xv_create call.  This function allows the setting of
*    PANEL_LABEL_FONT and PANEL_VALUE_FONT.
*
*    PARAMETERS:
*
*    parent     - The UIC parent of the object.
*    package    - The XView package that is being created.
*    terminator - Argument list terminator (allows compatibility with
*                 the use of var args in the future).
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::createWithFonts (UIObject *parent,
				   Xv_pkg   *package,
				   void     *terminator)
{
  if (labelFont)
    ((UIObject *) labelFont)->createObject (parent);

  if (valueFont)
    ((UIObject *) valueFont)->createObject (parent);

  if (labelFont && valueFont)
    owObject = xv_create (parent->getXViewObject (), package,
			  PANEL_LABEL_FONT, labelFont->getXViewObject (),
			  PANEL_VALUE_FONT, valueFont->getXViewObject (),
			  NULL);
  else if (labelFont)
    owObject = xv_create (parent->getXViewObject (), package,
			  PANEL_LABEL_FONT, labelFont->getXViewObject (),
			  NULL);
  else if (valueFont)
    owObject = xv_create (parent->getXViewObject (), package,
			  PANEL_VALUE_FONT, valueFont->getXViewObject (),
			  NULL);
  else
    owObject = xv_create (parent->getXViewObject (), package, NULL);
}

/*F+ void UIComponent::createWithFonts (UIObject              *parent,
*                                       Xv_pkg                *package,
*                                       Panel_attr             attribute1,
*                                       Panel_attribute_value  value1,
*                                       Panel_attr             attribute2,
*                                       Panel_attribute_value  value2,
*                                       void                  *terminator)
*
*    PURPOSE:
*
*    Internal function for creating an XView object that sets the values
*    of fonts in the xv_create call.  This function allows the setting of
*    PANEL_LABEL_FONT and PANEL_VALUE_FONT.
*
*    PARAMETERS:
*
*    parent     - The UIC parent of the object.
*    package    - The XView package that is being created.
*    attribute1 - An XView attribute name.
*    value1     - The value of the XView attribute.
*    attribute2 - Another XView attribute name.
*    value2     - The value of the second XView attribute.
*    terminator - Argument list terminator (allows compatibility with
*                 the use of var args in the future).
*
*    REVISION HISTORY:
*
*    Who       When       Why
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::createWithFonts (UIObject 		 *parent,
                                   Xv_pkg   		 *package,
                                   Panel_attr             attribute1,
                                   Panel_attribute_value  value1,
                                   Panel_attr             attribute2,
                                   Panel_attribute_value  value2,
                                   void     		 *terminator)
{
  if (labelFont)
    ((UIObject *) labelFont)->createObject (parent);
 
  if (valueFont)
    ((UIObject *) valueFont)->createObject (parent);
 
  if (labelFont && valueFont)
    owObject = xv_create (parent->getXViewObject (), package,
			  attribute1,       value1,
			  attribute2,       value2,
                          PANEL_LABEL_FONT, labelFont->getXViewObject (),
                          PANEL_VALUE_FONT, valueFont->getXViewObject (),
                          NULL);
  else if (labelFont)
    owObject = xv_create (parent->getXViewObject (), package,
			  attribute1,       value1,
			  attribute2,       value2,
                          PANEL_LABEL_FONT, labelFont->getXViewObject (),
                          NULL);
  else if (valueFont)
    owObject = xv_create (parent->getXViewObject (), package,
			  attribute1,       value1,
			  attribute2,       value2,
                          PANEL_VALUE_FONT, valueFont->getXViewObject (),
                          NULL);
  else
    owObject = xv_create (parent->getXViewObject (), package,
			  attribute1, value1,
			  attribute2, value2,
			  NULL);
}    
 
/*F+ UIComponent::setUIComponentAttributes ()
*
*    PURPOSE:
*
*    Sets common UIComponent XView attributes that are required for creating
*    the XView object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::setUIComponentAttributes (UIObject *parent)
{
  xv_set (owObject, XV_KEY_DATA, UIC_OBJECT_KEY, this, NULL);

  setHelpData ();

  if (xLoc != UIC_NOT_SET)
    xv_set (owObject, XV_X, xLoc, NULL);
  
  if (yLoc != UIC_NOT_SET)
    xv_set (owObject, XV_Y, yLoc, NULL);
  
  if (width != UIC_NOT_SET)
    xv_set (owObject, XV_WIDTH, width, NULL);

  if (height != UIC_NOT_SET)
    xv_set (owObject, XV_HEIGHT, height, NULL);
  
  if (valueX != UIC_NOT_SET)
    xv_set (owObject, PANEL_VALUE_X, valueX, NULL);
  
  if (valueY != UIC_NOT_SET)
    xv_set (owObject, PANEL_VALUE_Y, valueY, NULL);
  
  if (foregroundColor != UIC_NOT_SET)
    xv_set (owObject, PANEL_ITEM_COLOR, foregroundColor, NULL);
  
  if (label != (char *) NULL)
    xv_set (owObject, PANEL_LABEL_STRING, label, NULL);
  else if (imageLabel) {
    ((UIObject *) imageLabel)->createObject (parent);
    xv_set (owObject,
	    PANEL_LABEL_IMAGE, imageLabel->getXViewObject (),
	    NULL);
  }
  
  // If the user specified an event handler...
  if (eventHandler || eventTargets.getSize ()) {
    xv_set (owObject, PANEL_EVENT_PROC, eventProc, NULL);
    int noEvents = events.getSize ();
    for (int i = 0; i < noEvents; i++)
      xv_set (owObject, WIN_CONSUME_EVENT, (int) events [i], NULL); 
  }
  
  if (!showFlag) xv_set (owObject, XV_SHOW, FALSE, NULL);
}

/*F+ Boolean UIComponent::isScroller ()
*
*    PURPOSE:
*
*    Determines whether or not this object is a Scroller.
*
*    RETURN VALUES:
*
*    Returns TRUE if it is a Scroller.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Boolean UIComponent::isScroller ()
{
  return (FALSE);
}

/*F+ Boolean UIComponent::isDrawableOnCanvas ()
*
*    PURPOSE:
*
*    Determines whether or not this object is drawable on an XView canvas.
*
*    RETURN VALUES:
*
*    Returns TRUE if it is drawable on an XView canvas.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Boolean UIComponent::isDrawableOnCanvas ()
{
  return (FALSE);
}

/*F+ Boolean UIComponent::repaintComponent ()
*
*    PURPOSE:
*
*    This function is called to repaint the component if this is a
*    user defined component.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::repaintComponent (ComponentDisplay *cd)
{
}

/*F+ Boolean UIComponent::componentHotRegionEventProc (ComponentDisplay *cd)
*
*    PURPOSE:
*
*    This function is called if an event occurs in the hot region associated
*    with this component if this is a user defined component.
*
*    PARAMETERS:
*
*    cd - A pointer to the parent ComponentDisplay.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::componentHotRegionEventProc (ComponentDisplay *cd)
{
}

/*F+ Boolean UIComponent::componentHotRegionEventProc (ComponentDisplay *cd)
*
*    PURPOSE:
*
*    This function is called if an event occurs outside of the hot region
*    associated with this component, but inside of the components parent
*    ComponentDisplay object, if this is a user defined component.
*
*    PARAMETERS:
*
*    cd - A pointer to the parent ComponentDisplay.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIComponent::componentBackgroundEventProc (ComponentDisplay *cd)
{
}

#ifdef TEST
#endif /* TEST */
