/*H+ UIDisplay.cc
*
*    SCCS ID: @(#)UIDisplay.cc 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Defines the member functions of the UIDisplay class.
*
*    AUTHOR: Joe Warzecha and Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review. 
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)UIDisplay.cc 1.1 91/08/02 SMI"; 
#endif

// Includes
#include "UIDisplay.h"

/*F+ UIDisplay::UIDisplay ()
*
*    PURPOSE:
*
*    UIDisplay constructor.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
UIDisplay::UIDisplay  ()
{
  extendHeight = FALSE;
  extendWidth = FALSE;
  border = FALSE;
};

/*F+ UIDisplay::~UIDisplay ()
*
*    PURPOSE:
*
*    UIDisplay destructor.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
UIDisplay::~UIDisplay ()
{
}

/*F+ void UIDisplay::setExtendToEdge (Dimension dimension)
*
*    PURPOSE:
*
*    Specifies that the display object should be extended to edge
*    of its parent window in the direction of the specified dimension.
*
*    PARAMETERS:
*
*    dimension - The dimension to extend the edge (WIDTH or HEIGHT).
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIDisplay::setExtendToEdge (Dimension dimension)
{
  if (dimension == HEIGHT) {
    extendHeight = TRUE;

    if (owObject)
      xv_set (owObject, XV_HEIGHT, WIN_EXTEND_TO_EDGE, NULL);
  }
  else {
    extendWidth = TRUE;

    if (owObject)
      xv_set (owObject, XV_WIDTH, WIN_EXTEND_TO_EDGE, NULL);
  }
}

/*F+ Notify_value UIDisplay::eventProc (Xv_window          win,
*                                       Event             *event,
*				        Notify_arg         arg,
*				        Notify_event_type  type)
*
*    PURPOSE:
*
*    The XView display event proc.  This is called when ever an
*    event occurs on a display object.
*
*    PARAMETERS:
*
*    win   - The XView object the event occurred on.
*    event - The XView event info structure.
*    arg   - XView client data.
*    type  - The XView notifier event type: SAFE or IMMEDIATE.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Notify_value UIDisplay::eventProc (Xv_window          win,
				   Event             *event,
				   Notify_arg         arg,
				   Notify_event_type  type)
{
  UIDisplay *object = (UIDisplay *) xv_get (win, XV_KEY_DATA, UIC_OBJECT_KEY);
  
  // Set the UIC event information
  object->setEventInformation (event);
  
  // Call the user specified event handler
  if (object->eventHandler)
    (*object->eventHandler) (object);
  
  // Erase the UIC event information
  object->resetEventInformation ();

  // If the user specified an event result...
  if (object->eventResultSet) {
    object->eventResultSet = FALSE;

    // Return the specified result to the notifier
    return ((Notify_value) object->eventResult);
  }

  // Tell the notifier to handle the next event function in its list
  return (notify_next_event_func (win, (Notify_event) event, arg, type));
}

/*F+ char *UIDisplay::getDragFileName ()
*
*    PURPOSE:
*
*    Gets the name of the file that was drag and dropped into the
*    the TextDisplay.
*
*    RETURN VALUES:
*
*    Returns the name of the file.  The memory is allocated on the heap,
*    so the caller must delete it.  If no file was drag and dropped into
*    the TextDisplay, NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
char *UIDisplay::getDragFileName ()
{
  DragMessage   *msg = (DragMessage *) xv_get(owObject, WIN_MESSAGE_DATA);
  Display       *display = (Display *) XV_DISPLAY_FROM_WINDOW (owObject);
  long           long_offset = 0L;
  long           long_length = (long) (MAXPATHLEN / 4);
  int            delete_flag = TRUE;
  Atom           actual_type;
  int            actual_format;
  u_long         nitems;
  u_long         bytes_after;
  unsigned int  *value;
  char	        *return_value;
  
  if (msg == NULL || msg->property == None)
    return (char *) NULL;

  if (XGetWindowProperty (display, msg->window, msg->property,
			  long_offset, long_length, delete_flag,
			  AnyPropertyType, &actual_type, &actual_format,
			  &nitems, &bytes_after, &value) != Success)
    return (char *) NULL;
  
  return_value = strcpy (new char [strlen ((char *) value) + 1],
			 (char *) value);
  XFree ((char *) value);
  return (return_value);
}
       
#ifdef TEST
#endif /* TEST */
