/*H+ UIObject.cc
*
*    SCCS ID: @(#)UIObject.cc 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Defines the member functions of the UIObject class.
*
*    AUTHOR: Joe Warzecha and Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review. 
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)UIObject.cc 1.1 91/08/02 SMI"; 
#endif

// Includes
#include <ctype.h>
#include <memory.h>

#include "UIObject.h"
#include "ColorMap.h"
#include "CharacterFont.h"
#include "HotRegion.h"

#ifndef NO_XVPS
#ifdef OWNV3
#include <wire/wire.h>
#endif
#endif

// Constants

// Defines the version of the UIC
const char *uic_version = UICV1;

// Defines the default colors available to all UIC objects
const char *uic_colors [] = {
  "Aquamarine", "Blue", "Blue Violet", "Brown",
  "Cadet Blue", "Coral", "Cornflower Blue", "Cyan",
  "Dark Green", "Dark Olive Green", "Dark Orchid", "Dark Slate Blue",
  "Dark Slate Gray", "Dark Turquoise", "Dim Gray",
  "Firebrick", "Forest Green",
  "Gold", "Goldenrod", "Gray", "Green", "Green Yellow",
  "Indian Red", "Khaki",
  "Light Blue", "Light Gray", "Light Steel Blue", "Lime Green",
  "Magenta", "Maroon", "Medium Aquamarine", "Medium Blue",
  "Medium Orchid", "Medium Sea Green", "Medium Slate Blue",
  "Medium Spring Green", "Medium Turquoise", "Medium Violet Red",
  "Midnight Blue",
  "Navy", "Navy Blue",
  "Olive Drab", "Orange", "Orange Red", "Orchid",
  "Pale Green", "Pink", "Plum", "Red", 
  "Salmon", "Sea Green", "Sienna", "Sky Blue", 
  "Slate Blue", "Spring Green", "Steel Blue", "Tan", 
  "Thistle", "Turquoise",
  "Violet", "Violet Red", 
  "Wheat", "Yellow", "Yellow Green" };

/*F+ UIObject::UIObject ()
*
*    PURPOSE:
*
*    UIObject constructor.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
UIObject::UIObject ()
{
  setUIObjectDefaults ();
}

/*F+ UIObject::UIObject (char *string_label)
*
*    PURPOSE:
*
*    UIObject constructor.
*
*    PARAMETERS:
*
*    string_label - Specifies a string for the label.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
UIObject::UIObject (char *string_label)
{
  setUIObjectDefaults ();
  label = strcpy (new char [strlen (string_label) + 1], string_label);
}

/*F+ void UIObject::setUIObjectDefaults ()
*
*    PURPOSE:
*
*    Initializes the data members of the UIObject class.  Also generates
*    the unique ID every UIC object has.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setUIObjectDefaults ()
{
  // Keep a count of the actual UIC objects currently existing
  numUIObjects++;

  // Generate unique ID
  totalUIObjectsCreated++;
  uiObjectID = totalUIObjectsCreated;

  owObject = XV_NULL;
  label = (char *) NULL;
  xLoc = UIC_NOT_SET;
  yLoc = UIC_NOT_SET;
  width = UIC_NOT_SET;
  height = UIC_NOT_SET;
  help = (char *) NULL;
  foregroundColor = UIC_NOT_SET;
  backgroundColor = UIC_NOT_SET;
  font = (CharacterFont *) NULL;
  fontGC = (GC) NULL;
  eventHandler = UIC_NO_FUNCTION;
  eventResultSet = FALSE;
  objectData = new GenericHash (20);
  showFlag = TRUE;
  resetEventInformation ();
}

/*F+ UIObject::~UIObject ()
*
*    PURPOSE:
*
*    UIObject destructor.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
UIObject::~UIObject ()
{
  // This UIC object no longer exists
  numUIObjects--;

  if (help != (char *) NULL)
    delete help;
  if (label != (char *) NULL)
    delete label;
  
  // If no UIC objects exists any more...
  if (!numUIObjects)
    // Delete the default color map, if necessary
    if (defaultColorMap && colorMap)
      delete colorMap;
  
  /* Drop the items from the events list so that its destructor doesn't
     try to delete the integers */
  events.dropItems ();
  
  // Clear and delete the object data table
  objectData->dropItems ();
  delete objectData;
}

/*F+ const char *UIObject::getVersion ()
*
*    PURPOSE:
*
*    Returns the compatibility version string of the UIC.  Currently
*    all versions of the UIC that return "UICV1" are compatible.
*
*    RETURN VALUES:
*
*    Returns a string constant containing the current UIC version.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
const char *UIObject::getVersion ()
{
  return (uic_version);
}

/*F+ void UIObject::initUI (int &argc, char **argv,
*                           UICAttribute      attribute,
*			    UICAttributeValue value, ...)
*
*    PURPOSE:
*
*    In addition to initializing the graphics server, this function also is
*    used to specify attributes for the current session with the UIC.  The
*    only attribute currently implemented is UIC_OPEN_PS_CONNECTION.
*    UIC_OPEN_PS_CONNECTION can be set to TRUE or FALSE.  If TRUE, then the UIC
*    will open a PostScript connection to the NeWS server.  If set to FALSE,
*    then the UIC will not open the PostScript connection and many of the
*    PostScript member functions provided in the UIC ComponentDisplay class
*    will not work.  The following is an example of specifying the attribute as
*    TRUE:
*
*       	initUI (argc, argv, UIC_OPEN_PS_CONNECTION, TRUE, NULL);
*
*    If the attribute is not to be set, the following will default to the 
*    attribute being set FALSE:
*
*       	initUI (argc, argv);
*
*    PARAMETERS:
*
*    argc      - The argc from main.
*    argv      - The argv from main.
*    attribute - The name of the first UIC attribute.
*    value     - The value of the attribute.
*    ...       - A list of more attributes.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::initUI (int &argc, char **argv,
		       UICAttribute      attribute,
		       UICAttributeValue value, ...)
{ 
  // Initialize the display server and XView
  xv_init (XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

  // Create a temporary XView object for obtaining root window information
  Server_image temp_image = xv_create (NULL, SERVER_IMAGE, NULL);

  // Get the root window (frame buffer) display pointer, width and height
  rootDisplay = (Display *) xv_get (temp_image, XV_DISPLAY);
  rootWindowWidth = DisplayWidth (rootDisplay, 0);
  rootWindowHeight = DisplayHeight (rootDisplay, 0);

  xv_destroy_safe (temp_image);

  // Process UIC attributes

#ifndef NO_XVPS
  // If the PostScript connection attribute is specified...
  if (attribute == UIC_OPEN_PS_CONNECTION)
    if (((Boolean) value) == TRUE) {
      ps_open_PostScript ();
#ifdef OWNV3
      wire_Wire ps_wire = wire_Open (NULL);
      wire_SetCurrent (ps_wire);

      char *news_package = " /NeWS 3 0 findpackage beginpackage ";
      pprintf (PostScript,
	       (unsigned char []) news_package,
	       strlen (news_package));
      ps_flush_PostScript ();
#endif /* OWNV3 */
    }
#endif /* NO_XVPS */
}

/*F+ void UIObject::setEventInformation (Event *event)
*
*    PURPOSE:
*
*    Sets the internal UIC event information, including information
*    relating to what hot regions the event may have occurred in.
*
*    PARAMETERS:
*
*    event - XView event information structure.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setEventInformation (Event *event)
{
  // Save the XView event information
  lastEvent.setXViewEvent (event);

  // Get the X-Y location of the event
  int x = lastEvent.getX ();
  int y = lastEvent.getY ();

  // Loop through the hot regions attached to this object...
  for (int i = hotRegions.getSize () - 1; i > -1; i--) {
    // If this event is in this hot region...
    if (((HotRegion *) hotRegions [i])->isHotRegion (x, y, this)) {
      // This is a hot region event, so make a note of it
      hotRegionEventFlag = TRUE;

      // Add this hot region into the list of "active" hot regions
      activeHotRegions.addItem (hotRegions [i]);

      // If this hot region is not transparent...
      if (!((HotRegion *) hotRegions [i])->isTransparent ())
	// Don't process any more hot regions
	break;
    }
  }

  // Loop through the list of event targets attached to this object
  for (i = eventTargets.getSize () - 1; i > -1; i--) {
    UITarget *target = (UITarget *) eventTargets [i];
	
    /* Call the user specified target function */
    if (target->targetFunction)
      (*target->targetFunction) (this, target->targetObject);
  }
}

/*F+ void UIObject::resetEventInformation ()
*
*    PURPOSE:
*
*    Clears the internal UIC event information.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::resetEventInformation ()
{
  // Clear the XView event structure pointer
  lastEvent.setXViewEvent ((Event *) NULL);

  // Clear the hot region information
  hotRegionEventFlag = FALSE;
  activeHotRegions.dropItems ();
}

/*F+ void UIObject::setLoation (int new_x, int new_y)
*
*    PURPOSE:
*
*    Sets the X-Y location of the object relative to its parent.
*
*    PARAMETERS:
*
*    new_x - The new X coordinate.
*    new_y - The new Y coordinate.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setLocation (int new_x, int new_y)
{ 
  xLoc = new_x;
  yLoc = new_y;

  if (owObject)
    xv_set (owObject, XV_X, xLoc, XV_Y, yLoc, NULL);
}

/*F+ void UIObject::setX (int new_x)
*
*    PURPOSE:
*
*    Sets the X location of the object relative to its parent.
*
*    PARAMETERS:
*
*    new_x - The new X coordinate.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setX (int new_x)
{
  xLoc = new_x;

  if (owObject)
    xv_set (owObject, XV_X, xLoc, NULL);
}

/*F+ void UIObject::setY (int new_y)
*
*    PURPOSE:
*
*    Sets the Y location of the object relative to its parent.
*
*    PARAMETERS:
*
*    new_y - The new Y coordinate.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setY (int new_y)
{ 
  yLoc = new_y;

  if (owObject)
    xv_set (owObject, XV_Y, yLoc, NULL);
}

/*F+ void UIObject::setSize (int new_width, int new_height)
*
*    PURPOSE:
*
*    Sets the size of the object.
*
*    PARAMETERS:
*
*    new_width  - The new width of the object.
*    new_height - The new height of the object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setSize (int new_height, int new_width)
{ 
  height = new_height;
  width = new_width;

  if (owObject)
    xv_set (owObject, XV_WIDTH, width, XV_HEIGHT, height, NULL);
}

/*F+ void UIObject::setWidth (int new_width)
*
*    PURPOSE:
*
*    Sets the width of the object.
*
*    PARAMETERS:
*
*    new_width  - The new width of the object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setWidth (int new_width)
{ 
  width = new_width;

  if (owObject)
    xv_set (owObject, XV_WIDTH, width, NULL);
}

/*F+ void UIObject::setHeight (int new_height)
*
*    PURPOSE:
*
*    Sets the height of the object.
*
*    PARAMETERS:
*
*    new_height - The new height of the object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setHeight (int new_height)
{
  height = new_height;

  if (owObject)
    xv_set (owObject, XV_HEIGHT, height, NULL);
}

/*F+ void UIObject::setForegroundColor (Color color)
*
*    PURPOSE:
*
*    Sets the foreground color of the object.
*
*    PARAMETERS:
*
*    color - A UIC color structure containing the RGB components of the color.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setForegroundColor (Color color)
{
  // If no color map has been specified...
  if (!colorMap) {
    // Use the default UIC color map
    useDefaultColorMap ();

    // If the XView object already exists...
    if (owObject) {
      // Create the XView color map object
      ((UIObject *) colorMap)->createObject (this);

      // Determine the default XView background color
      int bg_index = colorMap->getDefaultBackgroundIndex ();
	
      // Set the color map and default bg color
      xv_set (owObject, WIN_CMS, colorMap->getXViewObject (), NULL);
      xv_set (owObject, WIN_BACKGROUND_COLOR, bg_index, NULL);
    }
  }
  
  // Find the foreground color index
  foregroundColor = colorMap->findColor (color);
  
  if (owObject)
    xv_set (owObject, WIN_FOREGROUND_COLOR, foregroundColor, NULL);
}

/*F+ void UIObject::setForegroundColor (char *color)
*
*    PURPOSE:
*
*    Sets the foreground color of the object.
*
*    PARAMETERS:
*
*    color - The name of the color.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setForegroundColor (char *color)
{
  // If no color map has been specified...
  if (!colorMap) {
    // Use the default UIC color map
    useDefaultColorMap ();

    // If the XView object already exists...
    if (owObject) {
      // Create the XView color map object
      ((UIObject *) colorMap)->createObject (this);

      // Determine the default XView background color
      int bg_index = colorMap->getDefaultBackgroundIndex ();
	
      // Set the color map and default bg color
      xv_set (owObject, WIN_CMS, colorMap->getXViewObject (), NULL);
      xv_set (owObject, WIN_BACKGROUND_COLOR, bg_index, NULL);
    }
  }
  
  // Find the foreground color index
  foregroundColor = colorMap->findColor (color);
  
  if (owObject)
    xv_set (owObject, WIN_FOREGROUND_COLOR, foregroundColor, NULL);
}

/*F+ void UIObject::setBackgroundColor (Color color)
*
*    PURPOSE:
*
*    Sets the background color of the object.
*
*    PARAMETERS:
*
*    color - A UIC color structure containing the RGB components of the color.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setBackgroundColor (Color color)
{
  // If no color map has been specified...
  if (!colorMap) {
    // Use the default UIC color map
    useDefaultColorMap ();

    // If the XView object already exists...
    if (owObject) {
      // Create the XView color map object
      ((UIObject *) colorMap)->createObject (this);

      // Determine the default XView foreground color
      int fg_index = colorMap->getDefaultForegroundIndex ();
	
      // Set the color map and default fg color
      xv_set (owObject, WIN_CMS, colorMap->getXViewObject (), NULL);
      xv_set (owObject, WIN_FOREGROUND_COLOR, fg_index, NULL);
    }
  }
  
  // Find the background color index
  backgroundColor = colorMap->findColor (color);
  
  if (owObject)
    xv_set (owObject, WIN_BACKGROUND_COLOR, backgroundColor, NULL);
}

/*F+ void UIObject::setBackgroundColor (char *color)
*
*    PURPOSE:
*
*    Sets the background color of the object.
*
*    PARAMETERS:
*
*    color - The name of the color.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setBackgroundColor (char *color)
{
  // If no color map has been specified...
  if (!colorMap) {
    // Use the default UIC color map
    useDefaultColorMap ();

    // If the XView object already exists...
    if (owObject) {
      // Create the XView color map object
      ((UIObject *) colorMap)->createObject (this);

      // Determine the default XView foreground color
      int fg_index = colorMap->getDefaultForegroundIndex ();
	
      // Set the color map and default fg color
      xv_set (owObject, WIN_CMS, colorMap->getXViewObject (), NULL);
      xv_set (owObject, WIN_FOREGROUND_COLOR, fg_index, NULL);
    }
  }
  
  // Find the background color index
  backgroundColor = colorMap->findColor (color);
  
  if (owObject)
    xv_set (owObject, WIN_BACKGROUND_COLOR, backgroundColor, NULL);
}

/*F+ void UIObject::setFont (CharacterFont &char_font)
*
*    PURPOSE:
*
*    Specifies the font to be used with this object.
*
*    PARAMETERS:
*
*    char_font - The UIC font to use.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setFont (CharacterFont &char_font)
{ 
  // If a font was previously specified...
  if (font)
    // If a graphics context was already created...
    if (fontGC) {
      Display *dpy;

      // Free the graphics context
      dpy = (Display *) xv_get (baseWindow->getXViewObject (), XV_DISPLAY);
      XFreeGC (dpy, fontGC);
    }

  // Save the new font
  font = &char_font;
  fontGC = (GC) NULL;

  // If the XView object already exists...
  if (owObject) {
    // Create the XView font object and set it
    ((UIObject *) font)->createObject (this);
    xv_set (owObject, XV_FONT, font->getXViewObject (), NULL);
  }
}

/*F+ void UIObject::setEventHandler (UICHandlerFunction handler)
*
*    PURPOSE:
*
*    Specified the event handler for the object.
*
*    PARAMETERS:
*
*    handler - The handler funtion.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setEventHandler (UICHandlerFunction handler)
{
  eventHandler = handler;
}

/*F+ int UIObject::addHotRegion (HotRegion &hot_region)
*
*    PURPOSE:
*
*    Adds a hot region to the list of hot regions for this object.
*
*    PARAMETERS:
*
*    hot_region - The hot region to add.
*
*    RETURN VALUES:
*
*    Returns the position in the hot region list for this object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int UIObject::addHotRegion (HotRegion &hot_region)
{
  hotRegions.addItem ((Generic *) &hot_region);
  return hotRegions.getSize ();
}

/*F+ HotRegion *UIObject::addHotRegion (int index)
*
*    PURPOSE:
*
*    Adds the specified hot region to this object's list of hot regions.
*
*    PARAMETERS:
*
*    index - The index into the list of hot regions.
*
*    RETURN VALUES:
*
*    Returns a pointer to the hot region removed.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
HotRegion *UIObject::removeHotRegion (int index)
{
  return ((HotRegion *) hotRegions.removeItem (index-1));
}

/*F+ HotRegion *UIObject::removeHotRegion (HotRegion &hot_region)
*
*    PURPOSE:
*
*    Removes the specified hot region from this object's list of hot regions.
*
*    PARAMETERS:
*
*    hot_region - The hot region to remove.
*
*    RETURN VALUES:
*
*    Returns a pointer to the hot region removed.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
HotRegion *UIObject::removeHotRegion (HotRegion &hot_region)
{
  int index = hotRegions.findItem ((Generic *) &hot_region);
  return ((HotRegion *) hotRegions.removeItem (index));
}

/*F+ int UIObject::addEventTarget (UIObject &object, UICTargetFunction func)
*
*    PURPOSE:
*
*    Adds a target to the list of event targets for this object.
*
*    PARAMETERS:
*
*    object - The target object.
*    func   - The target function.
*
*    RETURN VALUES:
*
*    Returns the position in the target list for this object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int UIObject::addEventTarget (UIObject &object, UICTargetFunction func)
{
  UITarget *target = new UITarget;

  target->targetObject = &object;
  target->targetFunction = func;

  eventTargets.addItem (target);
  return eventTargets.getSize ();
}

/*F+ GC UIObject::getFontGC ()
*
*    PURPOSE:
*
*    Returns the X graphics context of this objects font.
*
*    RETURN VALUES:
*
*    Returns the X graphics context of this objects font.  If no
*    font is specified, NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
GC UIObject::getFontGC ()
{
  // If the XView object exists and a font is specified...
  if (owObject && font) {
    // Create the XView font object, if necessary
    if (!font->getXViewObject ())
      ((UIObject *) font)->createObject (this);

    // If a graphics context has already been created...
    if (fontGC)
      return fontGC;
    // Else, create a graphics context...
    else {
      Display   *dpy;
      XGCValues  gcvalues;
      
      dpy = (Display *) xv_get (baseWindow->getXViewObject (), XV_DISPLAY);
      
      // Set the default values of the graphics context
      gcvalues.font = (Font) xv_get (font->getXViewObject (), XV_XID);
      gcvalues.foreground = BlackPixel (dpy, DefaultScreen (dpy));
      gcvalues.background = WhitePixel (dpy, DefaultScreen (dpy));
      gcvalues.graphics_exposures = FALSE;
      
      // Create the graphics context
      fontGC = XCreateGC (dpy,
			  DefaultScreen (dpy),
			  GCForeground |
			  GCBackground |
			  GCFont |
			  GCGraphicsExposures,
			  &gcvalues);
      
      return fontGC;
    }
  }

  return (GC) NULL;
}

/*F+ int UIObject::getX ()
*
*    PURPOSE:
*
*    Returns the X coordinate of this object.
*
*    RETURN VALUES:
*
*    The X coordinate is returned.  If the XView object does not exist
*    and no X coordinate has been set, UIC_NOT_SET is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int UIObject::getX ()
{
  return ((owObject) ? (int) xv_get (owObject, XV_X) : xLoc);
}

/*F+ int UIObject::getY ()
*
*    PURPOSE:
*
*    Returns the Y coordinate of this object.
*
*    RETURN VALUES:
*
*    The Y coordinate is returned.  If the XView object does not exist
*    and no Y coordinate has been set, UIC_NOT_SET is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int UIObject::getY ()
{
  return ((owObject) ? (int) xv_get (owObject, XV_Y) : yLoc);
}

/*F+ void UIObject::getLocation (int &x_loc, int &y_loc)
*
*    PURPOSE:
*
*    Returns the X-Y location of this object.
*
*    PARAMETERS:
*
*    x_loc - The X coordinate of the object.
*    y_loc - The Y coordinate of the object.
*
*    RETURN VALUES:
*
*    The X-Y location is is returned.  If the XView object does not exist
*    and no location has been set, UIC_NOT_SET is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::getLocation (int &x_loc, int &y_loc)
{
  x_loc = getX ();
  y_loc = getY ();
}

/*F+ int UIObject::getHeight ()
*
*    PURPOSE:
*
*    Returns the height of this object.
*
*    RETURN VALUES:
*
*    The height is returned.  If the XView object does not exist
*    and no height has been set, UIC_NOT_SET is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int UIObject::getHeight ()
{ 
  return ((owObject) ? (int) xv_get (owObject, XV_HEIGHT) : height);
}

/*F+ int UIObject::getWidth ()
*
*    PURPOSE:
*
*    Returns the width of this object.
*
*    RETURN VALUES:
*
*    The width is returned.  If the XView object does not exist
*    and no width has been set, UIC_NOT_SET is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int UIObject::getWidth ()
{
  return ((owObject) ? (int) xv_get (owObject, XV_WIDTH) : width);
}

/*F+ void UIObject::getSize (int &h, int &w)
*
*    PURPOSE:
*
*    Returns the size of this object.
*
*    PARAMETERS:
*
*    h - The height of the object.
*    w - The width of the object.
*
*    RETURN VALUES:
*
*    The height and width is returned.  If the XView object does not exist
*    and no size has been set, UIC_NOT_SET is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::getSize (int &h, int &w)
{
  h = getHeight ();
  w = getWidth ();
}

/*F+ void UIObject::show (Boolean show_flag)
*
*    PURPOSE:
*
*    Specifies whether or not to display this object.
*
*    PARAMETERS
*
*    show_flag - TRUE if the object is to be displayed.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::show (Boolean show_flag)
{ 
  showFlag = show_flag;

  if (owObject)
    xv_set (owObject, XV_SHOW, show_flag, NULL);
}

/*F+ void UIObject::consumeEvent (int event_type)
*
*    PURPOSE:
*
*    Specifies a type of event to be consumed.
*
*    PARAMETERS
*
*    event_type - The type of event to consume.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::consumeEvent (int event_type)
{
  events.addItem ((Generic *) event_type);
  if (owObject) xv_set (owObject, WIN_CONSUME_EVENT, event_type, NULL);
}

/*F+ void UIObject::useDefaultColorMap ()
*
*    PURPOSE:
*
*    Specifies the use of the default UIT color map.  This is not
*    normally called by a user, because setting a foreground or background
*    color will automatically use it.  However, if a user created their own
*    color map and then later needed to use the default one, this can
*    be called to switch color maps.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::useDefaultColorMap ()
{
  // If the default color map is already being used...
  if (defaultColorMap) return;

  defaultColorMap = TRUE;
  colorMap = new ColorMap;

  // Add the default UIC colros to the new color map
  int num_colors = sizeof (uic_colors) / 4;
  for (int i = 0; i < num_colors ; i++) 
    colorMap->addColor (uic_colors [i]);

  // If the XView object already exists...
  if (owObject) {
    ((UIObject *) colorMap)->createObject (this);

    // Determine the indexes of the default XView bg and fg
    int fg_index = colorMap->getDefaultForegroundIndex ();
    int bg_index = colorMap->getDefaultBackgroundIndex ();

    xv_set (owObject, WIN_CMS, colorMap->getXViewObject (), NULL);

    xv_set (owObject, WIN_FOREGROUND_COLOR, fg_index, NULL);
    xv_set (owObject, WIN_BACKGROUND_COLOR, bg_index, NULL);
  }
}

/*F+ void UIObject::setColorMap (ColorMap &color_map)
*
*    PURPOSE:
*
*    Specifies the use of a user defined color map.
*
*    PARAMETERS
*
*    color_map - The user defined color map.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setColorMap (ColorMap &color_map)
{
  // If a default color map has already been created...
  if (defaultColorMap && colorMap)
    delete colorMap;

  defaultColorMap = FALSE;

  // Save the user specified color map
  colorMap = &color_map;

  // If the XView object already exists
  if (owObject) {
    ((UIObject *) colorMap)->createObject (this);

    // Determine the indexs of the default XView bg and fg colors
    int fg_index = colorMap->getDefaultForegroundIndex ();
    int bg_index = colorMap->getDefaultBackgroundIndex ();

    xv_set (owObject, WIN_CMS, colorMap->getXViewObject (), NULL);

    xv_set (owObject, WIN_FOREGROUND_COLOR, fg_index, NULL);
    xv_set (owObject, WIN_BACKGROUND_COLOR, bg_index, NULL);
  }
}

/*F+ void UIObject::setLabel (char *string_label)
*
*    PURPOSE:
*
*    Specifies the string to use for the object's label.
*
*    PARAMETERS
*
*    string_label - The string to use.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::setLabel (char *string_label)
{
  if (label != (char *) NULL)
    delete label;

  label = strcpy (new char [strlen (string_label) + 1], string_label);

  if (owObject)
    xv_set (owObject, XV_LABEL, label, NULL);
}

/*F+ char *UIObject::getValue (char* &return_value)
*
*    PURPOSE:
*
*    Gets the object's last known string value.
*
*    PARAMETERS
*
*    return_value - The string value.
*
*    RETURN VALUES:
*
*    Returns the string value.  If no string value exists, NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
char *UIObject::getValue (char* &return_value)
{
  return (return_value = (char *) NULL);
}

/*F+ int UIObject::getValue (int &return_value)
*
*    PURPOSE:
*
*    Gets the object's last known integer value.
*
*    PARAMETERS
*
*    return_value - The integer value.
*
*    RETURN VALUES:
*
*    Returns the integer value.  If no integer value exists,
*    UIC_NOT_SET is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int UIObject::getValue (int &return_value)
{
  return (return_value = UIC_NOT_SET);
}

/*F+ char *UIObject::getChoice (char* &return_value)
*
*    PURPOSE:
*
*    Gets the string value of the either the object's last selected choice
*    or the choice with the lowest index.
*
*    PARAMETERS
*
*    return_value - The string value.
*
*    RETURN VALUES:
*
*    Returns the string value.  If no string value exists, NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
char *UIObject::getChoice (char *&return_value)
{
  return (return_value = (char *) NULL);
}

/*F+ int UIObject::getChoice (int &return_value)
*
*    PURPOSE:
*
*    Gets the integer value of the either the object's last selected choice
*    or the choice with the lowest index.
*
*    PARAMETERS
*
*    return_value - The integer value.
*
*    RETURN VALUES:
*
*    Returns the integer value.  If no integer value exists,
*    UIC_NOT_SET is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int UIObject::getChoice (int &return_value)
{
  return (return_value = UIC_NOT_SET);
}

/*F+ int UIObject::getChoice (int which_one, int &return_value)
*
*    PURPOSE:
*
*    Gets the string value of the object's specified choice.
*
*    PARAMETERS
*
*    which_one    - Specifies the choice to get the value of.
*    return_value - The integer value.
*
*    RETURN VALUES:
*
*    Returns the integer value.  If no integer value exists,
*    UIC_NOT_SET is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int UIObject::getChoice (int which_one, int &return_value)
{
  return (return_value = UIC_NOT_SET);
}

/*F+ char *UIObject::getChoice (int which_one, char* &return_value)
*
*    PURPOSE:
*
*    Gets the string value of the object's specified choice.
*
*    PARAMETERS
*
*    which_one    - Specifies the choice to get the value of.
*    return_value - The string value.
*
*    RETURN VALUES:
*
*    Returns the string value.  If no string value exists, NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
char *UIObject::getChoice (int which_one, char *&return_value)
{
  return (return_value = (char *) NULL);
}

/*F+ int UIObject::getNumberSelected ()
*
*    PURPOSE:
*
*    Returns the number of selected choices.
*
*    RETURN VALUES:
*
*    Returns the number of selected choices.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
int UIObject::getNumberSelected ()
{
  return (0);
}

/*F+ Boolean UIObject::isBaseWindow ()
*
*    PURPOSE:
*
*    Determines whether or not this object is a BaseWindow.
*
*    RETURN VALUES:
*
*    Returns TRUE if it is a BaseWindow.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Boolean UIObject::isBaseWindow ()
{
  return FALSE;
}

/*F+ void UIObject::createObject (UIObject *parent)
*
*    PURPOSE:
*
*    Creates the XView object.
*
*    PARAMETERS:
*
*    parent - The parent of this object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIObject::createObject (UIObject *parent)
{
}

#ifdef TEST
#endif /* TEST */
