/*H+ UIWindow.cc
*
*    SCCS ID: @(#)UIWindow.cc 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Defines the member functions of the UIWindow class.
*
*    AUTHOR: Joe Warzecha and Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review. 
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)UIWindow.cc 1.1 91/08/02 SMI"; 
#endif

// Includes
#include "UIWindow.h"

/*F+ UIWindow::UIWindow ()
*
*    PURPOSE:
*
*    UIWindow constructor.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
UIWindow::UIWindow ()
{
  setUIWindowDefaults ();
}

/*F+ UIWindow::UIWindow (char *string_label)
*
*    PURPOSE:
*
*    UIWindow constructor.
*
*    PARAMETERS:
*
*    string_label - Specifies a string for the label.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
UIWindow::UIWindow (char *string_label) : (string_label)
{
  setUIWindowDefaults ();
}

/*F+ void UIWindow::setUIWindowDefaults ()
*
*    PURPOSE:
*
*    Initializes the data members of the UIWindow class.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIWindow::setUIWindowDefaults ()
{
  foregroundColor = UIC_NOT_SET;
  backgroundColor = UIC_NOT_SET;
  help = (char *) NULL;
  displayFooter = TRUE;
  doneHandler = UIC_NO_FUNCTION;
  leftFooter = (char *) NULL;
  rightFooter = (char *) NULL;
  fitHeight = FALSE;
  fitWidth = FALSE;
  topWindow = (UIWindow *) NULL;
  leftWindow = (UIWindow *) NULL;
  parentWindow = (UIWindow *) NULL;
  
  // If the main base window is instantiated or this is a pop-up window...
  if (baseWindow)
    // Add this window into the list of peer windows of the main base window
    peerWindows->addItem ((Generic *) this);
  // Else, this is the main base window...
  else
    // Create a peer windows list
    peerWindows = new GenericList;
}

/*F+ UIWindow::~UIWindow ()
*
*    PURPOSE:
*
*    UIWindow destructor.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
UIWindow::~UIWindow ()
{
  if (label) delete label;
  if (leftFooter) delete leftFooter;
  if (rightFooter) delete rightFooter;

  childWindows.dropItems ();

  // If this is the main base window...
  if (baseWindow == this) {
    // Delete the peer windows list
    peerWindows->dropItems ();
    delete peerWindows;
  }
}

/*F+ void UIWindow::setDoneHandler (UICHandlerFunction handler)
*
*    PURPOSE:
*
*    Specified the done handler for the window.
*
*    PARAMETERS:
*
*    handler - The handler funtion.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
void UIWindow::setDoneHandler (UICHandlerFunction handler)
{
  doneHandler = handler;
}

/*F+ Notify_value UIWindow::eventProc (Xv_window          win,
*                                      Event             *event,
*				       Notify_arg         arg,
*				       Notify_event_type  type)
*
*    PURPOSE:
*
*    The XView window event proc.  This is called whenever an
*    event occurs on a window object.
*
*    PARAMETERS:
*
*    win   - The XView object the event occurred on.
*    event - The XView event info structure.
*    arg   - XView client data.
*    type  - The XView notifier event type: SAFE or IMMEDIATE.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
Notify_value UIWindow::eventProc (Xv_window          win,
				  Event             *event,
				  Notify_arg         arg, 
				  Notify_event_type  type)
{
  UIWindow *object = (UIWindow *) xv_get (win, XV_KEY_DATA, UIC_OBJECT_KEY);
  
  // Set the UIC event information
  object->setEventInformation (event);
  
  // Call the user specified event handler
  if (object->eventHandler)
    (*object->eventHandler) (object);
  
  // Erase the UIC event information
  object->resetEventInformation ();

  // If the user specified an event result...
  if (object->eventResultSet) {
    object->eventResultSet = FALSE;

    // Return the specified result to the notifier
    return (object->eventResult);
  }
  
  // Tell the notifier to handle the next event function in its list
  return (notify_next_event_func (win, (Notify_event) event, arg, type)); 
}

/*F+ char *UIWindow::getDragFileName ()
*
*    PURPOSE:
*
*    Gets the name of the file that was draged and dropped into the
*    the TextDisplay.
*
*    RETURN VALUES:
*
*    Returns the name of the file.  The memory is allocated on the heap,
*    so the caller must delete it.  If no file was drag and dropped into
*    the TextDisplay, NULL is returned.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*
*F-*/
char *UIWindow::getDragFileName ()
{
  DragMessage   *msg = (DragMessage *) xv_get (owObject, WIN_MESSAGE_DATA);
  Display       *display = (Display *) XV_DISPLAY_FROM_WINDOW (owObject);
  long           long_offset = 0L;
  long           long_length = (long) (MAXPATHLEN / 4);
  int            delete_flag = TRUE;
  Atom           actual_type;
  int            actual_format;
  u_long         nitems;
  u_long         bytes_after;
  unsigned int  *value;
  char	        *return_value;
  
  if (msg == NULL || msg->property == None)
    return (char *) NULL;

  if (XGetWindowProperty (display, msg->window, msg->property,
			  long_offset, long_length, delete_flag,
			  AnyPropertyType, &actual_type, &actual_format,
			  &nitems, &bytes_after, &value) != Success)
    return (char *) NULL;
  
  return_value = strcpy (new char [strlen ((char *) value) + 1],
			 (char *) value);
  XFree ((char *) value);
  return (return_value);
}

#ifdef TEST
#endif /* TEST */
