/*
*    SCCS ID: %Z%%M% %I% %E% SMI 
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*/

#ifndef IMAGE_BUTTON 
#define IMAGE_BUTTON 

#include "ColorMap.h"
#include "ComponentDisplay.h"
#include "UIComponent.h"

enum Mode {NOT_PRESSED, PRESSED};

class ImageButton : public UIComponent {
public:
  ImageButton (Image &image)
    {
      buttonImage = &image;

      ((UIObject *) &image)->createObject ();

      xLoc = 0;
      yLoc = 0;
      width = image.getWidth ();
      height = image.getWidth ();

      mode = NOT_PRESSED;
      leftButton = FALSE;
    }

  ~ImageButton () { cd->removePSProc (imageProcName);
	            cd->removePSProc (boxProcName);
		    cd->removePSProc (ubProcName);
		    cd->removePSProc (lbProcName); }

  void setX (int x) { xLoc = x; }
  void setY (int y) { yLoc = y; }

  int getX () { return xLoc; }
  int getY () { return yLoc; }

protected:
  Image		   *buttonImage;
  char              imageProcName [22];
  char		    boxProcName [20];
  char		    ubProcName [20];
  char		    lbProcName [20];
  char		    drawImage [80];
  char		    drawBox [80];
  char		    drawUB [80];
  char		    drawLB [80];

  ComponentDisplay *cd;
  HotRegion         hotRegion;
  ColorMap          componentColorMap;
  Color             BG1;
  Color             BG2;
  Color             BG3;
  Color             HL;
  Color		    FG;

  char              temp_string [128];
  Boolean	    leftButton;
  Mode		    mode;

  void repaintComponent (ComponentDisplay *parent)
    {
      parent->setPSFocus ();

      if (mode == NOT_PRESSED)
        cd->setPSColor (BG1);
      else
        cd->setPSColor (BG2);
      POSTSCRIPT_STRING (drawBox);

      if (mode == NOT_PRESSED)
        cd->setPSColor (HL);
      else
        cd->setPSColor (BG3);
      POSTSCRIPT_STRING (drawUB);

      if (mode == NOT_PRESSED)
        cd->setPSColor (BG3);
      else
        cd->setPSColor (HL);
      POSTSCRIPT_STRING (drawLB);

      cd->setPSColor (FG);
      POSTSCRIPT_STRING (drawImage);

      ps_flush_PostScript ();
    }

  void showFeedback (ComponentDisplay *parent)
    {
      parent->setPSFocus ();
 
      if (mode == NOT_PRESSED)
        cd->setPSColor (BG1);
      else
        cd->setPSColor (BG2);
      POSTSCRIPT_STRING (drawBox);
 
      if (mode == NOT_PRESSED)
        cd->setPSColor (HL);
      else
        cd->setPSColor (BG3);
      POSTSCRIPT_STRING (drawUB);
 
      if (mode == NOT_PRESSED)
        cd->setPSColor (BG3);
      else
        cd->setPSColor (HL);
      POSTSCRIPT_STRING (drawLB);
 
      ps_flush_PostScript ();
    }
 
  void componentHotRegionEventProc (ComponentDisplay *parent)
    {
      InputEvent *event = cd->getEvent ();

      if (event->isButtonDown () &&
	 ((event->getAction () == ACTION_SELECT) || leftButton)) {
	if (mode == NOT_PRESSED) {
	  mode = PRESSED;
	  leftButton = TRUE;
          showFeedback (parent);
	}
      }
      else if (event->isButtonUp () && (mode == PRESSED)) {
	mode = NOT_PRESSED;
	leftButton = FALSE;
        showFeedback (parent);

	if (notifyHandler)
	  (*notifyHandler) (this);
      }
    }

  void componentBackgroundEventProc (ComponentDisplay *parent)
    {
      InputEvent *event = cd->getEvent ();

      if (mode == PRESSED) {
        mode = NOT_PRESSED;
	showFeedback (parent);
      }

      if (event->isButtonDown () && (event->getAction () == ACTION_SELECT))
        leftButton = TRUE;
      else if (event->getID () != LOC_DRAG)
        leftButton = FALSE;
    }

  void createObject (UIObject *parent)
    {
      cd = (ComponentDisplay *) parent;

      hotRegion.setDefaultRectangle (xLoc, yLoc, width, height);
      ((UIObject *) &componentColorMap)->createObject ();

      BG1 = componentColorMap.getColorRGB ("BG1");
      BG2 = componentColorMap.getColorRGB ("BG2");
      BG3 = componentColorMap.getColorRGB ("BG3");
      HL  = componentColorMap.getColorRGB ("Highlight");
      FG  = componentColorMap.getColorRGB ("Foreground");

      int ID = uiObjectID;
      sprintf (imageProcName, "SIB_image_%d", ID);
      sprintf (boxProcName, "SIB_box_%d", ID);
      sprintf (ubProcName, "SIB_ub_%d", ID);
      sprintf (lbProcName, "SIB_lb_%d", ID);

      cd->storePSDrawImageProc (imageProcName, *buttonImage, TRUE);
      sprintf (drawImage, " %d %d %s ", xLoc, yLoc, imageProcName);

      cd->storePSDrawImageProc (boxProcName, *buttonImage, TRUE, TRUE);
      sprintf (drawBox, " %d %d %s ", xLoc, yLoc, boxProcName);

      POSTSCRIPT_STRING ("/");
      POSTSCRIPT_STRING (ubProcName);
      POSTSCRIPT_STRING (" { ");
      POSTSCRIPT_STRING ("/h exch def ");
      POSTSCRIPT_STRING ("/w exch def ");
      POSTSCRIPT_STRING ("/y exch def ");
      POSTSCRIPT_STRING ("/x exch def ");
      POSTSCRIPT_STRING ("newpath ");
      POSTSCRIPT_STRING ("x y h add moveto ");
      POSTSCRIPT_STRING ("0 h neg rlineto ");
      POSTSCRIPT_STRING ("w 0 rlineto ");
      POSTSCRIPT_STRING ("stroke ");
      POSTSCRIPT_STRING (" } def ");
      ps_flush_PostScript ();

      sprintf (drawUB, " %d %d %d %d %s ",
	       xLoc, yLoc, width, height, ubProcName);

      POSTSCRIPT_STRING ("/");
      POSTSCRIPT_STRING (lbProcName);
      POSTSCRIPT_STRING (" { ");
      POSTSCRIPT_STRING ("/h exch def ");
      POSTSCRIPT_STRING ("/w exch def ");
      POSTSCRIPT_STRING ("/y exch def ");
      POSTSCRIPT_STRING ("/x exch def ");
      POSTSCRIPT_STRING ("newpath ");
      POSTSCRIPT_STRING ("x w add y moveto ");
      POSTSCRIPT_STRING ("0 h rlineto ");
      POSTSCRIPT_STRING ("w neg 0 rlineto ");
      POSTSCRIPT_STRING ("stroke ");
      POSTSCRIPT_STRING (" } def ");
      ps_flush_PostScript ();

      sprintf (drawLB, " %d %d %d %d %s ",
	       xLoc, yLoc, width, height, lbProcName);

      cd->addComponentHotRegion (*this, hotRegion, TRUE);
      cd->addRepaintComponent (*this);
    }
};

#endif
