/*H+ events.c
*
*    SCCS ID: @(#)events.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Functions for writing event handler stubs.
*
*    GLOBAL ROUTINES:
*
*    guic_write_event_function_stub - Writes a stub for a UIC event handler.
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review.
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)events.c 1.1 91/08/02 SMI"; 
#endif

/* Includes */
#include <stdio.h>
#include <guide.h>
#include <gio.h>
#include "guic.h"
#include "events.h"
#include "list.h"
#include "xv.h"

/*F+ void guic_write_event_function_stub (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Writes the UIC event handler stub corresponding to the supplied object.
*
*    PARAMETERS:
*
*    obj - The object to write the stub for.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
void
guic_write_event_function_stub(obj)
	GUIC_OBJ        *obj;
{
	char	       objname[64];

	if (!obj->event_handler || guic_stub_written(obj->event_handler))
		return;

	/*
	 * Write comment.
	 */
	gio_printf("/*\n * Event handler function for `%s'.\n */\n",
		 obj->name);
	
	/*
	 * Write function header.
	 */
	gio_printf ("void %s (UIObject *object)\n", obj->event_handler);

	if (guic_verbose())
		fprintf(stderr, "  %s()\n", obj->event_handler);
	
	gio_puts("{\n");
	
	/*
	 * Write local variables
	 */
	gio_puts ("  InputEvent *event;\n");
	gio_putc ('\n');

	/*
	 * Write code to output message.
	 */
	gio_puts("  event = object->getEvent ();\n");
	gio_printf("  fprintf (stderr, \"%s: %s: event %%d\\n\", event->getID ());\n",
		   guic_application(), obj->event_handler);

	/*
	 * Write object-specific event code.
	 */
	if (obj->type == G_TEXT_PANE) {
		gio_putc ('\n');
		gio_puts("  if (event->getAction () == ACTION_DRAG_LOAD) {\n");
		gio_puts("    char *name;\n\n");
		gio_puts("    if (name = ((TextDisplay *) object)->getDragFileName ()) {\n");
		gio_puts("      ((TextDisplay *) object)->loadFile (name);\n");
		gio_puts("      object->setEventResult (NOTIFY_DONE);\n");
		gio_puts("      delete name;\n");
		gio_puts("    }\n");
		gio_puts("  }\n");

	} else if (obj->type == G_CANVAS_PANE) {
		gio_putc ('\n');
		gio_puts("  if (event->getAction () == ACTION_DRAG_LOAD) {\n");
		gio_puts("    char *name;\n\n");
		gio_puts("    if (name = ((ComponentDisplay *) object)->getDragFileName ()) {\n");
		gio_printf ("      fprintf (stderr, \"%s: %s: load %%s\\n\", name);\n",
			    guic_application(), obj->event_handler);
		gio_puts("      delete name;\n");
		gio_puts("    }\n");
		gio_puts("  }\n");
	}
	
	/*
	 * Write function footer.
	 */
	gio_puts("\n  return;\n");
	gio_puts("}\n\n");
}

#ifdef TEST
#endif /* TEST */
