/*H+ help.c
*
*    SCCS ID: @(#)help.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Create a help file for user interface objects.  This file contains
*    the help text for each object, indexed by object name.
*
*    GLOBAL ROUTINES:
*
*    guic_write_help_file - Write the help file.
*    guic_help_file - Returns the name of the help file.
*
*    LOCAL ROUTINES:
*
*    write_header - Writes the help file header.
*    write_help - Writes the help text for each object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review.
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)help.c 1.1 91/08/02 SMI"; 
#endif

/* Includes */
#include <sys/param.h>
#include <stdio.h>
#include <guide.h>
#include <gio.h>
#include "guic.h"
#include "list.h"
#include "help.h"

/* Macro Definitions */
#define	COMMENT			"# "		/* comment prefix */
#define	KEYWORD			":"		/* keyword prefix */

/* Local Function Declarations */
void            write_header();
void		write_help();

/*F+ int guic_write_help_file ()
*
*    PURPOSE:
*
*    Write the user interface object help file.
*
*    RETURN VALUES:
*
*    Returns OK if successful.  Returns ERROR if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int
guic_write_help_file()
{
	char           *errmsg;

	/*
	 * Open the output file.
	 */
	if (errmsg = gio_open_output(guic_help_file())) {
		guic_error(errmsg);
		return ERROR;
	}
	if (!guic_silent())
		fprintf(stderr, "%s: writing %s\n",
			guic_program(), guic_help_file());

	/*
	 * Write the output file.
	 */
	write_header();
	write_help();

	/*
	 * Close the output file.
	 */
	gio_close_output();
	return OK;
}

/*F+ char *guic_help_file ()
*
*    PURPOSE:
*
*    Returns the name of the help file.
*
*    RETURN VALUES:
*
*    Returns the name of the help file.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
guic_help_file()
{
	static char	name[MAXPATHLEN];

	if (!*name) {
		strcpy(name, guic_application());
		strcat(name, ".info");
	}
	return name;
}

/*F+ void static write_header ()
*
*    PURPOSE:
*
*    Write help file header.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static void
write_header()
{
	gio_puts(COMMENT);
	gio_putc('\n');
	gio_printf( "%s%s - User interface object help text.\n",
		   COMMENT, guic_help_file());
	gio_printf("%sThis file was generated by `%s' from `%s'.\n",
		   COMMENT, guic_program(), guic_gil_file());
	gio_puts(COMMENT);
	gio_puts("DO NOT EDIT BY HAND.\n");
}

/*F+ void static write_help ()
*
*    PURPOSE:
*
*    Write help text for each object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static void
write_help()
{
	GUIC_OBJ        *obj;

	for (obj = guic_first(); obj; obj = guic_next(obj)) {
		if (!obj->help)
			continue;

		gio_puts(COMMENT);
		gio_putc('\n');
		gio_printf( "%s%s\n", KEYWORD, obj->name);
		gio_puts(obj->help);
		gio_putc('\n');
	}
}

#ifdef TEST
#endif
