/*H+ init.c
*
*    SCCS ID: @(#)init.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Write a file containing object initialization functions.  These functions
*    will be compiled and linked with the application.
*
*    GLOBAL ROUTINES:
*
*    guic_init_file - Returns the name of the object initialization file.
*    guic_write_init_file - Writes the object initialization file.
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review.
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)init.c 1.1 91/08/02 SMI"; 
#endif

/* Includes */
#include <sys/param.h>
#include <stdio.h>
#include <guide.h>
#include "gio.h"
#include "guic.h"
#include "init.h"
#include "list.h"
#include "object.h"
#include "xv.h"
#include "strlib.h"
#include "memlib.h"

/* Macro Definitions */
#define NO_TYPE -1

/* Type Definitions */

/*
 * Define the includes array entry
 */
typedef struct include_entry {
  G_TYPES  type;       /* The GUIDE type */
  int      sub_type;   /* The GUIDE sub-type */
  char    *filename;   /* The name of the include file */
  int      used_index; /* Index of used flag for this include filename */
} IncludeEntry;

/* Global Variables */

/*
 * Define the includes array
 */
static IncludeEntry include_entries [] =
{ G_BASE_WINDOW, NO_TYPE, "BaseWindow.h", 0,
  G_BUTTON, NO_TYPE, "Button.h", 1,
  G_CANVAS_PANE, NO_TYPE, "ComponentDisplay.h", 2,
  G_CONTROL_AREA, NO_TYPE, "ComponentDisplay.h", 2,
  G_SETTING, G_CHECK, "CheckBoxChoice.h", 3,
  G_SCROLLING_LIST, NO_TYPE, "ListChoice.h", 4,
  G_MENU, G_EXCLUSIVE_MENU, "ChoiceMenu.h", 5,
  G_MENU, G_NONEXCLUSIVE_MENU, "ChoiceMenu.h", 5,
  G_MENU, G_COMMAND_MENU, "CommandMenu.h", 6,
  G_GAUGE, NO_TYPE, "Gauge.h", 7,
  G_TEXT_FIELD, G_ALPHANUMERIC, "AlphanumericInput.h", 8,
  G_TEXT_FIELD, G_NUMERIC, "NumericInput.h", 9,
  G_POPUP_WINDOW, NO_TYPE, "PopUpWindow.h", 10,
  G_SLIDER, NO_TYPE, "Slider.h", 11,
  G_SETTING, G_STACK, "StackChoice.h", 12,
  G_MESSAGE, NO_TYPE, "TextItem.h", 13,
  G_SETTING, G_EXCLUSIVE, "ToggleChoice.h", 14,
  G_SETTING, G_NONEXCLUSIVE, "ToggleChoice.h", 14,
  G_TERM_PANE, NO_TYPE, "TermDisplay.h", 15,
  G_TEXT_PANE, NO_TYPE, "TextDisplay.h", 16,
  NO_TYPE, NO_TYPE, (char *) NULL, -1 };

static int used_flag [18];

/* External Function Declarations */
extern int strcmp ();

/*F+ char *guic_init_file ()
*
*    PURPOSE:
*
*    Returns the name of the intializations file.
*
*    RETURN VALUES:
*
*    Returns the name of the intializations file.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char           *
guic_init_file()
{
	static char	name[MAXPATHLEN];

	if (!*name) {
		strcpy(name, guic_application());
		strcat(name, "_ui.cc");
	}
	return name;
}

/*F+ int guic_write_init_file ()
*
*    PURPOSE:
*
*    Write the user interface object initialization file.
*
*    RETURN VALUES:
*
*    Returns OK if successful.  Returns ERROR if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int
guic_write_init_file()
{
        char           *base_window = (char *) NULL;
	char           *errmsg;
	char          **func_names = (char **) NULL;
	int             i;
	int             menu_found = 0;
	GUIC_OBJ        *obj;

	/*
	 * Open the output file.
	 */
	if (errmsg = gio_open_output(guic_init_file())) {
		guic_error(errmsg);
		return ERROR;
	}
	if (!guic_silent())
		fprintf(stderr, "%s: writing %s\n",
			guic_program(), guic_init_file());

	/*
	 * Write file header.
	 */
	gio_puts("/*\n");
	gio_printf(" * %s - User interface object initialization functions.\n",
		 guic_init_file());
	gio_printf(" * This file was generated by `%s' from `%s'.\n",
		 guic_program(), guic_gil_file());
	gio_puts(" * DO NOT EDIT BY HAND.\n");
	gio_puts(" */\n\n");

	/*
	 *  Make copy of object name and write includes
	 */
	for (obj = guic_first(); obj; obj = guic_next(obj)) {
	  obj->original_name = str_dup (obj->name);

	  for (i=0; include_entries [i].type != NO_TYPE; i++) {
	    if (obj->type == include_entries [i].type) {
	      /* Menus have sub-types... */
	      if (obj->type == G_MENU) {
		if (obj->menu_type == include_entries [i].sub_type)
		  break;
	      }
	      /* Settings have sub-types... */
	      else if (obj->type == G_SETTING) {
		if (obj->setting_type == include_entries [i].sub_type)
		  break;
	      }
	      /* Text fields have sub-types... */
	      else if (obj->type == G_TEXT_FIELD) {
		if (obj->text_type == include_entries [i].sub_type)
		  break;
	      }
	      else
		break;
	    }
	  }

	  /* If this include file was not already used... */
	  if (!used_flag [include_entries [i].used_index]) {
	    gio_printf ("#include \"%s\"\n", include_entries [i].filename);
	    used_flag [include_entries [i].used_index] = 1;
	  }
	}

	/*
	 * If any scrollers are specified, we need to include the
	 * Scroller.h include file.
	 */
	for (obj = guic_first(); obj; obj = guic_next(obj))
	  if (obj->hscrollbar || obj->vscrollbar)
	    gio_puts ("#include \"Scroller.h\"\n");

	/* Always include the Notifier include file */
	gio_puts ("#include \"Notifier.h\"\n");
	gio_putc ('\n');

	/*
	 * Write external function references
	 */
	for (obj = guic_first(); obj; obj = guic_next(obj)) {
	  if (obj->done_handler) {
	    if (str_arrayfind (func_names, obj->done_handler, strcmp) == -1)
	      func_names = str_arrayadd (func_names, obj->done_handler);
	  }

	  if (obj->event_handler) {
	    if (str_arrayfind (func_names, obj->event_handler, strcmp) == -1)
	      func_names = str_arrayadd (func_names, obj->event_handler);
	  }

	  if (obj->menu_handler) {
	    if (str_arrayfind (func_names, obj->menu_handler, strcmp) == -1)
	      func_names = str_arrayadd (func_names, obj->menu_handler);
	  }

	  if (obj->menu_item_handler) {
	    for (i=0; obj->menu_item_label [i]; i++)
	      if (obj->menu_item_handler [i])
		if (str_arrayfind (func_names,
				   obj->menu_item_handler [i],
				   strcmp) == -1)
		  func_names = str_arrayadd (func_names,
					     obj->menu_item_handler [i]);
	  }

	  if (obj->notify_handler) {
	    if (str_arrayfind (func_names, obj->notify_handler, strcmp) == -1)
	      func_names = str_arrayadd (func_names, obj->notify_handler);
	  }

	  if (obj->repaint_proc) {
	    if (str_arrayfind (func_names, obj->repaint_proc, strcmp) == -1)
	      func_names = str_arrayadd (func_names, obj->repaint_proc);
	  }
        }

	if (func_names) {
	  for (i=0; func_names [i]; i++)
	    gio_printf ("extern void %s (UIObject *);\n", func_names [i]);
	  gio_putc ('\n');
	}

	/*
	 * Write main header.
	 */
	gio_puts ("main (int argc, char **argv)\n");
	gio_puts ("{\n");

	/*
	 * Get the name and object of the first base window specified in
	 * the .G file.
	 */
	for (obj = guic_first(); obj; obj = guic_next(obj)) {
	  if (obj->type == G_BASE_WINDOW) {
	    base_window = str_dup (obj->name);
	    break;
	  }
        }

	/*
	 * Write the instantiation and initialization of the first base
	 * window specified in the .G file.
	 */
	if (base_window) {
	  guic_write_object_instantiation (obj);
	  gio_printf ("  %s.initUI (argc, argv);\n", base_window);
	  guic_write_object_initialization (obj);
	  gio_putc ('\n');
	}

	/*
         * Because menus can reference any other menu as a sub-menu, all of
	 * the menus must instantiated first before their items are
	 * initialized.
	 */

	/*
	 * Instantiate menus
	 */
	for (obj = guic_first(); obj; obj = guic_next(obj)) {
	  if (obj->type == G_MENU) {
	    guic_write_object_instantiation (obj);
	    menu_found = 1;
	  }
	}
	if (menu_found) gio_putc ('\n');

	/*
	 * Initialize menus
	 */
	for (obj = guic_first(); obj; obj = guic_next(obj)) {
	  if (obj->type == G_MENU) {
	    guic_write_object_initialization (obj);
	    gio_putc ('\n');
	  }
	}

	/*
	 * Write all of the object instantiations, initializations and
	 * associations, except for the primary base window instantiation
	 * and initialization and menu instantiation and initialization.
	 */
	for (obj = guic_first(); obj; obj = guic_next(obj))
	  if (((strcmp (obj->name, base_window) ||
		(obj->type != G_BASE_WINDOW)) &&
	       (obj->type != G_MENU))) {
	    guic_write_object_instantiation (obj);
	    guic_write_object_initialization (obj);
	    guic_write_object_association (obj);
	    gio_putc ('\n');
	  }

	gio_puts ("  Notifier notifier;\n");
	gio_printf ("  notifier.start ();\n");
	gio_putc ('\n');

	gio_puts ("  exit (0);\n");
	gio_puts ("}\n");

	gio_close_output();

	str_arrayfree (func_names);
	FREE (base_window);

	return OK;
}

#ifdef TEST
#endif
