/*H+ list.c
*
*    SCCS ID: @(#)list.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Functions for managing the linked list of application UI objects.
*
*    GLOBAL ROUTINES:
*
*    guic_first - Returns the first object in the list.
*    guic_next - Returns the next object in the list.
*    guic_append - Appends the specified object to the list.
*    guic_object - Returns a pointer to the object with the specified name.
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review.
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)list.c 1.1 91/08/02 SMI"; 
#endif

/* Includes */
#include <stdio.h>
#include <guide.h>
#include "guic.h"
#include "list.h"

/* Global Variables */
static GUIC_OBJ *Head = NULL;			/* head of linked list */
static GUIC_OBJ *Tail = NULL;			/* tail of linked list */

/*F+ GUIC_OBJ *guic_first ()
*
*    PURPOSE:
*
*    Return the first object in the list.
*
*    RETURN VALUES:
*
*    Returns a pointer to an object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
GUIC_OBJ        *
guic_first()
{
	return Head;
}

/*F+ GUIC_OBJ *guic_next ()
*
*    PURPOSE:
*
*    Return the next object in the list.
*
*    RETURN VALUES:
*
*    Returns a pointer to an object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
GUIC_OBJ        *
guic_next(obj)
	GUIC_OBJ        *obj;
{
	return obj->next;
}

/*F+ void guic_append (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Append an object onto the list of objects.
*
*    PARAMETERS:
*
*    obj - The object to appemd to the list.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
void
guic_append(obj)
	GUIC_OBJ        *obj;
{
	if (!Head)
		Head = Tail = obj;
	else {
		Tail->next = obj;
		Tail = obj;
	}
}

/*F+ GUIC_OBJ *guic_object (char *name)
*
*    PURPOSE:
*
*    Find an object given its name.  Return a pointer to the object if
*    successful, otherwise NULL.
*
*    PARAMETERS:
*
*    name - The name of the object to find.
*
*    RETURN VALUES:
*
*    Returns a pointer to an object.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
GUIC_OBJ        *
guic_object(name)
	char           *name;
{
	GUIC_OBJ        *obj;

	if (!name)
		return NULL;

	for (obj = guic_first(); obj; obj = guic_next(obj))
		if (strcmp(obj->name, name) == 0)
			break;
	return obj;
}

#ifdef TEST
#endif
