/*H+ make.c
*
*    SCCS ID: @(#)make.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Create a make file that can be used to test the generated application.
*
*    GLOBAL ROUTINES:
*
*    guic_write_make_file - Write the make file.
*    guic_make_file - Returns the name of the make file.
*
*    LOCAL ROUTINES:
*
*    write_file - Write the makefile text.
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review.
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)make.c 1.1 91/08/02 SMI"; 
#endif

/* Includes */
#include <stdio.h>
#include <guide.h>
#include <gio.h>
#include "guic.h"
#include "list.h"
#include "make.h"

/* Macro Definitions */
#define	COMMENT			"# "		/* comment prefix */
#define	KEYWORD			":"		/* keyword prefix */

/* Local Function Declarations */
void		write_file();

/*F+ int guic_write_make_file ()
*
*    PURPOSE:
*
*    Write the make file.
*
*    RETURN VALUES:
*
*    Returns OK if successful.  Returns ERROR if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int
guic_write_make_file()
{
	char           *errmsg;
	FILE	       *fp;

	/*
	 * Make sure there are no other make files before continuing.
	 */
	if ((fp = fopen("makefile", "r")) || (fp = fopen("Makefile", "r"))) {
		fclose(fp);
		if (!guic_silent())
			fprintf(stderr, "%s: skipping %s\n",
				guic_program(), guic_make_file());
		return OK;
	}

	/*
	 * Open the output file.
	 */
	if (errmsg = gio_open_output(guic_make_file())) {
		guic_error(errmsg);
		return ERROR;
	}
	if (!guic_silent())
		fprintf(stderr, "%s: writing %s\n",
			guic_program(), guic_make_file());

	/*
	 * Write the output file.
	 */
	write_file();

	/*
	 * Close the output file.
	 */
	gio_close_output();
	return OK;
}

/*F+ char *guic_make_file ()
*
*    PURPOSE:
*
*    Returns the name of the make file.
*
*    RETURN VALUES:
*
*    Returns the name of the make file.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char	       *
guic_make_file()
{
	return "Makefile";
}

/*F+ void static write_file ()
*
*    PURPOSE:
*
*    Write the makefile text.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static void
write_file()
{
	gio_printf("# This file was generated by `%s' from `%s'.\n",
		   guic_program(), guic_gil_file());
	gio_putc('\n');

	gio_puts("# Parameters.\n");
	gio_putc('\n');

	gio_printf("PROGRAM = %s\n", guic_application());
	gio_puts("SOURCES.cc = \n");
	gio_puts("SOURCES.h = \n");
	gio_printf("SOURCES.G = %s\n", guic_gil_file());
	gio_printf("STUBS.G = %s\n", guic_gil_file());
	gio_putc('\n');

	gio_puts("# Derived parameters.\n");
	gio_putc('\n');

	gio_puts("SOURCES = \\\n");
	gio_puts("\t$(SOURCES.G) \\\n");
	gio_puts("\t$(SOURCES.h) \\\n");
	gio_puts("\t$(SOURCES.l) \\\n");
	gio_puts("\t$(SOURCES.y) \\\n");
	gio_puts("\t$(SOURCES.cc)\n");
	gio_putc('\n');

	gio_puts("TARGETS.cc = \\\n");
	gio_puts("\t$(SOURCES.G:%.G=%_ui.cc) \\\n");
	gio_puts("\t$(STUBS.G:%.G=%_stubs.cc)\n");
	gio_putc('\n');

	gio_puts("TARGETS.h = \\\n");
/* *** Original GXV Code ***
	gio_puts("\t$(SOURCES.G:%.G=%_ui.h) \\\n");
*/
	gio_puts("\t$(SOURCES.l:%.l=%.h) \\\n");
	gio_puts("\t$(SOURCES.y:%.y=%.h)\n");
	gio_putc('\n');

	gio_puts("TARGETS.other = \\\n");
	gio_puts("\t$(SOURCES.G:%.G=%.info)\n");
	gio_putc('\n');

	gio_puts("TARGETS = \\\n");
	gio_puts("\t$(TARGETS.other) \\\n");
	gio_puts("\t$(TARGETS.h) \\\n");
	gio_puts("\t$(TARGETS.cc)\n");
	gio_putc('\n');

	gio_puts("OBJECTS = \\\n");
	gio_puts("\t$(SOURCES.cc:%.cc=%.o) \\\n");
	gio_puts("\t$(TARGETS.cc:%.cc=%.o)\n");
	gio_putc('\n');

	gio_puts("# Compiler flags.\n");
	gio_putc('\n');

	gio_puts("CCFLAGS += -g\n");
#ifdef OWNV3
	gio_puts("CCFLAGS += -DOWNV3\n");
#endif
#ifdef NO_XVPS
	gio_puts("CCFLAGS += -DNO_XVPS\n");
#endif
	gio_puts("CPPFLAGS += -I$(UITHOME)/include/c++ -I$(OPENWINHOME)/include\n");
	gio_puts("LDFLAGS += -L$(UITHOME)/lib -L$(OPENWINHOME)/lib\n");
#ifndef NO_XVPS
	gio_puts("LDLIBS += -lUIC -lxvps -lxview -lolgx -lX\n");
#else
	gio_puts("LDLIBS += -lUIC -lxview -lolgx -lX\n");
#endif
#ifndef NO_XVPS
#ifdef OWNV3
	gio_puts("LDLIBS += -lwire\n");
#endif
#endif
#ifndef NO_XVPS
	gio_puts("LDLIBS += -lcps\n");
#endif
	gio_puts("LDLIBS += -lm\n");
	gio_putc('\n');

	gio_puts("# Standard targets.\n");
	gio_putc('\n');

	gio_puts("all:	 $(TARGETS.other) $(PROGRAM)\n");
	gio_puts("objects: $(SOURCES.cc) $(TARGETS.cc) $(TARGETS.h) $(OBJECTS)\n");
	gio_puts("sources: $(SOURCES)\n");
	gio_puts("targets: $(SOURCES) $(TARGETS)\n");
	gio_putc('\n');

	gio_puts("$(PROGRAM): $(SOURCES.cc) $(TARGETS.cc) $(TARGETS.h) $(OBJECTS)\n");
	gio_puts("\t$(LINK.cc) -o $(PROGRAM) $(OBJECTS) $(LDLIBS)\n");
	gio_putc('\n');

	gio_puts("# Targets to be used by a certain CASE tool.\n");
	gio_putc('\n');
	gio_puts("stest: $(SOURCES.cc) $(TARGETS.cc) $(TARGETS.h) $(OBJECTS)\n");
	gio_puts("\t#load $(CCFLAGS) $(CPPFLAGS) $(LDFLAGS) $(TARGET_ARCH) \\\n");
	gio_puts("\t\t-o $(PROGRAM) $(SOURCES.cc) $(TARGETS.cc) $(LDLIBS)\n");
	gio_puts("\t#link\n");
	gio_putc('\n');

/* *** Original GXV Code ***
	gio_puts("%_ui.cc + %_ui.h + %_stubs.cc + %.info: %.G\n");
*/
	gio_puts("%_ui.cc + %_stubs.cc + %.info: %.G\n");
	gio_puts("\t$(UITHOME)/bin/guic $*\n");
	gio_putc('\n');

	gio_puts("clean:\n");
	gio_puts("\t$(RM) $(TARGETS) $(OBJECTS) $(SOURCES.G:%.G=%_stubs.cc) core\n");
}

#ifdef TEST
#endif
