/*H+ menus.c
*
*    SCCS ID: @(#)menus.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Writes a the menu notify handler stubs to the stubs file.
*
*    GLOBAL ROUTINES:
*
*    guic_write_menu_function_stub - Writes the notify handler stubs for
*                                    a menu.
*
*    LOCAL ROUTINES:
*
*    menu_owner() - Determines the owner of the menu.
*    write_menu_function_stub() - Writes the menu's main handler stub.
*    write_menu_item_function_stub() - Writes a menu item handler stub.
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review.
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)menus.c 1.1 91/08/02 SMI"; 
#endif

/* Includes */
#include <stdio.h>
#include <guide.h>
#include <gio.h>
#include "guic.h"
#include "list.h"
#include "xv.h"
#include "menus.h"

/* Local Function Declarations */
GUIC_OBJ       *menu_owner();
void	       	write_menu_function_stub();
void	       	write_menu_item_function_stub();

/*F+ void guic_write_menu_function_stub (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Writes the UIC notify handler stubs corresponding to the specified menu.
*
*    PARAMETERS:
*
*    obj - The menu to write the stubs for.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
void
guic_write_menu_function_stub(obj)
	GUIC_OBJ        *obj;
{
	GUIC_OBJ	       *owner = menu_owner(obj);
	int             i;

	if (obj->type != G_MENU)
		return;

	write_menu_function_stub(obj, owner);

	for (i = 0; obj->menu_item_label[i]; i++)
		write_menu_item_function_stub(obj, i, owner);
}

/*F+ void write_menu_function_stub (GUIC_OBJ *obj, GUIC_OBJ *owner)
*
*    PURPOSE:
*
*    Writes the main UIC notify handler stub corresponding to the
*    specified menu.
*
*    PARAMETERS:
*
*    obj   - The menu to write the stub for.
*    owner - The owner of the menu.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static void
write_menu_function_stub(obj, owner)
	GUIC_OBJ	       *obj;
	GUIC_OBJ	       *owner;
{
	char		type[128];		/* instance typedef to use */

	if (!obj->menu_handler || guic_stub_written(obj->menu_handler))
		return;

	/*
	 * Write function header.
	 */
	gio_printf("/*\n * Menu handler for `%s'.\n", obj->name);
	gio_puts(" */\n");
	gio_printf ("void %s (UIObject *object)\n", obj->menu_handler);
	gio_puts("{\n");

	/*
	 * Write trace statement.
	 */
	gio_puts ("  int choice;\n\n");
	gio_printf ("  fprintf (stderr, \"%s: %s: Item %%d\\n\", object->getChoice (choice));\n",
                    guic_application (),
		    obj->menu_handler);
	gio_putc ('\n');
	gio_puts("  return;\n");
	
	/*
	 * Write function footer.
	 */		
	gio_puts("}\n\n");
}

/*F+ void write_menu_item_function_stub (GUIC_OBJ *obj,
*                                        int       item,
*                                        GUIC_OBJ *owner)
*
*    PURPOSE:
*
*    Writes the UIC notify handler stub corresponding to the
*    specified menu item.
*
*    PARAMETERS:
*
*    obj   - The menu to write the stub for.
*    item  - The menu item to write the stub for.
*    owner - The owner of the menu.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static void
write_menu_item_function_stub(obj, item, owner)
	GUIC_OBJ	       *obj;
	int		item;			/* menu item number */
	GUIC_OBJ	       *owner;
{
	char		type[128];		/* instance typedef to use */

	if (!obj->menu_item_handler [item] ||
	    guic_stub_written (obj->menu_item_handler [item]))
	  return;
	/*
	 * Write function header.
	 */
	gio_printf("/*\n * Menu handler for `%s (%s)'.\n */\n",
		 obj->name,
		 obj->menu_item_label [item]);
	gio_printf ("void %s (UIObject *object)\n",
		    obj->menu_item_handler [item]);
	gio_puts("{\n");
	
	/*
	 * Write trace statement.
	 */
	gio_puts ("  int choice;\n\n");
	gio_printf ("  fprintf (stderr, \"%s: %s: Item %%d\\n\", object->getChoice (choice));\n",
                    guic_application (),
		    obj->menu_item_handler [item]);
	gio_putc ('\n');
	gio_puts("  return;\n");

	/*
	 * Write function footer.
	 */
	gio_puts("}\n\n");
}

/*F+ GUIC_OBJ *menu_owner (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Returns the owner of the specified menu.
*
*    PARAMETERS:
*
*    obj   - The menu to return the owner of.
*    item  - The menu item to write the stub for.
*    owner - The owner of the menu.
*
*    RETURN VALUES:
*
*    Returns a pointer to the owner object.  Returns NULL if the owner
*    cannot be determined.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
GUIC_OBJ	       *
menu_owner(menu)
	GUIC_OBJ	       *menu;
{
	GUIC_OBJ	       *owner;

	for (owner = guic_first(); owner; owner = guic_next(owner))
		if (owner->menu_name &&
		    strcmp(owner->menu_name, menu->name) == 0)
			break;
	return owner;
}

#ifdef TEST
#endif
