/*H+ notify.c
*
*    SCCS ID: @(#)notify.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Writes a notify handler stub to the stubs file.
*
*    GLOBAL ROUTINES:
*
*    guic_write_notify_function_stub - Writes a notify handler stub.
*
*    LOCAL ROUTINES:
*
*    value_type - Returns the type of value returned by the "getValue" call
*                 in the stub.
*    value_type_format - Returns the printf format specifier of the value.
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review.
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)notify.c 1.1 91/08/02 SMI"; 
#endif

/* Includes */
#include <stdio.h>
#include <guide.h>
#include <gio.h>
#include "guic.h"
#include "list.h"
#include "notify.h"
#include "xv.h"

/* Local Function Declarations */
char           *value_type();
char           *value_type_format();

/*F+ void guic_write_notify_function_stub (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Writes the UIC notify handler stub corresponding to the supplied object.
*
*    PARAMETERS:
*
*    obj - The object to write the stub for.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
void
guic_write_notify_function_stub(obj)
	GUIC_OBJ        *obj;
{
	char           *type;		/* value_type() return */
	char           *type_format;	/* value_type_format() return */

	if (!obj->notify_handler || guic_stub_written(obj->notify_handler))
		return;
	
	/*
	 * Write a comment.
	 */
	gio_printf(
		 "/*\n * Notify handler function for `%s'.\n */\n",
		 obj->name);
	
	/*
	 * Write the function spec.
	 */
	gio_printf ("void %s (UIObject *object)\n", obj->notify_handler);
	gio_puts("{\n");
	
	/*
	 * Write local variables.
	 */
	if (obj->type == G_SETTING) {
	  gio_puts ("  int value;\n");
	  gio_putc ('\n');
	  gio_puts ("  value = object->getChoice (value);\n");
	}
	else if (type = value_type (obj)) {
	  gio_printf ("  %svalue;\n", type);
	  gio_putc ('\n');
	  gio_puts ("  value = object->getValue (value);\n");
	}
	else if (value_type_format (obj)) {
	  gio_puts ("  int value;\n");
	  gio_putc ('\n');
	  gio_puts ("  value = object->getValue (value);\n");
	}

	/*
	 * Output switch statement if a scrolling list.
	 */
	if (obj->type == G_SCROLLING_LIST) {
	  gio_puts ("  char *value;\n");
	  gio_putc ('\n');
	  gio_puts ("  value = object->getValue (value);\n");
	  gio_putc ('\n');

	  gio_puts("  switch (((ListChoice *) object)->getOperation ()) {\n");
	  gio_puts("  case DESELECT:\n");
	  gio_printf("    fprintf (stderr, \"%s: %s: DESELECT: %%s\\n\", value);\n",
		     guic_application(), obj->notify_handler);
	  gio_puts("    break;\n\n");
	  
	  gio_puts("  case SELECT:\n");
	  gio_printf("    fprintf (stderr, \"%s: %s: SELECT: %%s\\n\", value);\n",
		     guic_application(), obj->notify_handler);
	  gio_puts("    break;\n\n");
	  
	  gio_puts("  case VALIDATE:\n");
	  gio_printf("    fprintf (stderr, \"%s: %s: VALIDATE: %%s\\n\", value);\n",
	  guic_application(), obj->notify_handler);
	  gio_puts("    break;\n\n");
	  
	  gio_puts("  case DELETE:\n");
	  gio_printf("    fprintf (stderr, \"%s: %s: DELETE: %%s\\n\", value);\n",
		     guic_application(), obj->notify_handler);
	  gio_puts("    break;\n\n");
	  
	  gio_puts("  case UNKNOWN:\n");
	  gio_printf("    fprintf (stderr, \"%s: %s: UNKNOWN: %%s\\n\", value);\n",
		     guic_application(), obj->notify_handler);
	  gio_puts("    break;\n");
	  gio_puts("  }\n");

	  gio_puts ("\n  delete value;\n");
	}
	/*
	 * Otherwise, write out a trace message..
	 */
	else if (type_format = value_type_format (obj)) {
	  gio_printf ("  fprintf (stderr, \"%s: %s: value: %s\\n\", value);\n\n",
		      guic_application(),
		      obj->notify_handler,
		      type_format);

	  if (!strcmp (type_format, "%s"))
	    gio_puts ("  delete value;\n");
	}
	else 
	  gio_printf("  fputs (\"%s: %s\\n\", stderr);\n\n",
		     guic_application(),
		     obj->notify_handler);
	
	gio_puts ("  return;\n");
	gio_puts("}\n\n");
}


/*
 * Utility functions.
 */

/*F+ char *value_type (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Return a string representing the C++ data type of an object's value.
*
*    PARAMETERS:
*
*    obj - The object.
*
*    RETURN VALUES:
*
*    Return a string representing the C++ data type of an object's value.
*    Returns NULL if unknown or if code to set value variable should not be
*    output..
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static char    *
value_type(obj)
	GUIC_OBJ        *obj;
{
	switch (obj->type) {
	case G_TEXT_FIELD:
		return (obj->text_type == G_NUMERIC) ? "int " : "char *";

	default:
		return NULL;
	}
}

/*F+ char *value_type (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Return a string representing the printf format string for an object's
*    value.
*
*    PARAMETERS:
*
*    obj - The object.
*
*    RETURN VALUES:
*
*    Returns a string representing the printf format string for an object's
*    value.  Returns NULL if unknown.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static char    *
value_type_format(obj)
	GUIC_OBJ        *obj;
{
	switch (obj->type) {
	case G_SETTING:
		return "%d";

	case G_SLIDER:
		return "%d";

	case G_TEXT_FIELD:
		return (obj->text_type == G_NUMERIC) ? "%d" : "%s";

	default:
		return NULL;
	}
}

#ifdef TEST
#endif
