/*H+ object.c
*
*    SCCS ID: @(#)object.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Object instantiation, initialization and association functions.
*
*    GLOBAL FUNCTIONS:
*
*    void guic_write_object_instantiation () - Instantiates an object.
*    void guic_write_object_initialization () - Create member function calls.
*    void guic_write_object_association () - Associates object with its parent.
*
*    AUTHOR: Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       09/23/90   Original creation.
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)object.c 1.1 91/08/02 SMI"; 
#endif

/* Includes */
#include <stdio.h>
#include <sys/param.h>
#include <xview/icon_load.h>
#include <guide.h>
#include <gio.h>
#include "guic.h"
#include "done.h"
#include "list.h"
#include "object.h"
#include "repaint.h"
#include "xv.h"
#include "strlib.h"

/* Macro Definitions */
#define NO_TYPE -1

/* Typedefs */

/* Define the object types array entry */
typedef struct object_type_entry {
  G_TYPES  guide_type;     /* The GUIDE type */
  int      guide_sub_type; /* The GUIDE sub-type */
  char    *c_type;         /* The C type */
} ObjectTypeEntry;

/* Global Variables */

/* Define the object types array */
static ObjectTypeEntry object_types [] =
{ G_BASE_WINDOW, NO_TYPE, "BaseWindow",
  G_BUTTON, NO_TYPE, "Button",
  G_CANVAS_PANE, NO_TYPE, "ComponentDisplay",
  G_CONTROL_AREA, NO_TYPE, "ComponentDisplay",
  G_SETTING, G_CHECK, "CheckBoxChoice",
  G_SCROLLING_LIST, NO_TYPE, "ListChoice",
  G_MENU, G_EXCLUSIVE_MENU, "ChoiceMenu",
  G_MENU, G_NONEXCLUSIVE_MENU, "ChoiceMenu",
  G_MENU, G_COMMAND_MENU, "CommandMenu",
  G_GAUGE, NO_TYPE, "Gauge",
  G_TEXT_FIELD, G_ALPHANUMERIC, "AlphanumericInput",
  G_TEXT_FIELD, G_NUMERIC, "NumericInput",
  G_POPUP_WINDOW, NO_TYPE, "PopUpWindow",
  G_SLIDER, NO_TYPE, "Slider",
  G_SETTING, G_STACK, "StackChoice",
  G_MESSAGE, NO_TYPE, "TextItem",
  G_SETTING, G_EXCLUSIVE, "ToggleChoice",
  G_SETTING, G_NONEXCLUSIVE, "ToggleChoice",
  G_TERM_PANE, NO_TYPE, "TermDisplay",
  G_TEXT_PANE, NO_TYPE, "TextDisplay",
  NO_TYPE, NO_TYPE, (char *) NULL };

/* Local Functions */
static void  write_image_object_code ();
static char *server_image ();

/*F+ void guic_write_object_instantiation (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Writes the instantiation for a single object.
*
*    PARAMETERS:
*
*    obj - The object to write the instantiation for.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*    MGS       09/23/90   Original creation.
*
*F-*/
void guic_write_object_instantiation (obj)
GUIC_OBJ	*obj;
{
  int i;

  if (guic_medium_names () || guic_long_names ())
    if (obj->owner) {
      obj->name = str_catadd (obj->name, "_");

      if (guic_medium_names ())
	obj->name = str_catadd (obj->name, obj->owner->original_name);
      else if (guic_long_names ())
	obj->name = str_catadd (obj->name, obj->owner->name);
    }

  for (i=0; object_types [i].guide_type != NO_TYPE; i++) {
    if (object_types [i].guide_type == obj->type) {
      /* Menus have sub-types... */
      if (obj->type == G_MENU) {
	if (obj->menu_type == object_types [i].guide_sub_type)
	  break;
      }
      /* Settings have sub-types... */
      else if (obj->type == G_SETTING) {
	if (obj->setting_type == object_types [i].guide_sub_type)
	  break;
      }
      else if (obj->type == G_TEXT_FIELD) {
	if (obj->text_type == object_types [i].guide_sub_type)
	  break;
      }
      else
	break;
    }
  }

  gio_printf ("  %s %s", object_types [i].c_type, obj->name);
  if (obj->type == G_CONTROL_AREA)
    gio_puts (" (TRUE);\n");
  else
    gio_puts (";\n");

  /* For ToggleChoice set multiple versus single choice */
  if (obj->type == G_SETTING) {
    if (obj->setting_type == G_EXCLUSIVE)
      gio_printf ("  %s.setMultipleChoice (FALSE);\n", obj->name);
    else if (obj->setting_type == G_NONEXCLUSIVE)
      gio_printf ("  %s.setMultipleChoice (TRUE);\n", obj->name);
  }

  /* If G_TEXT_FIELD, set underlining on */
  if (obj->type == G_TEXT_FIELD)
    gio_printf ("  %s.setUnderline (TRUE);\n", obj->name);
}

/*F+ void guic_write_object_initialization (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Writes the initialization code for a single object.
*
*    PARAMETERS:
*
*    obj - The object to write the initialization code for.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*    MGS       09/23/90   Original creation.
*
*F-*/
void guic_write_object_initialization (obj)
GUIC_OBJ *obj;
{
  G_ATTRS        *attrp;
  int             i;
  GUIC_OBJ        *menu_obj;
  G_EVENT_TYPES  *p;
  int             x_loc = -1;
  int             y_loc = -1;

  for (attrp = G_object_attrs(obj->type); *attrp != -1; attrp++) {
    switch (*attrp) {
    case G_BACKGROUND_COLOR:
      if (!obj->background_color)
	break;
      gio_printf ("  %s.setBackgroundColor (\"%s\");\n",
		  obj->name,
		  obj->background_color);
      break;
      
    case G_BUTTON_TYPE:
      if (obj->button_type == G_ABBREVIATED)
	gio_printf ("  %s.setAbbreviated (TRUE);\n", obj->name);
      break;

    case G_CHOICES:
      if (!obj->choices)
	break;
      
      for (i=0; obj->choices [i]; i++) 
	if (obj->choice_label_type [i] == G_GLYPH) {
	  write_image_object_code (server_image (obj, i),
				   obj->choices [i]);
	  gio_printf("  (void) %s.addChoice (%s);\n",
		     obj->name,
		     server_image (obj, i));
	}
	else
	  gio_printf("  (void) %s.addChoice (\"%s\");\n",
		     obj->name,
		     obj->choices [i]);
      break;

    case G_CHOICE_COLORS:
      if (!obj->choice_color)
	break;
      for (i=0; obj->choice_color [i]; i++)
	if (strlen (obj->choice_color [i]))
	  gio_printf ("  %s.setChoiceColor (%d, \"%s\");\n",
		      obj->name,
		      i+1,
		      obj->choice_color [i]);
      break;

    case G_COLUMNS:
      if (obj->columns < 1)
	break;

      switch (obj->type) {
      case G_MENU:
      case G_SETTING:
	gio_printf ("  %s.setDisplayedColumns (%d);\n",
		    obj->name,
		    obj->columns);
	break;
	
      default:
	break;
      }
      break;
      
    case G_CONSTANT_WIDTH:
      if (!obj->constant_width)
	break;
      
      gio_printf ("  %s.setLabelWidth (%d);\n",
		  obj->name,
		  obj->width);
      break;
      
    case G_DONE_HANDLER:
      if (!obj->done_handler)
	break;
      
      gio_printf ("  %s.setDoneHandler (%s);\n", obj->name, obj->done_handler);
      break;
      
    case G_DRAWING_MODEL:
      if (obj->type == G_CANVAS_PANE)
        if (obj->drawing_model == G_XWINDOWS)
	  gio_printf("  %s.setDrawingModel (X11);\n", obj->name);
        else if (obj->drawing_model == G_POSTSCRIPT)
	  gio_printf("  %s.setDrawingModel (POSTSCRIPT);\n", obj->name);
      break;
      
    case G_EVENTS:
      if (obj->events)
	for (p = obj->events; *p != -1; p++)
	  gio_printf ("  %s.consumeEvent (%s);\n",
		      obj->name,
		      guic_xv_event (*p));

      if (!obj->event_handler)
	break;
      gio_printf ("  %s.setEventHandler (%s);\n",
		  obj->name,
		  obj->event_handler);
      break;
      
    case G_FOREGROUND_COLOR:
      if (!obj->foreground_color)
	break;
      gio_printf ("  %s.setForegroundColor (\"%s\");\n",
		  obj->name,
		  obj->foreground_color);
      break;
      
    case G_HEIGHT:
      if (guic_ispane(obj)) {
	if (obj->y + obj->height < obj->owner->height)
	  gio_printf ("  %s.setHeight (%d);\n", obj->name, obj->height);
	else
	  gio_printf ("  %s.setExtendToEdge (HEIGHT);\n", obj->name);
      }
      else if (obj->vscrollbar) {
	gio_printf ("  %s.setHeight (%d);\n",
		    obj->name,
		    obj->scrollable_height);
      }
      else {
	gio_printf ("  %s.setHeight (%d);\n",
		    obj->name,
		    obj->height);
      }
      break;
      
    case G_HELP:
      if (!obj->help)
	break;
      gio_printf("  %s.setHelp (\"%s:%s\");\n",
		 obj->name, guic_application(), obj->name);
      break;
      
    case G_HSCROLL:
      if (obj->hscrollbar) {
	gio_printf ("  Scroller %s_hscroller (HORIZONTAL);\n", obj->name);
	gio_printf ("  %s.addComponent (%s_hscroller);\n",
		    obj->name,
		    obj->name);
      }
      break;

    case G_ICON:
      if (!obj->icon)
	break;
      gio_printf ("  %s.setIcon (\"%s\");\n", obj->name, obj->icon);
      break;
      
    case G_ICON_MASK:
      if (!obj->icon_mask)
	break;
      gio_printf ("  %s.setIconMask (\"%s\");\n", obj->name, obj->icon_mask);
      break;
      
    case G_LABEL:
      if (!obj->label)
	break;
      if (guic_iscontrol(obj)) {
	if (obj->label_type == G_GLYPH) {
	  write_image_object_code (server_image (obj, 0), obj->label);
	  gio_printf ("  %s.setLabel (%s);\n",
		      obj->name,
		      server_image (obj, 0));
	}
	else
	  gio_printf ("  %s.setLabel (\"%s\");\n", obj->name, obj->label);
      }
      else {
	gio_printf ("  %s.setLabel (\"%s\");\n", obj->name, obj->label);
      }
      break;
      
    case G_LABEL_BOLD:
      gio_printf ("  %s.setLabelBold (%s);\n",
		  obj->name,
		  guic_boolean_string (obj->label_bold));
      break;
      
    case G_LAYOUT_TYPE:
      gio_printf ("  %s.setLabelPosition (%s);\n",
		  obj->name,
		  obj->layout_type == G_VERTICAL ?
		                      "ABOVE" : "LEFT");
      break;
      
    case G_MAPPED:
      if (obj->type == G_BASE_WINDOW)
	gio_printf ("  %s.setClosed (%s);\n",
		    obj->name,
		    guic_boolean_string (!obj->mapped));
      else
	gio_printf ("  %s.show (%s);\n",
		    obj->name,
		    guic_boolean_string (obj->mapped));
      break;
      
    case G_MAX_VALUE:
      if (obj->type == G_TEXT_FIELD)
	if (obj->text_type == G_ALPHANUMERIC)
	  break;
      gio_printf ("  %s.setMaxValue (%d);\n", obj->name, obj->max_value);
      break;
      
    case G_MENU_HANDLER:
      if (!obj->menu_handler)
	break;
      gio_printf ("  %s.setNotifyHandler (%s);\n",
		  obj->name,
		  obj->menu_handler);
      break;
      
    case G_MENU_NAME:
      if (!(menu_obj = guic_object (obj->menu_name)) ||
	  (menu_obj->type != G_MENU))
	break;

      gio_printf ("  %s.setMenu (%s);\n",
		  obj->name,
		  obj->menu_name);
      break;
      
    case G_MENU_ITEM_COLORS:
      if (!obj->menu_item_color)
	break;
      for (i=0; obj->menu_item_color [i]; i++)
	if (strlen (obj->menu_item_color [i]))
	  gio_printf ("  %s.setItemColor (%d, \"%s\");\n",
		      obj->name,
		      i+1,
		      obj->menu_item_color [i]);
      break;

    case G_MENU_ITEM_DEFAULTS:
      for (i=0; obj->menu_item_label[i]; i++)
	if (obj->menu_item_default [i]) {
	  gio_printf("  %s.setDefaultItem (%d);\n", obj->name, i+1);
	  break;
	}
      break;
      
    case G_MENU_ITEM_LABELS:
      if (obj->menu_type == G_NONEXCLUSIVE_MENU)
	gio_printf ("  %s.setMultipleChoice (TRUE);\n", obj->name);

      for (i=0; obj->menu_item_label[i]; i++) {
	if (obj->menu_item_label_type[i] == G_GLYPH) {
	  write_image_object_code (server_image (obj, i),
				   obj->menu_item_label [i]);
	  gio_printf("  (void) %s.addItem (%s);\n",
		     obj->name,
		     server_image (obj, i));
	}
	else
	  gio_printf("  (void) %s.addItem (\"%s\");\n",
		     obj->name,
		     obj->menu_item_label [i]);
	
	if (obj->menu_item_handler [i])
	  gio_printf ("  %s.setItemHandler (%d, %s);\n",
		      obj->name,
		      i+1,
		      obj->menu_item_handler [i]);
      
	if (guic_object (obj->menu_item_menu [i]))
	  gio_printf("  %s.setItemMenu (%d, %s);\n",
		     obj->name,
		     i+1,
		     obj->menu_item_menu [i]);
      }
      break;

    case G_MENU_TITLE:
      if (obj->menu_title)
	gio_printf ("  %s.setLabel (\"%s\");\n", obj->name, obj->menu_title);
      break;

    case G_MIN_VALUE:
      if (obj->type == G_TEXT_FIELD)
	if (obj->text_type == G_ALPHANUMERIC)
	  break;
      gio_printf ("  %s.setMinValue (%d);\n", obj->name, obj->min_value);
      break;
      
    case G_MULTIPLE_SELECTIONS:
      gio_printf ("  %s.setMultipleChoice (%s);\n",
		  obj->name,
		  guic_boolean_string (obj->multiple_selections));
      break;
      
    case G_NOTIFY_HANDLER:
      if (!obj->notify_handler)
	break;
      
      gio_printf ("  %s.setNotifyHandler (%s);\n",
		  obj->name,
		  obj->notify_handler);
      break;
      
    case G_ORIENTATION:
      gio_printf ("  %s.setOrientation (%s);\n",
		  obj->name,
		  obj->orientation == G_VERTICAL ? "VERTICAL" : "HORIZONTAL");
      break;
      
    case G_PINNABLE:
      if (!obj->pinnable)
	break;
      switch (obj->type) {
      case G_MENU:
	gio_printf ("  %s.setPinnable (TRUE);\n", obj->name);
	break;
	
      default:
	break;
      }
      break;
      
    case G_PINNED:
      gio_printf ("  %s.setPinned (%s);\n",
		  obj->name,
		  guic_boolean_string (obj->pinned));
      break;
      
    case G_READ_ONLY:
      gio_printf ("  %s.setReadOnly (%s);\n",
		  obj->name,
		  guic_boolean_string (obj->read_only));
      break;
      
    case G_REPAINT_PROC:
      if (!obj->repaint_proc)
	break;
      gio_printf ("  %s.setRepaintHandler (%s);\n",
		  obj->name,
		  obj->repaint_proc);
      break;
      
    case G_RESIZABLE:
      gio_printf ("  %s.setResizable (%s);\n",
		  obj->name,
		  guic_boolean_string (obj->resizable));
      break;
      
    case G_ROWS:
      if (obj->rows < 1)
	break;
      
      switch (obj->type) {
      case G_MENU:
      case G_SCROLLING_LIST:
      case G_SETTING:
	gio_printf ("  %s.setDisplayedRows (%d);\n",
		    obj->name,
		    obj->rows);
	break;
	
      default:
	break;
      }
      break;
      
    case G_SELECTION_REQUIRED:
      if (obj->type == G_SCROLLING_LIST)
        gio_printf ("  %s.setSelectionRequired (%s);\n",
		    obj->name,
		    guic_boolean_string (obj->selection_required));
      break;
      
    case G_SHOW_ENDBOXES:
      gio_printf ("  %s.setDisplayEndBoxes (%s);\n",
		  obj->name,
		  guic_boolean_string (obj->show_endboxes));
      break;
      
    case G_SHOW_BORDER:
      gio_printf ("  %s.setDisplayBorders (%s);\n",
		  obj->name,
		  guic_boolean_string (obj->show_border));
      break;
      
    case G_SHOW_FOOTER:
      gio_printf ("  %s.setDisplayFooter (%s);\n",
		  obj->name,
		  guic_boolean_string (obj->show_footer));
      break;
      
    case G_SHOW_RANGE:
      gio_printf ("  %s.setDisplayRange (%s);\n",
		  obj->name,
		  guic_boolean_string (obj->show_range));
      break;
      
    case G_SHOW_VALUE:
      gio_printf ("  %s.setDisplayValue (%s);\n",
		  obj->name,
		  guic_boolean_string (obj->show_value));
      break;
      
    case G_SLIDER_WIDTH:
      if (obj->type == G_GAUGE)
        gio_printf ("  %s.setGaugeWidth (%d);\n",
		    obj->name,
		    obj->slider_width);
      else
        gio_printf ("  %s.setSliderWidth (%d);\n",
		    obj->name,
		    obj->slider_width);
      break;
      
    case G_STORED_LENGTH:
      if (obj->stored_length < obj->value_length)
	break;
      
      gio_printf ("  %s.setMaxInputLength (%d);\n",
		  obj->name,
		  obj->stored_length);
      break;
      
    case G_TICKS:
      gio_printf ("  %s.setTicks (%d);\n", obj->name, obj->ticks);
      break;
      
    case G_VALUE_LENGTH:
      gio_printf ("  %s.setInputDisplayLength (%d);\n",
		  obj->name,
		  obj->value_length);
      break;
      
    case G_VALUE_X:
      gio_printf ("  %s.setValueX (%d);\n", obj->name, obj->value_x);
      break;
      
    case G_VALUE_Y:
      gio_printf ("  %s.setValueY (%d);\n", obj->name, obj->value_y);
      break;
      
    case G_VSCROLL:
      if (obj->vscrollbar) {
	gio_printf ("  Scroller %s_vscroller (VERTICAL);\n", obj->name);
	gio_printf ("  %s.addComponent (%s_vscroller);\n",
		    obj->name,
		    obj->name);
      }
      break;

    case G_WIDTH:
      if (guic_ispane(obj)) {
	if (obj->x + obj->width < obj->owner->width)
	  gio_printf ("  %s.setWidth (%d);\n", obj->name, obj->width);
	else
	  gio_printf ("  %s.setExtendToEdge (WIDTH);\n", obj->name);
      }
      else if (obj->hscrollbar) {	
	gio_printf ("  %s.setWidth (%d);\n",
		    obj->name,
		    obj->scrollable_width);
      }
      else {
	gio_printf ("  %s.setWidth (%d);\n",
		    obj->name,
		    obj->width);
      }
      break;
      
    case G_X:
      gio_printf ("  %s.setX (%d);\n", obj->name, obj->x);
      break;
      
    case G_Y:
      gio_printf ("  %s.setY (%d);\n", obj->name, obj->y);
      break;
      
    default:
      break;
    }
  }
}

/*F+ void guic_write_object_association (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Writes the associations of the object with its owner.
*
*    PARAMETERS:
*
*    obj - The object to write the association for.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*    MGS       09/23/90   Original creation.
*
*F-*/
void guic_write_object_association (obj)
GUIC_OBJ *obj;
{
  if ((obj->type == G_CANVAS_PANE) ||
      (obj->type == G_CONTROL_AREA) ||
      (obj->type == G_TERM_PANE) ||
      (obj->type == G_TEXT_PANE)) {
    if (obj->owner)
      gio_printf ("  %s.addDisplay (%s);\n", obj->owner->name, obj->name);
  }
  else if ((obj->type != G_POPUP_WINDOW) &&
	   (obj->type != G_BASE_WINDOW)) {
    if (obj->owner)
      gio_printf ("  %s.addComponent (%s);\n", obj->owner->name, obj->name);
  }
}

/*F+ static void write_image_object_code (char *image_name, char *image_file)
*
*    PURPOSE:
*
*    Writes the instantiation for an image object.
*
*    PARAMETERS:
*
*    image_name - The name of the image to create.
*    image_file - The image bits file.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*    MGS       09/23/90   Original creation.
*
*F-*/
static void write_image_object_code (image_name, image_file)
char *image_name;
char *image_file;
{
  FILE                *fp;
  char		       buf[MAXPATHLEN];
  
  fp = fopen (image_file, "r");

  if (!fp) {
    strcpy (buf, "/usr/include/");
    strcat (buf, image_file);
    fp = fopen (buf, "r");
  }

  if (!fp) {
    fprintf (stderr, "%s: %s: Can't open image file\n",
	     guic_program(),
	     image_file);
    exit(1);
  }

  fclose(fp);

  gio_printf ("  Image %s (\"%s\");\n", image_name, image_file);
}

/*F+ static char *server_image (GUIC_OBJ *obj, int item)
*
*    PURPOSE:
*
*    Return the name of a server image object variable for an object.
*
*    PARAMETERS:
*
*    obj  - The object.
*    item - The item number in the object (such as a choice item number).
*
*    RETURN VALUES:
*
*    Returns the name of a server image variable.
*
*    REVISION HISTORY:
*
*    Who       When       Why                    
*    --------- ---------- ------------------------
*    MGS       09/23/90   Original creation.
*
*F-*/
static char *server_image (obj, item)
GUIC_OBJ *obj;
int      item;
{
  static char buf [128];
  
  sprintf (buf, "%s_image_%d", obj->name, item);
  return buf;
}

#ifdef TEST
#endif
