/*H+ stubs.c
*
*    SCCS ID: @(#)stubs.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Create a file containing handler stubs.  These are example notify, event
*    and repaint handlers that will serve as a template for the actual handlers
*    in the application.
*
*    GLOBAL ROUTINES:
*
*    guic_stubs_file - Returns the name of the application handler stubs file.
*    guic_write_stubs_file - Write the application handler stubs file.
*
*    LOCAL ROUTINES:
*
*    has_handler - Determines if the object has a handler specified.
*
*    REVISION HISTORY:
*
*    Who       When       Why                              
*    --------- ---------- ------------------------
*    MGS       05/10/91   Preparation for code review.
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)stubs.c 1.1 91/08/02 SMI"; 
#endif

/* Includes */
#include <sys/param.h>
#include <stdio.h>
#include <guide.h>
#include <gio.h>
#include "guic.h"
#include "done.h"
#include "events.h"
#include "list.h"
#include "menus.h"
#include "notify.h"
#include "repaint.h"
#include "stubs.h"
#include "xv.h"

/* Macro Definitions */
#define NO_TYPE -1

/* Type Definitions */

/*
 * Definition of includes hash table data entry
 */
typedef struct include_entry {
  G_TYPES  type;       /* The GUIDE type */
  int      sub_type;   /* The GUIDE sub-type */
  char    *filename;   /* The name of the include file */
  int      used_index; /* Index of used flag for this include filename */
} IncludeEntry;

/* Global Variables */

/*
 * Define the include entries to be placed in the hash table
 */
static IncludeEntry include_entries [] =
{ G_BASE_WINDOW, NO_TYPE, "BaseWindow.h", 0,
  G_BUTTON, NO_TYPE, "Button.h", 1,
  G_CANVAS_PANE, NO_TYPE, "ComponentDisplay.h", 2,
  G_CONTROL_AREA, NO_TYPE, "ComponentDisplay.h", 3,
  G_SETTING, G_CHECK, "CheckBoxChoice.h", 4,
  G_SCROLLING_LIST, NO_TYPE, "ListChoice.h", 5,
  G_MENU, G_EXCLUSIVE_MENU, "ChoiceMenu.h", 6,
  G_MENU, G_NONEXCLUSIVE_MENU, "ChoiceMenu.h", 6,
  G_MENU, G_COMMAND_MENU, "CommandMenu.h", 7,
  G_GAUGE, NO_TYPE, "Gauge.h", 8,
  G_TEXT_FIELD, G_ALPHANUMERIC, "AlphanumericInput.h", 9,
  G_TEXT_FIELD, G_NUMERIC, "NumericInput.h", 10,
  G_POPUP_WINDOW, NO_TYPE, "PopUpWindow.h", 11,
  G_SLIDER, NO_TYPE, "Slider.h", 12,
  G_SETTING, G_STACK, "StackChoice.h", 13,
  G_MESSAGE, NO_TYPE, "TextItem.h", 14,
  G_SETTING, G_EXCLUSIVE, "ToggleChoice.h", 15,
  G_SETTING, G_NONEXCLUSIVE, "ToggleChoice.h", 15,
  G_TERM_PANE, NO_TYPE, "TermDisplay.h", 16,
  G_TEXT_PANE, NO_TYPE, "TextDisplay.h", 17,
  NO_TYPE, NO_TYPE, (char *) NULL, -1 };

static int used_flag [18];

/* Local Function Declarations */
static int has_handler ();

/* External Function Declarations */
extern int strcmp ();

/*F+ char *guic_stubs_file ()
*
*    PURPOSE:
*
*    Returns the name of the stubs file.
*
*    RETURN VALUES:
*
*    Returns the name of the stubs file.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
char           *
guic_stubs_file()
{
	static char	name[MAXPATHLEN];

	if (!*name) {
		strcpy(name, guic_application());
		strcat(name, "_stubs.cc");
	}
	return name;
}

/*F+ int guic_write_stubs_file ()
*
*    PURPOSE:
*
*    Write the stubs file.
*
*    RETURN VALUES:
*
*    Returns OK if successful.  Returns ERROR if unsuccessful.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
int
guic_write_stubs_file()
{
	char    *errmsg;
	int      i;
	GUIC_OBJ *obj;

	/*
	 * Open the output file.
	 */
	if (errmsg = gio_open_output(guic_stubs_file())) {
		guic_error(errmsg);
		return ERROR;
	}
	if (!guic_silent())
		fprintf(stderr, "%s: writing %s\n",
			guic_program(), guic_stubs_file()); 

	/*
	 * Write file header.
	 */
	gio_puts("/*\n");
	gio_printf(" * %s - Notify and event callback function stubs.\n",
		   guic_stubs_file());
	gio_printf(" * This file was generated by `%s' from `%s'.\n",
		   guic_program(), guic_gil_file());
	gio_puts(" * DO NOT EDIT BY HAND.\n");
	gio_puts(" */\n\n");

	/*
	 * Write includes.
	 */
	for (obj = guic_first(); obj; obj = guic_next(obj)) {
	  for (i=0; include_entries [i].type != NO_TYPE; i++) {
	    if (obj->type == include_entries [i].type) {
	      /* Menus have sub-types... */
	      if (obj->type == G_MENU) {
		if (obj->menu_type == include_entries [i].sub_type)
		  break;
	      }
	      /* Settings have sub-types... */
	      else if (obj->type == G_SETTING) {
		if (obj->setting_type == include_entries [i].sub_type)
		  break;
	      }
	      else
		break;
	    }
	  }

	  /* If this object has a handler and the include file was
	     not already used... */
	  if (has_handler (obj))
	    if (!used_flag [include_entries [i].used_index]) {
	      gio_printf ("#include \"%s\"\n", include_entries [i].filename);
	      used_flag [include_entries [i].used_index] = 1;
	    }
	}
	gio_putc ('\n');

	/*
	 * Write callback functions.
	 */
	for (obj = guic_first(); obj; obj = guic_next(obj)) {
		guic_write_done_function_stub(obj);
		guic_write_event_function_stub(obj);
		guic_write_notify_function_stub(obj);
		guic_write_repaint_function_stub(obj);
		guic_write_menu_function_stub(obj);
	}

	gio_close_output();
	return OK;
}

/*F+ int has_handler (GUIC_OBJ *obj)
*
*    PURPOSE:
*
*    Determines if the specified object has a handler specified for it.
*
*    PARAMETERS:
*
*    obj - The object to check.
*
*    RETURN VALUES:
*
*    Returns 0 if no handler specified.  Returns 1 if a handler is specified.
*
*    REVISION HISTORY:
*
*    Who       When       Why                       
*    --------- ---------- ------------------------
*
*F-*/
static int has_handler (obj)
GUIC_OBJ *obj;
{
  int i;

  if (obj->done_handler ||
      obj->event_handler ||
      obj->menu_handler ||
      obj->notify_handler ||
      obj->repaint_proc)
    return (1);

  if (obj->menu_item_handler)
    for (i=0; obj->menu_item_label [i]; i++)
      if (obj->menu_item_handler [i])
	return (1);

  return (0);
}

#ifdef TEST
#endif
