/**H+strlib.h ***************************************************************
*
*    SCCS ID: @(#)strlib.h 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    Include file for strlib routines.
*
*    AUTHOR: Mark Soloway
*
*    REVISION HISTORY:
*
*                                                          Documentation
*                                                             Updated?
*    Who       When       Why                               Pcode InLine
*    --------- ---------- --------------------------------- ----- ------
*    MGS       01/30/90   Original creation.                  N     Y
*
*H-*/

#ifndef _STRLIB_H
#define _STRLIB_H

/*D+ char *str_dup (char *s)
* char *s (I) String to duplicate
* char *r (R) Duplicate of s
*D-*/                      
extern char *str_dup ();

/*D+ char *str_ndup (char *s, int n)
* char *s (I) String to duplicate
* int   n (I) Number of characters
* char *r (R) Duplicate of first n characters of s
*D-*/
extern char *str_ndup (); 

/*D+ char *str_catdup (char *s1, char *s2)
* char *s1 (I) String to duplicate
* char *s2 (I) String to duplicate and cat to first
* char *r  (R) New string containing s1 + s2
*D-*/
extern char *str_catdup (); 

/*D+ char *str_catadd (char *s1, char *s2)
* char *s1 (I) String to reallocate
* char *s2 (I) String to duplicate and cat to first
* char *r  (R) Reallocated s1 containing s1 + s2
*D-*/
extern char *str_catadd ();

/*D+ char *str_ncopy (char *s1, char *s2)
* char *s1 (I) Destination string
* char *s2 (I) Source string
* int   n  (I) Number of chars to copy
* char *r  (R) Null terminated destination string
*D-*/
extern char *str_ncopy (); 

/*D+ int str_iccmp (char *s1, char *s2)
* char *s1 (I) First compare string
* char *s2 (I) Second compare string
* int   r  (R) Returns -1 if s1 < s2, +1 if s1 > s2, 0 if s1 = s2 
*              with both s1 and s2 converted to uppercase.
*D-*/
extern int str_iccmp ();

/*D+ int str_niccmp (char *s1, char *s2, int n)
* char *s1 (I) First string
* char *s2 (I) Second string
* int   n  (I) Number of chars to compare
* int   r  (R) Returns -1 if s1 < s2, +1 if s1 > s2, 0 if s1 = s2 
*              with both s1 and s2 converted to uppercase.
*D-*/
extern int str_niccmp (); 

/*D+ char *str_tolow (char *s)
* char *s (I) String to convert
* char *r (R) Returns lowercase string
*D-*/
extern char *str_tolow (); 

/*D+ char *str_toup (char *s)
* char *s (I) String to convert
* char *r (R) Returns uppercase string
*D-*/
extern char *str_toup (); 

/*D+ char *str_subs (char *s, char *cs1, char *cs2)
* char *s   (I) String   
* char *cs1 (I) Characters to substitute in s
* char *cs2 (I) Substitution characters 
* char *r   (R) Returns string with one to one character substitutions
*D-*/
extern char *str_subs ();

/*D+ char *str_escapes (char *s)
* char *s (I) String to convert
* char *r (R) Returns string with escape codes converted to ASCII
*D-*/
extern char *str_escapes (); 

/*D+ char *str_strip (char *s, chsr *cs)
* char *s  (I) String
* char *cs (I) Characters to strip
* char *r  (R) Returns string stripped of characters from cs
*D-*/
extern char *str_strip (); 

/*D+ char *str_trim (char *s, char *cs)
* char *s  (I) String to trim
* char *cs (I) Character set of chars to trim from end of string
* char *r  (R) String with charcters trimmed off the end
*D-*/
extern char *str_trim ();

/*D+ char *str_lead (char *s, chsr *cs)
* char *s  (I) String to trim
* char *cs (I) Character set of chars to trim from start of the string
* char *r  (R) String with characters trimmed off the beginning
*D-*/
extern char *str_lead ();

/*D+ char *str_norep (char *s, chsr *cs)
* char *s  (I) String to process
* char *cs (I) Set of chars to remove repeats
* char *r  (R) Returns string with no repeats of characters in cs
*D-*/
extern char *str_norep (); 

/*D+ char *str_chop (char *s, int n)
* char *s (I) String   
* char *n (I) Number of chars to chop
* char *r (R) Returns string with n characters chopped off
*D-*/
extern char *str_chop ();

/*D+ char *str_field (char *s, char *cs, int n)
* char *s  (I) String to parse
* char *cs (I) Characters to define field delimiters
* int   n  (I) Field number to copy
* char *t  (R) Returns token 
*D-*/
extern char *str_field ();

/*D+ char *str_parse (char *s, char *cs)
* char  *s  (I) String to parse
* char  *cs (I) Characters to define field delimiters
* char **a  (R) Returns array of tokens
*D-*/
extern char **str_parse (); 

/*D+ char *str_array (int n, int s)
* int    n (I) Number of elements in array
* int    s (I) Size (bytes) of each element
* char **a (R) Returns array of size n with elements of size s
*D-*/
extern char **str_array (); 

/*D+ char *str_arrayadd (char **a, char *s)
* char **a  (I) Null terminated string array to add to
* char  *s  (I) String to add
* char **r  (R) Returns array with string added
*D-*/
extern char **str_arrayadd ();

/*D+ char *str_arraydup (char **a)
* char **a (I) Null terminated string array to duplicate
* char **r (R) Returns duplicate of array
*D-*/
extern char  **str_arraydup ();

/*D+ char *str_arraylen (char **a)
* char **a (I) Null terminated string array 
* int    l (R) Returns length of array
*D-*/
extern int str_arraylen ();

/*D+ char *str_arrayfind (char **a, char *s, int (*cf) ())
* char **a      (I) Null terminated string array to search
* char  *s      (I) String to search for
* int  (*cf) () (I) Compare function
* int   index   (R) Returns the index of the array where the string was found
*D-*/
extern int str_arrayfind ();

/*D+ char *str_arrayrem (char **a, int index)
* char **a     (I) Null terminated string array to remove element from
* int    index (I) Index of element to remove
* char **r     (R) Returns array with element removed
*D-*/
extern char **str_arrayrem ();

/*D+ char *str_arrayfree (char **a)
* char **a (I) Null terminated string array to free
* char **r (R) Returns NULL
*D-*/
extern char **str_arrayfree ();

#endif
