/**H+strlib.c ***************************************************************
*
*    SCCS ID: @(#)strlib.c 1.1 91/08/02 SMI
*
*    Copyright (c) 1990,1991 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE:
*
*    General purpose string manipulation routines.
*
*    CONTENTS:
*
*              #include "strlib.h"
*
*              char  *str_dup (char *s)
*              char  *str_ndup (char *s, int n)
*              char  *str_catdup (char *s1, char *s2)
*              char  *str_catadd (char *s1, char *s2)
*              char  *str_ncopy (char *s1, char *s2, int n)
*
*              int    str_iccmp (char *s1, char *s2)
*              int    str_niccmp (char *s1, char *s2, int n)
*              char  *str_tolow (char *s)
*              char  *str_toup (char *s)
*              char  *str_subs (char *s, char *cs1, char *cs2)
*              char  *str_escapes (char *s)
*
*              char  *str_strip (char *s, char *cs)
*              char  *str_trim (char *s, char *cs)
*              char  *str_lead (char *s, char *cs)
*              char  *str_norep (char *s, char *cs)
*              char  *str_chop (char *s, int n)
*
*              char  *str_field (char *s, char *cs, int n)
*              char **str_parse (char *s, char *cs)
*
*              char **str_array (int n, int s)
*              char **str_arrayadd (char **a, char *s)
*              char **str_arraydup (char **a)
*              int    str_arraylen (char **a) 
*              int    str_arrayfind (char **a, char *s, int (*cf) ())
*              char **str_arrayrem (char **a, int index)
*              char **str_arrayfree (char **a) 
*
*    AUTHOR: Mark Soloway
*
*    REVISION HISTORY:
*
*                                                          Documentation
*                                                             Updated?
*    Who       When       Why                               Pcode InLine
*    --------- ---------- --------------------------------- ----- ------
*    MGS       01/30/90   Original creation.                  N     Y
*
*H-*/
#ifndef lint
static char sccsid [] = "@(#)strlib.c 1.1 91/08/02 SMI";
#endif

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "memlib.h"
#include "strlib.h"


/**F+str_dup *****************************************************************
*
*    PURPOSE:
*
*    Duplicates the input string and returns a new character pointer.
*    Returns NULL on error.
*
*    NOTES:
*
*    Allocates memory for new string.
*
*F-*/
char *str_dup (s)
char *s;
{
  return ((s ? strcpy (MALLOC (strlen (s) + 1), s) : NULL));
}


/**F+str_ndup *****************************************************************
*
*    PURPOSE:
*
*    Duplicates the first n characters of input string and returns a
*    new character pointer.  Returns NULL on error.
*
*    NOTES:
*
*    Allocates memory for new string.
*
*F-*/
char *str_ndup (s, n)
char *s;    
int   n;
{         
  return ((s ? str_ncopy (MALLOC (n+1), s, n) : NULL));
}


/**F+str_catdup *****************************************************************
*
*    PURPOSE:
*
*    Concatenates two strings and returns the pointer to the new string.
*    Returns NULL on error.
*
*    NOTES:
*
*    Allocates memory for new string.
*
*F-*/
char *str_catdup (s1, s2)
char *s1;
char *s2;
{
  char *s;

  if (!s1 && !s2) return (NULL);
  if (!s1) return (str_dup (s2));
  if (!s2) return (str_dup (s1));

  if (!(s = MALLOC (strlen (s1) + strlen (s2) + 1)))
    return (NULL);

  strcpy (s, s1);
  return (strcat (s, s2));
}


/**F+str_catadd *****************************************************************
*
*    PURPOSE:
*
*    Concatenates two strings and returns the pointer to the first string.
*    Returns NULL on error.
*
*    NOTES:
*
*    Reallocates memory of first string.
*
*F-*/
char *str_catadd (s1, s2)
char *s1;
char *s2;
{
  char *s = s1;

  if (!s1 && !s2) return (NULL);       
  if (!s2) return (s1);      
  if (!s1) *(s = MALLOC (1)) = '\0';

  if (!(s = REALLOC (s, strlen (s) + strlen (s2) + 1)))
    return (NULL);

  return (strcat (s, s2));
}


/**F+str_ncopy *****************************************************************
*
*    PURPOSE:
*
*    Copies the first n characters of string s2 into string s1 and
*    NULL terminates s1.
*
*    NOTES:
*
*    s1 must be large enough to hold n+1 bytes.
*
*F-*/
char *str_ncopy (s1, s2, n)
char          *s1;
register char *s2;
int            n;
{
  register int   i;
  register char *s = s1;      

  if (s1 && s2) {
    for (i=0; (i < n) && (*s++ = *s2++); i++) ;
    if (i == n) *s = '\0';
  }
                
  return (s1);
}                     
                           

/**F+str_iccmp *****************************************************************
*
*    PURPOSE:
*
*    Compare two strings after converting to upper case.  If s1 > s2 return +1.
*    if s1 < s2 return -1.  If s1 = s2 return 0.
*
*F-*/
int str_iccmp (s1, s2)
register char *s1;
register char *s2;
{
  static   int  upper_init = 0;
  static   char upper [255];
  register int  i;

  if (!s1 || !s2) {
    if (!s1 && !s2) 
      return (0);

    return (s1 ? 1 : -1);
  }

  if (!upper_init) 
    for (i=0, upper_init=1; i < 255; i++)
      upper [i] = ((islower (i)) ? toupper (i) : i);

  for (; upper [*s1] == upper [*s2]; s1++, s2++)
    if (upper [*s1] == '\0')
      return(0);

  return ((upper [*s1] > upper [*s2]) ? 1 : -1);
}


/**F+str_niccmp *****************************************************************
*
*    PURPOSE:
*
*    Compare up to the first n characters of two strings after converting
*    to upper case.  If s1 > s2 return +1.  if s1 < s2 return -1.
*    If s1 = s2 return 0.
*
*F-*/
int str_niccmp (s1, s2, n)
register char *s1;
register char *s2;
int            n;
{
  static   int  upper_init = 0;
  static   char upper [255];
  register int  count;
  register int  i;

  if (!s1 || !s2) {
    if (!s1 && !s2) 
      return (0);

    return (s1 ? 1 : -1);
  }

  if (!upper_init) 
    for (i=0, upper_init=1; i < 255; i++)
      upper [i] = ((islower (i)) ? toupper (i) : i);

  for (count=0;
       (upper [*s1] == upper [*s2]) && (count < n);
       s1++, s2++, count++)
    if (*s1 == '\0')
      return(0);

  return ((count == n) ? 0 : ((upper [*s1] > upper [*s2]) ? 1 : -1));
}


/**F+str_tolow *****************************************************************
*
*    PURPOSE:
*
*    Convert string to lower case.
*
*F-*/
char *str_tolow (s)
char *s;
{
  register char *p = s;

  if (!s) return (s);

  for (; *p; p++)
    if (isupper (*p)) *p = tolower (*p);

  return (s);
}


/**F+str_toup *****************************************************************
*
*    PURPOSE:
*
*    Convert string to upper case.
*
*F-*/
char *str_toup (s)
char *s;
{
  register char *p = s;

  if (!s) return (s);

  for (; *p; p++)
    if (islower (*p)) *p = toupper (*p);

  return (s);
}


/**F+str_subs *****************************************************************
*
*    PURPOSE:
*
*    Create a one character to one character correspondence between
*    cs1 and cs2 with wrap around on cs2 and substitute one for one
*    in string s.
*
*F-*/
char *str_subs (s, cs1, cs2)
char *s;
char *cs1;
char *cs2;
{
  register char *p = s;
  char          *ptr;
  int            cs2_len = strlen (cs2);

  if (s && cs1 && cs2) {
    for (; *p; p++)
      if (ptr = strchr (cs1, *p))
	*p = *(cs2 + ((ptr - cs1) % cs2_len));
  }

  return (s);
}
            

/**F+str_escapes *****************************************************************
*
*    PURPOSE:
*
*    Do the following substitutions in on the input string:
*
*                     \\   -> \
*                     \"   -> "
*                     \e   -> 033
*                     \t   -> 011
*                     \n   -> newline
*                     \0nn -> 0nn
*                     \r   -> cr
*                     \0   -> NULL
*
*     A single \ in front of other characters is ignored.
*
*     NOTES:
*
*     A \0 in a string will terminate the string at that position.
* 
*F-*/
char *str_escapes (s)
char *s;
{
  register char *a = s;
  register char *b = s;
  register int   result;
  register int   i;
  
  while (*a) {
    if (*a == '\\') {
      switch (*(b+1)) {
        case 'e': /* Escape */
          *b++ = '\033';
          a += 2;
          break;
        case 't': /* Tab */
          *b++ = '\t';
          a += 2;
          break;
        case 'n': /* Newline */
          *b++ = '\n';
          a += 2;
          break;
        case 'r': /* Carriage return */
          *b++ = '\r';
          a += 2;
          break;
        case '\\': /* Backslash */
          *b++ = '\\';
          a += 2;
          break;
        case '0': { /* Octal constant (handles NULL as well) */
          a++;
          for (result = i = 0; (i <= 3) && (*a >= '0') && (*a <= '7');) {
            result <<= 3;
            result += (*a++) - '0';
          }

          *b++ = result & 0377;
          break;
        }
        default: /* Backslash not used as an escape... ignore it... */
          a++;
          break;
      }
    }
    else
      *b++ = *a++;
  }

  *b = '\0';
  return (s);
}


/**F+str_strip *****************************************************************
*
*    PURPOSE:
*
*    Remove all occurances of characters in cs from s.
*
*F-*/
char *str_strip (s, cs)
char *s;
char *cs;
{
  register char *a;
  register char *b;

  if (s && cs) {
    a = b = (char *) strpbrk (s, cs);

    if (a) {
      for (; *a; a++)
        if (!strchr (cs, *a)) *b++ = *a;

      *b = '\0';
    }
  }                 

  return (s);
}


/**F+str_trim *****************************************************************
*
*    PURPOSE:
*
*    Trims all occurances of the characters in cs from the end of s
*    until the first character not in cs.
*
*F-*/
char *str_trim (s, cs)
char *s;
char *cs;
{
  register int i;

  if (s && cs) {
    i = strlen (s);
    while (strchr (cs, *(s+i)) && (i > -1)) i--;
    *(s+i+1) = '\0';          
  }

  return (s);
}


/**F+str_lead *****************************************************************
*
*    PURPOSE:
*
*    Remove characters in cs from the beginning of s until the first
*    character not in cs.
*
*F-*/
char *str_lead (s, cs)
char *s;
char *cs;
{
  register char *a;
  register char *b;

  if (s) {
    for (a = b = s; *a && strchr (cs, *a); a++);
    while (*b++ = *a++) ;
  }

  return (s);
}
                                

/**F+str_norep *****************************************************************
*
*    PURPOSE:
*
*    Crunch adjacent repitions of the characters in cs to one character in
*    the string s.
*
*F-*/
char *str_norep (s, cs)
char *s;
char *cs;
{
  register char *a;
  register char *b;
  char          last = '\0';

  if (s && cs) {
    for (a = b = s; a && *a; last = *a++)       
      if (*a != last || !strchr (cs, *a)) *b++ = *a;

    *b = '\0';
  }

  return (s);
}
                                  

/**F+str_chop *****************************************************************
*
*    PURPOSE:
*
*    Chop off the first n characters of the string s.
*
*F-*/
char *str_chop (s, n)
char *s;
int   n;
{
  register char *p = s;

  if (!s || strlen (s) < n) return (NULL);

  while (*p = *(p+n)) p++;
  return (s);
}


/**F+str_field *****************************************************************
*
*    PURPOSE:
*
*    Return a duplicate of field number n in string s as delimited by
*    the characters in cs.
*
*    NOTES:
*
*    Allocates new string.  Runs strtok on input string.
*
*F-*/
char *str_field (s, cs, n)
char *s;
char *cs;
int   n;                 
{
  char         *t;
  register int  i;

  if (!s || !cs || (n < 1)) return (NULL);
  if ((t = strtok (s, cs)) == NULL) return (NULL);

  if (n == 1) return (str_dup (t));

  for (i=1; i <= n; i++) {
    if ((t = strtok ((char *) NULL, cs)) == NULL)
      return (NULL);

    if (i+1 == n) return (str_dup (t));
  }               

  return (NULL);
}


/**F+str_parse *****************************************************************
*
*    PURPOSE:
*
*    Returns NULL terminated string array containing all of the fields
*    in string s as delimited by the characters in cs.
*
*    NOTES:
*
*    Allocates NULL terminated string array.  Runs strtok on input string.
*
*F-*/
char **str_parse (s, cs)
char *s;
char *cs;
{
  char         **a = NULL;
  char          *t;
          
  if (!s || !cs) return (NULL);
  if ((t = strtok (s, cs)) == NULL) return (NULL);

  a = str_arrayadd (a, t);
  while ((t = strtok ((char *) NULL, cs)) != NULL)
    a = str_arrayadd (a, t);

  return (a);
}


/**F+str_array *****************************************************************
*
*    PURPOSE:
*
*    Returns NULL terminated string array containing with n strings
*    of size s.
*
*    NOTES:
*
*    Allocates NULL terminated string array.
*
*F-*/
char **str_array (n, s)
int n;
int s;
{
  register int   i;
  char         **a;

  a = (char **) CALLOC (n+1, sizeof (char *));

  if (s > 0)
    for (i=0; i < n; i++)
      *(a [i] = MALLOC (s + 1)) = '\0';

  return (a);
}


/**F+str_arrayadd *****************************************************************
*
*    PURPOSE:
*
*    Duplicates and adds the string s to the array a.
*
*    NOTES:
*
*    Reallocates array a.
*
*F-*/
char **str_arrayadd (a, s)
char **a;
char  *s;
{
  register int    i = 0;
  register char **p = a;

  if (s) {
    if (a) {
      for (i=0; *p++; i++);
      a = (char **) REALLOC (a, (i + 2) * sizeof (char *));
    }
    else
      a = (char **) MALLOC (2 * sizeof (char *));

    a [i] = str_dup (s);
    a [i+1] = NULL;
  }

  return (a);
}


/**F+str_arraydup **************************************************************
*
*    PURPOSE:
*
*    Duplicates the string array a.
*
*    NOTES:
*
*    Allocates a new NULL terminated string array.
*
*F-*/
char **str_arraydup (a)
char **a;
{
  register int    i, j;
  register char **p = a;
  char          **new = NULL;

  if (a) {
    for (i=0; *p++; i++) ;
    new = (char **) MALLOC ((i + 1) * sizeof(char *));

    for (j=0; j < i; j++) new [j] = str_dup (a [j]);
    new [j] = NULL;
  }

  return (new);
}


/**F+str_arraylen *************************************************************
*
*    PURPOSE:
*
*    Returns the length of array a.
*
*F-*/
int str_arraylen (a)
char **a;
{
  register int    i = 0;
  register char **p = a;

  if (a) for (i=0; *p++; i++) ;
  return (i);
}                             


/**F+str_arrayfind ************************************************************
*
*    PURPOSE:
*
*    Returns the index of search string s in the array a.  The compare
*    function, cf, must be a function that compares two (char *), such
*    such as strcmp or str_iccmp.  A -1 is returned if the search string
*    is not found.
*
*F-*/
int str_arrayfind (a, s, cf)
char **a;
char  *s;
int  (*cf) ();
{
  register char **p = a;
  register int    i;

  if (!p || !s || !cf) return (-1);
        
  for (i=0; *p; p++, i++)
    if (!(*cf) (*p, s)) return (i);

  return(-1);
}                


/**F+str_arrayrem *************************************************************
*
*    PURPOSE:
*
*    Removes the element from array a corresponding to the specified index.
*
*    NOTES:
*
*    Reallocates the shortened array a.
*
*F-*/
char **str_arrayrem (a, index)
char **a;
int    index;
{
  register int    i;
  register int    len = 0;
  register char **p = a;

  if (a && (index >= 0)) {
    for (len=0; *p++; len++) ;

    if (index < len) {
      for (i=index; i < len-1; i++) a [i] = a [i+1];

      a = (char **) REALLOC (a, len * sizeof (char *));
      a [len-1] = NULL;
    }
  }

  return (a);
}


/**F+str_arrayfree ************************************************************
*
*    PURPOSE:
*
*    Frees a NULL terminated string array.
*
*F-*/
char **str_arrayfree (a)
char **a;
{
  register int i;

  if (!a) return;

  for (i=0; a [i]; i++) FREE ((void *) a [i]);
  return ((char **) FREE_NULL ((void *) a));
}


#ifdef TEST

main()
{       
  {
    char *s1 = "str_dup test";
    char *s2;

    printf ("Testing str_dup...\n\n");
    s2 = str_dup (s1);
    printf ("Original string = %s Duplicate string = %s\n\n", s1, s2);
  }

  {
    char *s1 = "1234567890";
    char *s2;
    int   n;

    printf ("Testing str_ndup...\n\n");

    n = 5;
    s2 = str_ndup (s1, n);
    printf("Original string = %s N = %d Duplicate string = %s\n\n", s1, n, s2);

    printf ("Testing str_ncopy...\n\n");

    n = 3;
    s2 = str_ncopy (s2, s1, n);
    printf("Source string = %s N = %d Destination string = %s\n\n", s1, n, s2);
  }

  {
    char *s;
    char *s1 = "12345";
    char *s2 = "67890";

    printf ("Testing str_catdup...\n\n");

    s = str_catdup (s1, s2);
    printf("Original strings = %s %s Cat string = %s\n\n", s1, s2, s);
  }

  {
    char *s;
    char *s1;
    char *s2 = "67890";

    printf ("Testing str_catadd...\n\n");

    s1 = str_dup ("12345");
    s = str_dup (s1);
    s1 = str_catadd (s1, s2);
    printf("Original strings = %s %s Cat string = %s\n\n", s, s2, s1);
    FREE (s);
    FREE (s1);
  }

  {
    char *s1 = "abc";
    char *s2 = "ABC";
    int   r;

    printf ("Testing str_iccmp...\n\n");  

    r = str_iccmp (s1, s2);
    printf("Compare strings = %s %s Result = %d\n\n", s1, s2, r);
  }

  {
    char *s1 = "abc4";
    char *s2 = "ABC45";
    int   n, r;

    printf ("Testing str_niccmp...\n\n");  

    n = 3;
    r = str_niccmp (s1, s2, n);
    printf("Compare strings = %s %s N = %d Result = %d\n\n", s1, s2, n, r);
  }

  {
    char *s1 = "AbCdEfG";
    char *s2;

    printf ("Testing str_tolow...\n\n");

    s2 = str_tolow (str_dup (s1));
    printf ("Original string = %s Converted string = %s\n\n", s1, s2);
    FREE (s2);

    printf ("Testing str_toup...\n\n");

    s2 = str_toup (str_dup (s1));
    printf ("Original string = %s Converted string = %s\n\n", s1, s2);
    FREE (s2);
  }

  {
    char *s1 = "abcdef";
    char *cs1 = "abcdef";
    char *cs2 = "123";
    char *s2;

    printf ("Testing str_subs...\n\n");

    s2 = str_subs (str_dup (s1), cs1, cs2);
    printf ("Original string = %s Subs sets = %s %s Result = %s\n\n",
	    s1, cs1, cs2, s2);
    FREE (s2);
  }

  {
    char *s1 = "12345\\n67890";
    char *s2;

    printf ("Testing str_escapes...\n\n");  

    s2 = str_escapes (str_dup (s1));
    printf ("Original string = %s Result = %s\n\n", s1, s2);
    FREE (s2);
  }

  {
    char *s1 = "1X2Y3Z";
    char *cs = "XYZ";
    char *s2;

    printf ("Testing str_strip...\n\n");  

    s2 = str_strip (str_dup (s1), cs);
    printf ("Original string = %s Strip set = %s Result = %s\n\n", s1, cs, s2);
    FREE (s2);
  }

  {
    char *s1 = "112233456778899";
    char *cs1 = "789";
    char *cs2 = "123";
    char *s2;

    printf ("Testing str_trim...\n\n");   

    s2 = str_trim (str_dup (s1), cs1);
    printf ("Original string = %s Trim set= %s Result = %s\n\n", s1, cs1, s2);
    FREE (s2);

    printf ("Testing str_lead...");  

    s2 = str_lead (str_dup (s1), cs2);
    printf ("Original string = %s Trim set= %s Result = %s\n\n", s1, cs2, s2);
    FREE (s2);
  }

  {
    char *s1 = "11223344556677889900";
    char *cs = "1234567890";
    char *s2;

    printf ("Testing str_norep...\n\n");  

    s2 = str_norep (str_dup (s1), cs);
    printf("Original string = %s Repeat set= %s Result = %s\n\n", s1, cs, s2);
    FREE (s2);
  }

  {
    char *s1 = "1234567890";
    char *s2;
    int   n;

    printf ("Testing str_chop...\n\n");            

    n = 3;
    s2 = str_chop (str_dup (s1), n);
    printf("Original string = %s N = %d Result = %s\n\n", s1, n, s2);
    FREE (s2);
  }

  {
    char *s1 = "1 2 3 4 5";
    char *cs = " ";
    char *s2;
    char *s3;	
    int   n;

    printf ("Testing str_field...\n\n");  

    n = 3;
    s2 = str_field (s3 = str_dup (s1), cs, n);
    printf("Original string = %s Delim Set = %s N = %d Field = %s\n\n",
	   s1, cs, n, s2);
    FREE (s2);
    FREE (s3);
  }

  {
    char  *s1 = "1 2 3 4 5";
    char  *s2;
    char  *cs = " ";
    char **a;
    int    i;

    printf ("Testing str_parse, str_arraylen, str_arrayfree...\n\n");

    a = str_parse (s2 = str_dup (s1), cs);
    printf("Original string = %s Delim Set = %s\n\n", s1, cs);
    FREE (s2);

    for (i=0; i < str_arraylen (a); i++)
      printf ("Token %d: %s\n", i, a [i]);
    printf ("\n");

    str_arrayfree (a);
  }

  {
    char **a;
    int    n, s;

    printf ("Testing str_array...\n\n");

    n = 5;
    s = 5;
    a = str_array (n, s);
    printf ("Number = %d Size = %d Len of array = %d\n\n",
	    n, s, str_arraylen (a));
    str_arrayfree (a);
  }

  {
    char  *s1 = "1 2 3 4 5";
    char  *s2 = "6";
    char  *s3 = "3";
    char  *cs = " ";
    char **a1;
    char **a2;
    int    i;

    printf ("Testing str_arrayadd...\n");
    printf ("    and str_arraydup...\n");
    printf ("    and str_arrayfind...\n");
    printf ("    and str_arrayrem...\n\n");

    a1 = str_parse (s1, cs);
    a2 = str_arraydup (a1);

    printf ("Original array:\n\n");

    for (i=0; i < str_arraylen (a1); i++)
      printf ("Element %d: %s\n", i, a1 [i]);
    printf ("\n");

    printf ("Duplicate array:\n\n");

    for (i=0; i < str_arraylen (a2); i++)
      printf ("Element %d: %s\n", i, a2 [i]);
    printf ("\n");

    printf ("String to add to array: %s\n\n", s2);
    printf ("Array with string added:\n\n");

    a1 = str_arrayadd (a1, s2);

    for (i=0; i < str_arraylen (a1); i++)
      printf ("Element %d: %s\n", i, a1 [i]);
    printf ("\n");

    printf ("Element %d + 1 = %s\n\n", str_arrayfind (a1, s3, strcmp), s3);

    printf ("Same array with element %s removed:\n\n", s3);

    a1 = str_arrayrem (a1, str_arrayfind (a1, s3, strcmp));

    for (i=0; i < str_arraylen (a1); i++)
      printf ("Element %d: %s\n", i, a1 [i]);
    printf ("\n");

    str_arrayfree (a1);
    str_arrayfree (a2);
  }
}

#endif
